{% load static %}

<link href="{% static 'appearance/node_modules/@fortawesome/fontawesome-free/css/all.min.css' %}" media="screen" rel="stylesheet" type="text/css" />
<link href="{% static 'appearance/node_modules/bootswatch/flatly/bootstrap.min.css' %}" media="screen" rel="stylesheet" type="text/css" />
<link href="{% static 'appearance/node_modules/@fancyapps/fancybox/dist/jquery.fancybox.min.css' %}" media="screen" rel="stylesheet" type="text/css" />
<link href="{% static 'appearance/node_modules/select2/dist/css/select2.min.css' %}" media="screen" rel="stylesheet" type="text/css" />
<link href="{% static 'appearance/node_modules/toastr/build/toastr.min.css' %}" media="screen" rel="stylesheet" type="text/css" />
<link href="{% static 'appearance/css/appearance.css' %}" media="screen" rel="stylesheet" type="text/css" />

<style>
    @font-face {
        font-family: 'mayan-edms';
        src: url("{% static 'appearance/fonts/mayan-edms.ttf' %}") format('truetype');
        font-weight: normal;
        font-style: normal;
    }

    [class^="icon-"], [class*=" icon-"] {
        font-family: 'mayan-edms';
        speak: none;
        font-style: normal;
        font-weight: normal;
        font-variant: normal;
        text-transform: none;
        line-height: 1;
        -webkit-font-smoothing: antialiased;
        -moz-osx-font-smoothing: grayscale;
    }

    .icon-mayan-edms-logo:before {
        content: "\e600";
        font-size: 90%;
    }

    .mayan-edms-logo {
        font-family: 'IM Fell English SC', serif;
        font-size: 150%;
        text-shadow: 5px 5px 1px rgba(0, 0, 0, 0.3);
    }

    @media (min-width: 768px) {
        .mayan-edms-logo {
            font-size: 200%;
        }
    }

    @media (min-width: 991px) {
        .mayan-edms-logo {
            font-size: 300%;
        }
    }
</style>
