from mayan.apps.appearance.classes import Icon

icon_user_theme_settings_detail = Icon(
    driver_name='fontawesome', symbol='palette'
)
icon_user_theme_settings_edit = Icon(
    driver_name='fontawesome-dual', primary_symbol='palette',
    secondary_symbol='pencil-alt'
)

icon_menu_actions = Icon(driver_name='fontawesome', symbol='ellipsis-v')
icon_menu_views = Icon(driver_name='fontawesome', symbol='eye')

icon_theme = Icon(driver_name='fontawesome', symbol='palette')
icon_theme_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='palette',
    secondary_symbol='plus'
)
icon_theme_delete = Icon(driver_name='fontawesome', symbol='times')
icon_theme_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')
icon_theme_setup = icon_theme
icon_theme_list = icon_theme
