from mayan.apps.dependencies.classes import PythonDependency

PythonDependency(
    module=__name__, name='gevent', version_string='==23.9.1'
)
PythonDependency(
    module=__name__, name='greenlet', version_string='==3.0.3'
)
PythonDependency(
    module=__name__, name='gunicorn', version_string='==21.2.0'
)
PythonDependency(
    module=__name__, name='sentry-sdk', version_string='==1.40.6'
)
PythonDependency(
    module=__name__, name='whitenoise', version_string='==6.6.0'
)
