TEST_SOURCE_SANE_SCANNER_FILE_CHECKSUM = 'e526d9deec60afeb84e6aa357d7815f4ecff6918e182d2398cb37b22e87c2bd0'

TEST_SOURCE_BACKEND_PATH_SANE_SCANNER = 'mayan.apps.source_sane_scanners.source_backends.SourceBackendSANEScanner'
