{% load navigation_tags %}

{% if not hide_columns %}
    {% for column in source_columns_non_identifier %}
        <td>
            <div class="{{ table_cell_container_classes }} {{ column.html_extra_classes }}">
                {% navigation_source_column_resolve column=column as column_value %}{{ column_value }}
                {# Use explicit 'as column_value ' to force date rendering #}
            </div>
        </td>
    {% endfor %}
{% endif %}
