from mayan.apps.dependencies.classes import PythonDependency

PythonDependency(
    module=__name__, name='boto3', version_string='==1.33.7'
)
PythonDependency(
    module=__name__, name='django-storages', version_string='==1.14.2'
)
PythonDependency(
    module=__name__, name='extract-msg', version_string='==0.48.0'
)
PythonDependency(
    module=__name__, name='pycryptodome', version_string='==3.20.0'
)
