from mayan.apps.appearance.classes import Icon

# Cluster

icon_setting_cluster_configuration_save = Icon(
    driver_name='fontawesome-dual', primary_symbol='sliders-h',
    secondary_symbol='check'
)
icon_setting_cluster_namespace_list = Icon(
    driver_name='fontawesome-dual', primary_symbol='sliders-h',
    secondary_symbol='folder'
)

# Namespace

icon_setting_namespace_detail = Icon(
    driver_name='fontawesome', symbol='sliders-h'
)

# Setting

icon_setting_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')
icon_setting_revert = Icon(driver_name='fontawesome', symbol='undo')
icon_setting_setup = Icon(
    driver_name='fontawesome', symbol='sliders-h'
)
