# -*- coding: utf-8 -*-

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0028_newversionblock')
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='language',
            field=models.CharField(
                default=b'eng', max_length=8, verbose_name='Language',
                blank=True, choices=[
                    [b'aaa', b'Ghotuo'],
                    [b'aab', b'Alumu-Tesu'],
                    [b'aac', b'Ari'],
                    [b'aad', b'Amal'],
                    [b'aae', 'Albanian, Arb\xebresh\xeb'],
                    [b'aaf', b'Aranadan'],
                    [b'aag', b'Ambrak'],
                    [b'aah', b"Arapesh, Abu'"],
                    [b'aai', b'Arifama-Miniafia'],
                    [b'aak', b'Ankave'],
                    [b'aal', b'Afade'],
                    [b'aam', b'Aramanik'],
                    [b'aan', 'Anamb\xe9'],
                    [b'aao', b'Arabic, Algerian Saharan'],
                    [b'aap', 'Ar\xe1ra, Par\xe1'],
                    [b'aaq', b'Abnaki, Eastern'],
                    [b'aar', b'Afar'],
                    [b'aas', 'Aas\xe1x'],
                    [b'aat', b'Albanian, Arvanitika'],
                    [b'aau', b'Abau'],
                    [b'aaw', b'Solong'],
                    [b'aax', b'Mandobo Atas'],
                    [b'aaz', b'Amarasi'],
                    [b'aba', 'Ab\xe9'],
                    [b'abb', b'Bankon'],
                    [b'abc', b'Ayta, Ambala'],
                    [b'abd', b'Manide'],
                    [b'abe', b'Abnaki, Western'],
                    [b'abf', b'Abai Sungai'],
                    [b'abg', b'Abaga'],
                    [b'abh', b'Arabic, Tajiki'],
                    [b'abi', b'Abidji'],
                    [b'abj', b'Aka-Bea'],
                    [b'abk', b'Abkhazian'],
                    [b'abl', b'Lampung Nyo'],
                    [b'abm', b'Abanyom'],
                    [b'abn', b'Abua'],
                    [b'abo', b'Abon'],
                    [b'abp', b'Ayta, Abellen'],
                    [b'abq', b'Abaza'],
                    [b'abr', b'Abron'],
                    [b'abs', b'Malay, Ambonese'],
                    [b'abt', b'Ambulas'],
                    [b'abu', b'Abure'],
                    [b'abv', b'Arabic, Baharna'],
                    [b'abw', b'Pal'],
                    [b'abx', b'Inabaknon'],
                    [b'aby', b'Aneme Wake'],
                    [b'abz', b'Abui'],
                    [b'aca', b'Achagua'],
                    [b'acb', '\xc1nc\xe1'],
                    [b'acd', b'Gikyode'],
                    [b'ace', b'Achinese'],
                    [b'acf', b'Creole French, Saint Lucian'],
                    [b'ach', b'Acoli'],
                    [b'aci', b'Aka-Cari'],
                    [b'ack', b'Aka-Kora'],
                    [b'acl', b'Akar-Bale'],
                    [b'acm', b'Arabic, Mesopotamian'],
                    [b'acn', b'Achang'],
                    [b'acp', b'Acipa, Eastern'],
                    [b'acq', b"Arabic, Ta'izzi-Adeni"],
                    [b'acr', b'Achi'],
                    [b'acs', 'Acro\xe1'],
                    [b'act', b'Achterhoeks'],
                    [b'acu', b'Achuar-Shiwiar'],
                    [b'acv', b'Achumawi'],
                    [b'acw', b'Arabic, Hijazi'],
                    [b'acx', b'Arabic, Omani'],
                    [b'acy', b'Arabic, Cypriot'],
                    [b'acz', b'Acheron'],
                    [b'ada', b'Adangme'],
                    [b'adb', b'Adabe'],
                    [b'add', b'Dzodinka'],
                    [b'ade', b'Adele'],
                    [b'adf', b'Arabic, Dhofari'],
                    [b'adg', b'Andegerebinha'],
                    [b'adh', b'Adhola'],
                    [b'adi', b'Adi'],
                    [b'adj', b'Adioukrou'],
                    [b'adl', b'Galo'],
                    [b'adn', b'Adang'],
                    [b'ado', b'Abu'],
                    [b'adp', b'Adap'],
                    [b'adq', b'Adangbe'],
                    [b'adr', b'Adonara'],
                    [b'ads', b'Adamorobe Sign Language'],
                    [b'adt', b'Adnyamathanha'],
                    [b'adu', b'Aduge'],
                    [b'adw', b'Amundava'],
                    [b'adx', b'Tibetan, Amdo'],
                    [b'ady', b'Adyghe'],
                    [b'adz', b'Adzera'],
                    [b'aea', b'Areba'],
                    [b'aeb', b'Arabic, Tunisian'],
                    [b'aec', b'Arabic, Saidi'],
                    [b'aed', b'Argentine Sign Language'],
                    [b'aee', b'Pashayi, Northeast'],
                    [b'aek', b'Haeke'],
                    [b'ael', b'Ambele'],
                    [b'aem', b'Arem'],
                    [b'aen', b'Armenian Sign Language'],
                    [b'aeq', b'Aer'],
                    [b'aer', b'Arrernte, Eastern'],
                    [b'aes', b'Alsea'],
                    [b'aeu', b'Akeu'],
                    [b'aew', b'Ambakich'],
                    [b'aey', b'Amele'],
                    [b'aez', b'Aeka'],
                    [b'afb', b'Arabic, Gulf'],
                    [b'afd', b'Andai'],
                    [b'afe', b'Putukwam'],
                    [b'afg', b'Afghan Sign Language'],
                    [b'afh', b'Afrihili'],
                    [b'afi', b'Akrukay'],
                    [b'afk', b'Nanubae'],
                    [b'afn', b'Defaka'],
                    [b'afo', b'Eloyi'],
                    [b'afp', b'Tapei'],
                    [b'afr', b'Afrikaans'],
                    [b'afs', b'Creole, Afro-Seminole'],
                    [b'aft', b'Afitti'],
                    [b'afu', b'Awutu'],
                    [b'afz', b'Obokuitai'],
                    [b'aga', b'Aguano'],
                    [b'agb', b'Legbo'],
                    [b'agc', b'Agatu'],
                    [b'agd', b'Agarabi'],
                    [b'age', b'Angal'],
                    [b'agf', b'Arguni'],
                    [b'agg', b'Angor'],
                    [b'agh', b'Ngelima'],
                    [b'agi', b'Agariya'],
                    [b'agj', b'Argobba'],
                    [b'agk', b'Agta, Isarog'],
                    [b'agl', b'Fembe'],
                    [b'agm', b'Angaataha'],
                    [b'agn', b'Agutaynen'],
                    [b'ago', b'Tainae'],
                    [b'agq', b'Aghem'],
                    [b'agr', b'Aguaruna'],
                    [b'ags', b'Esimbi'],
                    [b'agt', b'Agta, Central Cagayan'],
                    [b'agu', b'Aguacateco'],
                    [b'agv', b'Dumagat, Remontado'],
                    [b'agw', b'Kahua'],
                    [b'agx', b'Aghul'],
                    [b'agy', b'Alta, Southern'],
                    [b'agz', b'Agta, Mt. Iriga'],
                    [b'aha', b'Ahanta'],
                    [b'ahb', b'Axamb'],
                    [b'ahg', b'Qimant'],
                    [b'ahh', b'Aghu'],
                    [b'ahi', b'Aizi, Tiagbamrin'],
                    [b'ahk', b'Akha'],
                    [b'ahl', b'Igo'],
                    [b'ahm', b'Aizi, Mobumrin'],
                    [b'ahn', '\xc0h\xe0n'],
                    [b'aho', b'Ahom'],
                    [b'ahp', b'Aizi, Aproumu'],
                    [b'ahr', b'Ahirani'],
                    [b'ahs', b'Ashe'],
                    [b'aht', b'Ahtena'],
                    [b'aia', b'Arosi'],
                    [b'aib', b'Ainu (China)'],
                    [b'aic', b'Ainbai'],
                    [b'aid', b'Alngith'],
                    [b'aie', b'Amara'],
                    [b'aif', b'Agi'],
                    [b'aig', b'Creole English, Antigua and Barbuda'],
                    [b'aih', b'Ai-Cham'],
                    [b'aii', b'Neo-Aramaic, Assyrian'],
                    [b'aij', b'Lishanid Noshan'],
                    [b'aik', b'Ake'],
                    [b'ail', b'Aimele'],
                    [b'aim', b'Aimol'],
                    [b'ain', b'Ainu (Japan)'],
                    [b'aio', b'Aiton'],
                    [b'aip', b'Burumakok'],
                    [b'aiq', b'Aimaq'],
                    [b'air', b'Airoran'],
                    [b'ais', b'Amis, Nataoran'],
                    [b'ait', b'Arikem'],
                    [b'aiw', b'Aari'],
                    [b'aix', b'Aighon'],
                    [b'aiy', b'Ali'],
                    [b'aja', b'Aja (Sudan)'],
                    [b'ajg', b'Aja (Benin)'],
                    [b'aji', 'Aji\xeb'],
                    [b'ajn', b'Andajin'],
                    [b'ajp', b'Arabic, South Levantine'],
                    [b'ajt', b'Arabic, Judeo-Tunisian'],
                    [b'aju', b'Arabic, Judeo-Moroccan'],
                    [b'ajw', b'Ajawa'],
                    [b'ajz', b'Karbi, Amri'],
                    [b'aka', b'Akan'],
                    [b'akb', b'Batak Angkola'],
                    [b'akc', b'Mpur'],
                    [b'akd', b'Ukpet-Ehom'],
                    [b'ake', b'Akawaio'],
                    [b'akf', b'Akpa'],
                    [b'akg', b'Anakalangu'],
                    [b'akh', b'Angal Heneng'],
                    [b'aki', b'Aiome'],
                    [b'akj', b'Aka-Jeru'],
                    [b'akk', b'Akkadian'],
                    [b'akl', b'Aklanon'],
                    [b'akm', b'Aka-Bo'],
                    [b'ako', b'Akurio'],
                    [b'akp', b'Siwu'],
                    [b'akq', b'Ak'],
                    [b'akr', b'Araki'],
                    [b'aks', b'Akaselem'],
                    [b'akt', b'Akolet'],
                    [b'aku', b'Akum'],
                    [b'akv', b'Akhvakh'],
                    [b'akw', b'Akwa'],
                    [b'akx', b'Aka-Kede'],
                    [b'aky', b'Aka-Kol'],
                    [b'akz', b'Alabama'],
                    [b'ala', b'Alago'],
                    [b'alc', b'Qawasqar'],
                    [b'ald', b'Alladian'],
                    [b'ale', b'Aleut'],
                    [b'alf', b'Alege'],
                    [b'alh', b'Alawa'],
                    [b'ali', b'Amaimon'],
                    [b'alj', b'Alangan'],
                    [b'alk', b'Alak'],
                    [b'all', b'Allar'],
                    [b'alm', b'Amblong'],
                    [b'aln', b'Albanian, Gheg'],
                    [b'alo', b'Larike-Wakasihu'],
                    [b'alp', b'Alune'],
                    [b'alq', b'Algonquin'],
                    [b'alr', b'Alutor'],
                    [b'als', b'Albanian, Tosk'],
                    [b'alt', b'Altai, Southern'],
                    [b'alu', b"'Are'are"],
                    [b'alw', 'Alaba-K\u2019abeena'],
                    [b'alx', b'Amol'],
                    [b'aly', b'Alyawarr'],
                    [b'alz', b'Alur'],
                    [b'ama', 'Amanay\xe9'],
                    [b'amb', b'Ambo'],
                    [b'amc', b'Amahuaca'],
                    [b'ame', b"Yanesha'"],
                    [b'amf', b'Hamer-Banna'],
                    [b'amg', b'Amurdak'],
                    [b'amh', b'Amharic'],
                    [b'ami', b'Amis'],
                    [b'amj', b'Amdang'],
                    [b'amk', b'Ambai'],
                    [b'aml', b'War-Jaintia'],
                    [b'amm', b'Ama (Papua New Guinea)'],
                    [b'amn', b'Amanab'],
                    [b'amo', b'Amo'],
                    [b'amp', b'Alamblak'],
                    [b'amq', b'Amahai'],
                    [b'amr', b'Amarakaeri'],
                    [b'ams', b'Amami-Oshima, Southern'],
                    [b'amt', b'Amto'],
                    [b'amu', b'Amuzgo, Guerrero'],
                    [b'amv', b'Ambelau'],
                    [b'amw', b'Neo-Aramaic, Western'],
                    [b'amx', b'Anmatyerre'],
                    [b'amy', b'Ami'],
                    [b'amz', b'Atampaya'],
                    [b'ana', b'Andaqui'],
                    [b'anb', b'Andoa'],
                    [b'anc', b'Ngas'],
                    [b'and', b'Ansus'],
                    [b'ane', 'X\xe2r\xe2c\xf9\xf9'],
                    [b'anf', b'Animere'],
                    [b'ang', b'English, Old (ca. 450-1100)'],
                    [b'anh', b'Nend'],
                    [b'ani', b'Andi'],
                    [b'anj', b'Anor'],
                    [b'ank', b'Goemai'],
                    [b'anl', b'Chin, Anu-Hkongso'],
                    [b'anm', b'Anal'],
                    [b'ann', b'Obolo'],
                    [b'ano', b'Andoque'],
                    [b'anp', b'Angika'],
                    [b'anq', b'Jarawa (India)'],
                    [b'anr', b'Andh'],
                    [b'ans', b'Anserma'],
                    [b'ant', b'Antakarinya'],
                    [b'anu', b'Anuak'],
                    [b'anv', b'Denya'],
                    [b'anw', b'Anaang'],
                    [b'anx', b'Andra-Hus'],
                    [b'any', b'Anyin'],
                    [b'anz', b'Anem'],
                    [b'aoa', b'Angolar'],
                    [b'aob', b'Abom'],
                    [b'aoc', b'Pemon'],
                    [b'aod', b'Andarum'],
                    [b'aoe', b'Angal Enen'],
                    [b'aof', b'Bragat'],
                    [b'aog', b'Angoram'],
                    [b'aoh', b'Arma'],
                    [b'aoi', b'Anindilyakwa'],
                    [b'aoj', b'Mufian'],
                    [b'aok', 'Arh\xf6'],
                    [b'aol', b'Alor'],
                    [b'aom', '\xd6mie'],
                    [b'aon', b'Arapesh, Bumbita'],
                    [b'aor', b'Aore'],
                    [b'aos', b'Taikat'],
                    [b'aot', b"A'tong"],
                    [b'aou', b"A'ou"],
                    [b'aox', b'Atorada'],
                    [b'aoz', b'Uab Meto'],
                    [b'apb', b"Sa'a"],
                    [b'apc', b'Arabic, North Levantine'],
                    [b'apd', b'Arabic, Sudanese'],
                    [b'ape', b'Bukiyip'],
                    [b'apf', b'Agta, Pahanan'],
                    [b'apg', b'Ampanang'],
                    [b'aph', b'Athpariya'],
                    [b'api', 'Apiak\xe1'],
                    [b'apj', b'Apache, Jicarilla'],
                    [b'apk', b'Apache, Kiowa'],
                    [b'apl', b'Apache, Lipan'],
                    [b'apm', b'Apache, Mescalero-Chiricahua'],
                    [b'apn', 'Apinay\xe9'],
                    [b'apo', b'Ambul'],
                    [b'app', b'Apma'],
                    [b'apq', b'A-Pucikwar'],
                    [b'apr', b'Arop-Lokep'],
                    [b'aps', b'Arop-Sissano'],
                    [b'apt', b'Apatani'],
                    [b'apu', 'Apurin\xe3'],
                    [b'apv', b'Alapmunte'],
                    [b'apw', b'Apache, Western'],
                    [b'apx', b'Aputai'],
                    [b'apy', 'Apala\xed'],
                    [b'apz', b'Safeyoka'],
                    [b'aqc', b'Archi'],
                    [b'aqd', b'Dogon, Ampari'],
                    [b'aqg', b'Arigidi'],
                    [b'aqm', b'Atohwaim'],
                    [b'aqn', b'Alta, Northern'],
                    [b'aqp', b'Atakapa'],
                    [b'aqr', 'Arh\xe2'],
                    [b'aqz', b'Akuntsu'],
                    [b'ara', b'Arabic'],
                    [b'arb', b'Arabic, Standard'],
                    [b'arc', b'Aramaic, Official (700-300 BCE)'],
                    [b'ard', b'Arabana'],
                    [b'are', b'Arrarnta, Western'],
                    [b'arg', b'Aragonese'],
                    [b'arh', b'Arhuaco'],
                    [b'ari', b'Arikara'],
                    [b'arj', b'Arapaso'],
                    [b'ark', 'Arikap\xfa'],
                    [b'arl', b'Arabela'],
                    [b'arn', b'Mapudungun'],
                    [b'aro', b'Araona'],
                    [b'arp', b'Arapaho'],
                    [b'arq', b'Arabic, Algerian'],
                    [b'arr', b'Karo (Brazil)'],
                    [b'ars', b'Arabic, Najdi'],
                    [b'aru', 'Aru\xe1 (Amazonas State)'],
                    [b'arv', b'Arbore'],
                    [b'arw', b'Arawak'],
                    [b'arx', 'Aru\xe1 (Rodonia State)'],
                    [b'ary', b'Arabic, Moroccan'],
                    [b'arz', b'Arabic, Egyptian'],
                    [b'asa', b'Asu (Tanzania)'],
                    [b'asb', b'Assiniboine'],
                    [b'asc', b'Asmat, Casuarina Coast'],
                    [b'asd', b'Asas'],
                    [b'ase', b'American Sign Language'],
                    [b'asf', b'Australian Sign Language'],
                    [b'asg', b'Cishingini'],
                    [b'ash', b'Abishira'],
                    [b'asi', b'Buruwai'],
                    [b'asj', b'Sari'],
                    [b'ask', b'Ashkun'],
                    [b'asl', b'Asilulu'],
                    [b'asm', b'Assamese'],
                    [b'asn', 'Asurin\xed, Xing\xfa'],
                    [b'aso', b'Dano'],
                    [b'asp', b'Algerian Sign Language'],
                    [b'asq', b'Austrian Sign Language'],
                    [b'asr', b'Asuri'],
                    [b'ass', b'Ipulo'],
                    [b'ast', b'Asturian'],
                    [b'asu', b'Asurini, Tocantins'],
                    [b'asv', b'Asoa'],
                    [b'asw', b'Australian Aborigines Sign Language'],
                    [b'asx', b'Muratayak'],
                    [b'asy', b'Asmat, Yaosakor'],
                    [b'asz', b'As'],
                    [b'ata', b'Pele-Ata'],
                    [b'atb', b'Zaiwa'],
                    [b'atc', b'Atsahuaca'],
                    [b'atd', b'Manobo, Ata'],
                    [b'ate', b'Atemble'],
                    [b'atg', b'Ivbie North-Okpela-Arhe'],
                    [b'ati', 'Atti\xe9'],
                    [b'atj', b'Atikamekw'],
                    [b'atk', b'Ati'],
                    [b'atl', b'Agta, Mt. Iraya'],
                    [b'atm', b'Ata'],
                    [b'atn', b'Ashtiani'],
                    [b'ato', b'Atong'],
                    [b'atp', b'Atta, Pudtol'],
                    [b'atq', b'Aralle-Tabulahan'],
                    [b'atr', b'Waimiri-Atroari'],
                    [b'ats', b'Gros Ventre'],
                    [b'att', b'Atta, Pamplona'],
                    [b'atu', b'Reel'],
                    [b'atv', b'Altai, Northern'],
                    [b'atw', b'Atsugewi'],
                    [b'atx', b'Arutani'],
                    [b'aty', b'Aneityum'],
                    [b'atz', b'Arta'],
                    [b'aua', b'Asumboa'],
                    [b'aub', b'Alugu'],
                    [b'auc', b'Waorani'],
                    [b'aud', b'Anuta'],
                    [b'aue', b"=/Kx'au//'ein"],
                    [b'aug', b'Aguna'],
                    [b'auh', b'Aushi'],
                    [b'aui', b'Anuki'],
                    [b'auj', b'Awjilah'],
                    [b'auk', b'Heyo'],
                    [b'aul', b'Aulua'],
                    [b'aum', b'Asu (Nigeria)'],
                    [b'aun', b'One, Molmo'],
                    [b'auo', b'Auyokawa'],
                    [b'aup', b'Makayam'],
                    [b'auq', b'Anus'],
                    [b'aur', b'Aruek'],
                    [b'aut', b'Austral'],
                    [b'auu', b'Auye'],
                    [b'auw', b'Awyi'],
                    [b'aux', 'Aur\xe1'],
                    [b'auy', b'Awiyaana'],
                    [b'auz', b'Arabic, Uzbeki'],
                    [b'ava', b'Avaric'],
                    [b'avb', b'Avau'],
                    [b'avd', b'Alviri-Vidari'],
                    [b'ave', b'Avestan'],
                    [b'avi', b'Avikam'],
                    [b'avk', b'Kotava'],
                    [b'avl', b'Arabic, Eastern Egyptian Bedawi'],
                    [b'avm', b'Angkamuthi'],
                    [b'avn', b'Avatime'],
                    [b'avo', b'Agavotaguerra'],
                    [b'avs', b'Aushiri'],
                    [b'avt', b'Au'],
                    [b'avu', b'Avokaya'],
                    [b'avv', 'Av\xe1-Canoeiro'],
                    [b'awa', b'Awadhi'],
                    [b'awb', b'Awa (Papua New Guinea)'],
                    [b'awc', b'Cicipu'],
                    [b'awe', 'Awet\xed'],
                    [b'awg', b'Anguthimri'],
                    [b'awh', b'Awbono'],
                    [b'awi', b'Aekyom'],
                    [b'awk', b'Awabakal'],
                    [b'awm', b'Arawum'],
                    [b'awn', b'Awngi'],
                    [b'awo', b'Awak'],
                    [b'awr', b'Awera'],
                    [b'aws', b'Awyu, South'],
                    [b'awt', 'Arawet\xe9'],
                    [b'awu', b'Awyu, Central'],
                    [b'awv', b'Awyu, Jair'],
                    [b'aww', b'Awun'],
                    [b'awx', b'Awara'],
                    [b'awy', b'Awyu, Edera'],
                    [b'axb', b'Abipon'],
                    [b'axe', b'Ayerrerenge'],
                    [b'axg', 'Ar\xe1ra, Mato Grosso'],
                    [b'axk', b'Yaka (Central African Republic)'],
                    [b'axl', b'Aranda, Lower Southern'],
                    [b'axm', b'Armenian, Middle'],
                    [b'axx', 'X\xe2r\xe2gur\xe8'],
                    [b'aya', b'Awar'],
                    [b'ayb', b'Gbe, Ayizo'],
                    [b'ayc', b'Aymara, Southern'],
                    [b'ayd', b'Ayabadhu'],
                    [b'aye', b'Ayere'],
                    [b'ayg', b'Ginyanga'],
                    [b'ayh', b'Arabic, Hadrami'],
                    [b'ayi', b'Leyigha'],
                    [b'ayk', b'Akuku'],
                    [b'ayl', b'Arabic, Libyan'],
                    [b'aym', b'Aymara'],
                    [b'ayn', b'Arabic, Sanaani'],
                    [b'ayo', b'Ayoreo'],
                    [b'ayp', b'Arabic, North Mesopotamian'],
                    [b'ayq', b'Ayi (Papua New Guinea)'],
                    [b'ayr', b'Aymara, Central'],
                    [b'ays', b'Ayta, Sorsogon'],
                    [b'ayt', b'Ayta, Magbukun'],
                    [b'ayu', b'Ayu'],
                    [b'ayy', b'Ayta, Tayabas'],
                    [b'ayz', b'Mai Brat'],
                    [b'aza', b'Azha'],
                    [b'azb', b'Azerbaijani, South'],
                    [b'azd', b'Nahuatl, Eastern Durango'],
                    [b'aze', b'Azerbaijani'],
                    [b'azg', b'Amuzgo, San Pedro Amuzgos'],
                    [b'azj', b'Azerbaijani, North'],
                    [b'azm', b'Amuzgo, Ipalapa'],
                    [b'azn', b'Nahuatl, Western Durango'],
                    [b'azo', b'Awing'],
                    [b'azt', b'Atta, Faire'],
                    [b'azz', b'Nahuatl, Highland Puebla'],
                    [b'baa', b'Babatana'],
                    [b'bab', 'Bainouk-Gunyu\xf1o'],
                    [b'bac', b'Badui'],
                    [b'bae', 'Bar\xe9'],
                    [b'baf', b'Nubaca'],
                    [b'bag', b'Tuki'],
                    [b'bah', b'Creole English, Bahamas'],
                    [b'baj', b'Barakai'],
                    [b'bak', b'Bashkir'],
                    [b'bal', b'Baluchi'],
                    [b'bam', b'Bambara'],
                    [b'ban', b'Balinese'],
                    [b'bao', b'Waimaha'],
                    [b'bap', b'Bantawa'],
                    [b'bar', b'Bavarian'],
                    [b'bas', b'Basa (Cameroon)'],
                    [b'bau', b'Bada (Nigeria)'],
                    [b'bav', b'Vengo'],
                    [b'baw', b'Bambili-Bambui'],
                    [b'bax', b'Bamun'],
                    [b'bay', b'Batuley'],
                    [b'bba', b'Baatonum'],
                    [b'bbb', b'Barai'],
                    [b'bbc', b'Batak Toba'],
                    [b'bbd', b'Bau'],
                    [b'bbe', b'Bangba'],
                    [b'bbf', b'Baibai'],
                    [b'bbg', b'Barama'],
                    [b'bbh', b'Bugan'],
                    [b'bbi', b'Barombi'],
                    [b'bbj', "Ghom\xe1l\xe1'"],
                    [b'bbk', b'Babanki'],
                    [b'bbl', b'Bats'],
                    [b'bbm', b'Babango'],
                    [b'bbn', b'Uneapa'],
                    [b'bbo', 'Bobo Madar\xe9, Northern'],
                    [b'bbp', b'Banda, West Central'],
                    [b'bbq', b'Bamali'],
                    [b'bbr', b'Girawa'],
                    [b'bbs', b'Bakpinka'],
                    [b'bbt', b'Mburku'],
                    [b'bbu', b'Kulung (Nigeria)'],
                    [b'bbv', b'Karnai'],
                    [b'bbw', b'Baba'],
                    [b'bbx', b'Bubia'],
                    [b'bby', b'Befang'],
                    [b'bbz', b'Creole Arabic, Babalia'],
                    [b'bca', b'Bai, Central'],
                    [b'bcb', b'Bainouk-Samik'],
                    [b'bcc', b'Balochi, Southern'],
                    [b'bcd', b'Babar, North'],
                    [b'bce', b'Bamenyam'],
                    [b'bcf', b'Bamu'],
                    [b'bcg', b'Baga Binari'],
                    [b'bch', b'Bariai'],
                    [b'bci', 'Baoul\xe9'],
                    [b'bcj', b'Bardi'],
                    [b'bck', b'Bunaba'],
                    [b'bcl', b'Bikol, Central'],
                    [b'bcm', b'Bannoni'],
                    [b'bcn', b'Bali (Nigeria)'],
                    [b'bco', b'Kaluli'],
                    [b'bcp', b'Bali (Democratic Republic of Congo)'],
                    [b'bcq', b'Bench'],
                    [b'bcr', b'Babine'],
                    [b'bcs', b'Kohumono'],
                    [b'bct', b'Bendi'],
                    [b'bcu', b'Awad Bing'],
                    [b'bcv', b'Shoo-Minda-Nye'],
                    [b'bcw', b'Bana'],
                    [b'bcy', b'Bacama'],
                    [b'bcz', b'Bainouk-Gunyaamolo'],
                    [b'bda', b'Bayot'],
                    [b'bdb', b'Basap'],
                    [b'bdc', 'Ember\xe1-Baud\xf3'],
                    [b'bdd', b'Bunama'],
                    [b'bde', b'Bade'],
                    [b'bdf', b'Biage'],
                    [b'bdg', b'Bonggi'],
                    [b'bdh', b'Baka (Sudan)'],
                    [b'bdi', b'Burun'],
                    [b'bdj', b'Bai'],
                    [b'bdk', b'Budukh'],
                    [b'bdl', b'Bajau, Indonesian'],
                    [b'bdm', b'Buduma'],
                    [b'bdn', b'Baldemu'],
                    [b'bdo', b'Morom'],
                    [b'bdp', b'Bende'],
                    [b'bdq', b'Bahnar'],
                    [b'bdr', b'Bajau, West Coast'],
                    [b'bds', b'Burunge'],
                    [b'bdt', b'Bokoto'],
                    [b'bdu', b'Oroko'],
                    [b'bdv', b'Bodo Parja'],
                    [b'bdw', b'Baham'],
                    [b'bdx', b'Budong-Budong'],
                    [b'bdy', b'Bandjalang'],
                    [b'bdz', b'Badeshi'],
                    [b'bea', b'Beaver'],
                    [b'beb', b'Bebele'],
                    [b'bec', b'Iceve-Maci'],
                    [b'bed', b'Bedoanas'],
                    [b'bee', b'Byangsi'],
                    [b'bef', b'Benabena'],
                    [b'beg', b'Belait'],
                    [b'beh', b'Biali'],
                    [b'bei', b"Bekati'"],
                    [b'bej', b'Beja'],
                    [b'bek', b'Bebeli'],
                    [b'bel', b'Belarusian'],
                    [b'bem', b'Bemba (Zambia)'],
                    [b'ben', b'Bengali'],
                    [b'beo', b'Beami'],
                    [b'bep', b'Besoa'],
                    [b'beq', b'Beembe'],
                    [b'bes', b'Besme'],
                    [b'bet', 'B\xe9te, Guiberoua'],
                    [b'beu', b'Blagar'],
                    [b'bev', 'B\xe9t\xe9, Daloa'],
                    [b'bew', b'Betawi'],
                    [b'bex', b'Jur Modo'],
                    [b'bey', b'Beli (Papua New Guinea)'],
                    [b'bez', b'Bena (Tanzania)'],
                    [b'bfa', b'Bari'],
                    [b'bfb', b'Bareli, Pauri'],
                    [b'bfc', b'Bai, Northern'],
                    [b'bfd', b'Bafut'],
                    [b'bfe', b'Betaf'],
                    [b'bff', b'Bofi'],
                    [b'bfg', b'Kayan, Busang'],
                    [b'bfh', b'Blafe'],
                    [b'bfi', b'British Sign Language'],
                    [b'bfj', b'Bafanji'],
                    [b'bfk', b'Ban Khor Sign Language'],
                    [b'bfl', 'Banda-Nd\xe9l\xe9'],
                    [b'bfm', b'Mmen'],
                    [b'bfn', b'Bunak'],
                    [b'bfo', b'Birifor, Malba'],
                    [b'bfp', b'Beba'],
                    [b'bfq', b'Badaga'],
                    [b'bfr', b'Bazigar'],
                    [b'bfs', b'Bai, Southern'],
                    [b'bft', b'Balti'],
                    [b'bfu', b'Gahri'],
                    [b'bfw', b'Bondo'],
                    [b'bfx', b'Bantayanon'],
                    [b'bfy', b'Bagheli'],
                    [b'bfz', b'Pahari, Mahasu'],
                    [b'bga', b'Gwamhi-Wuri'],
                    [b'bgb', b'Bobongko'],
                    [b'bgc', b'Haryanvi'],
                    [b'bgd', b'Bareli, Rathwi'],
                    [b'bge', b'Bauria'],
                    [b'bgf', b'Bangandu'],
                    [b'bgg', b'Bugun'],
                    [b'bgi', b'Giangan'],
                    [b'bgj', b'Bangolan'],
                    [b'bgk', b'Bit'],
                    [b'bgl', b'Bo (Laos)'],
                    [b'bgm', b'Baga Mboteni'],
                    [b'bgn', b'Balochi, Western'],
                    [b'bgo', b'Baga Koga'],
                    [b'bgp', b'Balochi, Eastern'],
                    [b'bgq', b'Bagri'],
                    [b'bgr', b'Chin, Bawm'],
                    [b'bgs', b'Tagabawa'],
                    [b'bgt', b'Bughotu'],
                    [b'bgu', b'Mbongno'],
                    [b'bgv', b'Warkay-Bipim'],
                    [b'bgw', b'Bhatri'],
                    [b'bgx', b'Turkish, Balkan Gagauz'],
                    [b'bgy', b'Benggoi'],
                    [b'bgz', b'Banggai'],
                    [b'bha', b'Bharia'],
                    [b'bhb', b'Bhili'],
                    [b'bhc', b'Biga'],
                    [b'bhd', b'Bhadrawahi'],
                    [b'bhe', b'Bhaya'],
                    [b'bhf', b'Odiai'],
                    [b'bhg', b'Binandere'],
                    [b'bhh', b'Bukharic'],
                    [b'bhi', b'Bhilali'],
                    [b'bhj', b'Bahing'],
                    [b'bhl', b'Bimin'],
                    [b'bhm', b'Bathari'],
                    [b'bhn', b'Neo-Aramaic, Bohtan'],
                    [b'bho', b'Bhojpuri'],
                    [b'bhp', b'Bima'],
                    [b'bhq', b'Tukang Besi South'],
                    [b'bhr', b'Malagasy, Bara'],
                    [b'bhs', b'Buwal'],
                    [b'bht', b'Bhattiyali'],
                    [b'bhu', b'Bhunjia'],
                    [b'bhv', b'Bahau'],
                    [b'bhw', b'Biak'],
                    [b'bhx', b'Bhalay'],
                    [b'bhy', b'Bhele'],
                    [b'bhz', b'Bada (Indonesia)'],
                    [b'bia', b'Badimaya'],
                    [b'bib', b'Bissa'],
                    [b'bic', b'Bikaru'],
                    [b'bid', b'Bidiyo'],
                    [b'bie', b'Bepour'],
                    [b'bif', b'Biafada'],
                    [b'big', b'Biangai'],
                    [b'bij', b'Vaghat-Ya-Bijim-Legeri'],
                    [b'bik', b'Bikol'],
                    [b'bil', b'Bile'],
                    [b'bim', b'Bimoba'],
                    [b'bin', b'Bini'],
                    [b'bio', b'Nai'],
                    [b'bip', b'Bila'],
                    [b'biq', b'Bipi'],
                    [b'bir', b'Bisorio'],
                    [b'bis', b'Bislama'],
                    [b'bit', b'Berinomo'],
                    [b'biu', b'Biete'],
                    [b'biv', b'Birifor, Southern'],
                    [b'biw', b'Kol (Cameroon)'],
                    [b'bix', b'Bijori'],
                    [b'biy', b'Birhor'],
                    [b'biz', b'Baloi'],
                    [b'bja', b'Budza'],
                    [b'bjb', b'Banggarla'],
                    [b'bjc', b'Bariji'],
                    [b'bje', b'Mien, Biao-Jiao'],
                    [b'bjf', b'Neo-Aramaic, Barzani Jewish'],
                    [b'bjg', b'Bidyogo'],
                    [b'bjh', b'Bahinemo'],
                    [b'bji', b'Burji'],
                    [b'bjj', b'Kanauji'],
                    [b'bjk', b'Barok'],
                    [b'bjl', b'Bulu (Papua New Guinea)'],
                    [b'bjm', b'Bajelani'],
                    [b'bjn', b'Banjar'],
                    [b'bjo', b'Banda, Mid-Southern'],
                    [b'bjp', b'Fanamaket'],
                    [b'bjr', b'Binumarien'],
                    [b'bjs', b'Bajan'],
                    [b'bjt', b'Balanta-Ganja'],
                    [b'bju', b'Busuu'],
                    [b'bjv', b'Bedjond'],
                    [b'bjw', 'Bakw\xe9'],
                    [b'bjx', b'Itneg, Banao'],
                    [b'bjy', b'Bayali'],
                    [b'bjz', b'Baruga'],
                    [b'bka', b'Kyak'],
                    [b'bkc', b'Baka (Cameroon)'],
                    [b'bkd', b'Binukid'],
                    [b'bkf', b'Beeke'],
                    [b'bkg', b'Buraka'],
                    [b'bkh', b'Bakoko'],
                    [b'bki', b'Baki'],
                    [b'bkj', b'Pande'],
                    [b'bkk', b'Brokskat'],
                    [b'bkl', b'Berik'],
                    [b'bkm', b'Kom (Cameroon)'],
                    [b'bkn', b'Bukitan'],
                    [b'bko', b"Kwa'"],
                    [b'bkp', b'Boko (Democratic Republic of Congo)'],
                    [b'bkq', 'Bakair\xed'],
                    [b'bkr', b'Bakumpai'],
                    [b'bks', b'Sorsoganon, Northern'],
                    [b'bkt', b'Boloki'],
                    [b'bku', b'Buhid'],
                    [b'bkv', b'Bekwarra'],
                    [b'bkw', b'Bekwel'],
                    [b'bkx', b'Baikeno'],
                    [b'bky', b'Bokyi'],
                    [b'bkz', b'Bungku'],
                    [b'bla', b'Siksika'],
                    [b'blb', b'Bilua'],
                    [b'blc', b'Bella Coola'],
                    [b'bld', b'Bolango'],
                    [b'ble', b'Balanta-Kentohe'],
                    [b'blf', b'Buol'],
                    [b'blg', b'Balau'],
                    [b'blh', b'Kuwaa'],
                    [b'bli', b'Bolia'],
                    [b'blj', b'Bolongan'],
                    [b'blk', b"Karen, Pa'o"],
                    [b'bll', b'Biloxi'],
                    [b'blm', b'Beli (Sudan)'],
                    [b'bln', b'Bikol, Southern Catanduanes'],
                    [b'blo', b'Anii'],
                    [b'blp', b'Blablanga'],
                    [b'blq', b'Baluan-Pam'],
                    [b'blr', b'Blang'],
                    [b'bls', b'Balaesang'],
                    [b'blt', b'Tai Dam'],
                    [b'blv', b'Bolo'],
                    [b'blw', b'Balangao'],
                    [b'blx', b'Ayta, Mag-Indi'],
                    [b'bly', b'Notre'],
                    [b'blz', b'Balantak'],
                    [b'bma', b'Lame'],
                    [b'bmb', b'Bembe'],
                    [b'bmc', b'Biem'],
                    [b'bmd', b'Manduri, Baga'],
                    [b'bme', b'Limassa'],
                    [b'bmf', b'Bom'],
                    [b'bmg', b'Bamwe'],
                    [b'bmh', b'Kein'],
                    [b'bmi', b'Bagirmi'],
                    [b'bmj', b'Bote-Majhi'],
                    [b'bmk', b'Ghayavi'],
                    [b'bml', b'Bomboli'],
                    [b'bmm', b'Malagasy, Northern Betsimisaraka'],
                    [b'bmn', b'Bina (Papua New Guinea)'],
                    [b'bmo', b'Bambalang'],
                    [b'bmp', b'Bulgebi'],
                    [b'bmq', b'Bomu'],
                    [b'bmr', b'Muinane'],
                    [b'bms', b'Kanuri, Bilma'],
                    [b'bmt', b'Biao Mon'],
                    [b'bmu', b'Somba-Siawari'],
                    [b'bmv', b'Bum'],
                    [b'bmw', b'Bomwali'],
                    [b'bmx', b'Baimak'],
                    [b'bmy', b'Bemba (Democratic Republic of Congo)'],
                    [b'bmz', b'Baramu'],
                    [b'bna', b'Bonerate'],
                    [b'bnb', b'Bookan'],
                    [b'bnc', b'Bontok'],
                    [b'bnd', b'Banda (Indonesia)'],
                    [b'bne', b'Bintauna'],
                    [b'bnf', b'Masiwang'],
                    [b'bng', b'Benga'],
                    [b'bni', b'Bangi'],
                    [b'bnj', b'Tawbuid, Eastern'],
                    [b'bnk', b'Bierebo'],
                    [b'bnl', b'Boon'],
                    [b'bnm', b'Batanga'],
                    [b'bnn', b'Bunun'],
                    [b'bno', b'Bantoanon'],
                    [b'bnp', b'Bola'],
                    [b'bnq', b'Bantik'],
                    [b'bnr', b'Butmas-Tur'],
                    [b'bns', b'Bundeli'],
                    [b'bnu', b'Bentong'],
                    [b'bnv', b'Bonerif'],
                    [b'bnw', b'Bisis'],
                    [b'bnx', b'Bangubangu'],
                    [b'bny', b'Bintulu'],
                    [b'bnz', b'Beezen'],
                    [b'boa', b'Bora'],
                    [b'bob', b'Aweer'],
                    [b'bod', b'Tibetan'],
                    [b'boe', b'Mundabli'],
                    [b'bof', b'Bolon'],
                    [b'bog', b'Bamako Sign Language'],
                    [b'boh', b'Boma'],
                    [b'boi', 'Barbare\xf1o'],
                    [b'boj', b'Anjam'],
                    [b'bok', b'Bonjo'],
                    [b'bol', b'Bole'],
                    [b'bom', b'Berom'],
                    [b'bon', b'Bine'],
                    [b'boo', 'Bozo, Tiemac\xe8w\xe8'],
                    [b'bop', b'Bonkiman'],
                    [b'boq', b'Bogaya'],
                    [b'bor', 'Bor\xf4ro'],
                    [b'bos', b'Bosnian'],
                    [b'bot', b'Bongo'],
                    [b'bou', b'Bondei'],
                    [b'bov', b'Tuwuli'],
                    [b'bow', b'Rema'],
                    [b'box', b'Buamu'],
                    [b'boy', b'Bodo (Central African Republic)'],
                    [b'boz', 'Bozo, Ti\xe9yaxo'],
                    [b'bpa', b'Daakaka'],
                    [b'bpb', b'Barbacoas'],
                    [b'bpd', b'Banda-Banda'],
                    [b'bpg', b'Bonggo'],
                    [b'bph', b'Botlikh'],
                    [b'bpi', b'Bagupi'],
                    [b'bpj', b'Binji'],
                    [b'bpk', b'Orowe'],
                    [b'bpl', b'Broome Pearling Lugger Pidgin'],
                    [b'bpm', b'Biyom'],
                    [b'bpn', b'Dzao Min'],
                    [b'bpo', b'Anasi'],
                    [b'bpp', b'Kaure'],
                    [b'bpq', b'Malay, Banda'],
                    [b'bpr', b'Blaan, Koronadal'],
                    [b'bps', b'Blaan, Sarangani'],
                    [b'bpt', b'Barrow Point'],
                    [b'bpu', b'Bongu'],
                    [b'bpv', b'Marind, Bian'],
                    [b'bpw', b'Bo (Papua New Guinea)'],
                    [b'bpx', b'Bareli, Palya'],
                    [b'bpy', b'Bishnupriya'],
                    [b'bpz', b'Bilba'],
                    [b'bqa', b'Tchumbuli'],
                    [b'bqb', b'Bagusa'],
                    [b'bqc', b'Boko (Benin)'],
                    [b'bqd', b'Bung'],
                    [b'bqf', b'Baga Kaloum'],
                    [b'bqg', b'Bago-Kusuntu'],
                    [b'bqh', b'Baima'],
                    [b'bqi', b'Bakhtiari'],
                    [b'bqj', b'Bandial'],
                    [b'bqk', 'Banda-Mbr\xe8s'],
                    [b'bql', b'Bilakura'],
                    [b'bqm', b'Wumboko'],
                    [b'bqn', b'Bulgarian Sign Language'],
                    [b'bqo', b'Balo'],
                    [b'bqp', b'Busa'],
                    [b'bqq', b'Biritai'],
                    [b'bqr', b'Burusu'],
                    [b'bqs', b'Bosngun'],
                    [b'bqt', b'Bamukumbit'],
                    [b'bqu', b'Boguru'],
                    [b'bqv', b'Koro Wachi'],
                    [b'bqw', b'Buru (Nigeria)'],
                    [b'bqx', b'Baangi'],
                    [b'bqy', b'Bengkala Sign Language'],
                    [b'bqz', b'Bakaka'],
                    [b'bra', b'Braj'],
                    [b'brb', b'Lave'],
                    [b'brc', b'Creole Dutch, Berbice'],
                    [b'brd', b'Baraamu'],
                    [b'bre', b'Breton'],
                    [b'brf', b'Bera'],
                    [b'brg', b'Baure'],
                    [b'brh', b'Brahui'],
                    [b'bri', b'Mokpwe'],
                    [b'brj', b'Bieria'],
                    [b'brk', b'Birked'],
                    [b'brl', b'Birwa'],
                    [b'brm', b'Barambu'],
                    [b'brn', b'Boruca'],
                    [b'bro', b'Brokkat'],
                    [b'brp', b'Barapasi'],
                    [b'brq', b'Breri'],
                    [b'brr', b'Birao'],
                    [b'brs', b'Baras'],
                    [b'brt', b'Bitare'],
                    [b'bru', b'Bru, Eastern'],
                    [b'brv', b'Bru, Western'],
                    [b'brw', b'Bellari'],
                    [b'brx', b'Bodo (India)'],
                    [b'bry', b'Burui'],
                    [b'brz', b'Bilbil'],
                    [b'bsa', b'Abinomn'],
                    [b'bsb', b'Bisaya, Brunei'],
                    [b'bsc', b'Bassari'],
                    [b'bse', b'Wushi'],
                    [b'bsf', b'Bauchi'],
                    [b'bsg', b'Bashkardi'],
                    [b'bsh', b'Kati'],
                    [b'bsi', b'Bassossi'],
                    [b'bsj', b'Bangwinji'],
                    [b'bsk', b'Burushaski'],
                    [b'bsl', b'Basa-Gumna'],
                    [b'bsm', b'Busami'],
                    [b'bsn', b'Barasana-Eduria'],
                    [b'bso', b'Buso'],
                    [b'bsp', b'Baga Sitemu'],
                    [b'bsq', b'Bassa'],
                    [b'bsr', b'Bassa-Kontagora'],
                    [b'bss', b'Akoose'],
                    [b'bst', b'Basketo'],
                    [b'bsu', b'Bahonsuai'],
                    [b'bsv', 'Baga Soban\xe9'],
                    [b'bsw', b'Baiso'],
                    [b'bsx', b'Yangkam'],
                    [b'bsy', b'Bisaya, Sabah'],
                    [b'bta', b'Bata'],
                    [b'btc', b'Bati (Cameroon)'],
                    [b'btd', b'Batak Dairi'],
                    [b'bte', b'Gamo-Ningi'],
                    [b'btf', b'Birgit'],
                    [b'btg', 'B\xe9t\xe9, Gagnoa'],
                    [b'bth', b'Bidayuh, Biatah'],
                    [b'bti', b'Burate'],
                    [b'btj', b'Malay, Bacanese'],
                    [b'btl', b'Bhatola'],
                    [b'btm', b'Batak Mandailing'],
                    [b'btn', b'Ratagnon'],
                    [b'bto', b'Bikol, Rinconada'],
                    [b'btp', b'Budibud'],
                    [b'btq', b'Batek'],
                    [b'btr', b'Baetora'],
                    [b'bts', b'Batak Simalungun'],
                    [b'btt', b'Bete-Bendi'],
                    [b'btu', b'Batu'],
                    [b'btv', b'Bateri'],
                    [b'btw', b'Butuanon'],
                    [b'btx', b'Batak Karo'],
                    [b'bty', b'Bobot'],
                    [b'btz', b'Batak Alas-Kluet'],
                    [b'bua', b'Buriat'],
                    [b'bub', b'Bua'],
                    [b'buc', b'Bushi'],
                    [b'bud', b'Ntcham'],
                    [b'bue', b'Beothuk'],
                    [b'buf', b'Bushoong'],
                    [b'bug', b'Buginese'],
                    [b'buh', b'Bunu, Younuo'],
                    [b'bui', b'Bongili'],
                    [b'buj', b'Basa-Gurmana'],
                    [b'buk', b'Bugawac'],
                    [b'bul', b'Bulgarian'],
                    [b'bum', b'Bulu (Cameroon)'],
                    [b'bun', b'Sherbro'],
                    [b'buo', b'Terei'],
                    [b'bup', b'Busoa'],
                    [b'buq', b'Brem'],
                    [b'bus', b'Bokobaru'],
                    [b'but', b'Bungain'],
                    [b'buu', b'Budu'],
                    [b'buv', b'Bun'],
                    [b'buw', b'Bubi'],
                    [b'bux', b'Boghom'],
                    [b'buy', b'Bullom So'],
                    [b'buz', b'Bukwen'],
                    [b'bva', b'Barein'],
                    [b'bvb', b'Bube'],
                    [b'bvc', b'Baelelea'],
                    [b'bvd', b'Baeggu'],
                    [b'bve', b'Malay, Berau'],
                    [b'bvf', b'Boor'],
                    [b'bvg', b'Bonkeng'],
                    [b'bvh', b'Bure'],
                    [b'bvi', b'Belanda Viri'],
                    [b'bvj', b'Baan'],
                    [b'bvk', b'Bukat'],
                    [b'bvl', b'Bolivian Sign Language'],
                    [b'bvm', b'Bamunka'],
                    [b'bvn', b'Buna'],
                    [b'bvo', b'Bolgo'],
                    [b'bvp', b'Bumang'],
                    [b'bvq', b'Birri'],
                    [b'bvr', b'Burarra'],
                    [b'bvt', b'Bati (Indonesia)'],
                    [b'bvu', b'Malay, Bukit'],
                    [b'bvv', b'Baniva'],
                    [b'bvw', b'Boga'],
                    [b'bvx', b'Dibole'],
                    [b'bvy', b'Baybayanon'],
                    [b'bvz', b'Bauzi'],
                    [b'bwa', b'Bwatoo'],
                    [b'bwb', b'Namosi-Naitasiri-Serua'],
                    [b'bwc', b'Bwile'],
                    [b'bwd', b'Bwaidoka'],
                    [b'bwe', b'Karen, Bwe'],
                    [b'bwf', b'Boselewa'],
                    [b'bwg', b'Barwe'],
                    [b'bwh', b'Bishuo'],
                    [b'bwi', b'Baniwa'],
                    [b'bwj', 'Bwamu, L\xe1\xe1 L\xe1\xe1'],
                    [b'bwk', b'Bauwaki'],
                    [b'bwl', b'Bwela'],
                    [b'bwm', b'Biwat'],
                    [b'bwn', b'Bunu, Wunai'],
                    [b'bwo', b'Boro (Ethiopia)'],
                    [b'bwp', b'Mandobo Bawah'],
                    [b'bwq', 'Bobo Madar\xe9, Southern'],
                    [b'bwr', b'Bura-Pabir'],
                    [b'bws', b'Bomboma'],
                    [b'bwt', b'Bafaw-Balong'],
                    [b'bwu', b'Buli (Ghana)'],
                    [b'bww', b'Bwa'],
                    [b'bwx', b'Bunu, Bu-Nao'],
                    [b'bwy', b'Bwamu, Cwi'],
                    [b'bwz', b'Bwisi'],
                    [b'bxa', b'Tairaha'],
                    [b'bxb', b'Bor, Belanda'],
                    [b'bxc', b'Molengue'],
                    [b'bxd', b'Pela'],
                    [b'bxe', b'Birale'],
                    [b'bxf', b'Bilur'],
                    [b'bxg', b'Bangala'],
                    [b'bxh', b'Buhutu'],
                    [b'bxi', b'Pirlatapa'],
                    [b'bxj', b'Bayungu'],
                    [b'bxk', b'Bukusu'],
                    [b'bxl', b'Jalkunan'],
                    [b'bxm', b'Buriat, Mongolia'],
                    [b'bxn', b'Burduna'],
                    [b'bxo', b'Barikanchi'],
                    [b'bxp', b'Bebil'],
                    [b'bxq', b'Beele'],
                    [b'bxr', b'Buriat, Russia'],
                    [b'bxs', b'Busam'],
                    [b'bxu', b'Buriat, China'],
                    [b'bxv', b'Berakou'],
                    [b'bxw', b'Bankagooma'],
                    [b'bxx', b'Borna (Democratic Republic of Congo)'],
                    [b'bxz', b'Binahari'],
                    [b'bya', b'Batak'],
                    [b'byb', b'Bikya'],
                    [b'byc', b'Ubaghara'],
                    [b'byd', b"Benyadu'"],
                    [b'bye', b'Pouye'],
                    [b'byf', b'Bete'],
                    [b'byg', b'Baygo'],
                    [b'byh', b'Bhujel'],
                    [b'byi', b'Buyu'],
                    [b'byj', b'Bina (Nigeria)'],
                    [b'byk', b'Biao'],
                    [b'byl', b'Bayono'],
                    [b'bym', b'Bidyara'],
                    [b'byn', b'Bilin'],
                    [b'byo', b'Biyo'],
                    [b'byp', b'Bumaji'],
                    [b'byq', b'Basay'],
                    [b'byr', b'Baruya'],
                    [b'bys', b'Burak'],
                    [b'byt', b'Berti'],
                    [b'byv', b'Medumba'],
                    [b'byw', b'Belhariya'],
                    [b'byx', b'Qaqet'],
                    [b'byy', b'Buya'],
                    [b'byz', b'Banaro'],
                    [b'bza', b'Bandi'],
                    [b'bzb', b'Andio'],
                    [b'bzc', b'Malagasy, Southern Betsimisaraka'],
                    [b'bzd', b'Bribri'],
                    [b'bze', b'Bozo, Jenaama'],
                    [b'bzf', b'Boikin'],
                    [b'bzg', b'Babuza'],
                    [b'bzh', b'Buang, Mapos'],
                    [b'bzi', b'Bisu'],
                    [b'bzj', b'Kriol English, Belize'],
                    [b'bzk', b'Creole English, Nicaragua'],
                    [b'bzl', b'Boano (Sulawesi)'],
                    [b'bzm', b'Bolondo'],
                    [b'bzn', b'Boano (Maluku)'],
                    [b'bzo', b'Bozaba'],
                    [b'bzp', b'Kemberano'],
                    [b'bzq', b'Buli (Indonesia)'],
                    [b'bzr', b'Biri'],
                    [b'bzs', b'Brazilian Sign Language'],
                    [b'bzt', b'Brithenig'],
                    [b'bzu', b'Burmeso'],
                    [b'bzv', b'Naami'],
                    [b'bzw', b'Basa (Nigeria)'],
                    [b'bzx', 'Bozo, K\u025bl\u025bngaxo'],
                    [b'bzy', b'Obanliku'],
                    [b'bzz', b'Evant'],
                    [b'caa', 'Chort\xed'],
                    [b'cab', b'Garifuna'],
                    [b'cac', b'Chuj'],
                    [b'cad', b'Caddo'],
                    [b'cae', b'Lehar'],
                    [b'caf', b'Carrier, Southern'],
                    [b'cag', 'Nivacl\xe9'],
                    [b'cah', b'Cahuarano'],
                    [b'caj', 'Chan\xe9'],
                    [b'cak', b'Kaqchikel'],
                    [b'cal', b'Carolinian'],
                    [b'cam', 'Cemuh\xee'],
                    [b'can', b'Chambri'],
                    [b'cao', 'Ch\xe1cobo'],
                    [b'cap', b'Chipaya'],
                    [b'caq', b'Nicobarese, Car'],
                    [b'car', b'Carib, Galibi'],
                    [b'cas', 'Tsiman\xe9'],
                    [b'cat', b'Catalan'],
                    [b'cav', 'Cavine\xf1a'],
                    [b'caw', b'Callawalla'],
                    [b'cax', b'Chiquitano'],
                    [b'cay', b'Cayuga'],
                    [b'caz', b'Canichana'],
                    [b'cbb', 'Cabiyar\xed'],
                    [b'cbc', b'Carapana'],
                    [b'cbd', b'Carijona'],
                    [b'cbe', b'Chipiajes'],
                    [b'cbg', b'Chimila'],
                    [b'cbh', b'Cagua'],
                    [b'cbi', b'Chachi'],
                    [b'cbj', b'Ede Cabe'],
                    [b'cbk', b'Chavacano'],
                    [b'cbl', b'Chin, Bualkhaw'],
                    [b'cbn', b'Nyahkur'],
                    [b'cbo', b'Izora'],
                    [b'cbr', b'Cashibo-Cacataibo'],
                    [b'cbs', b'Cashinahua'],
                    [b'cbt', b'Chayahuita'],
                    [b'cbu', b'Candoshi-Shapra'],
                    [b'cbv', b'Cacua'],
                    [b'cbw', b'Kinabalian'],
                    [b'cby', b'Carabayo'],
                    [b'cca', b'Cauca'],
                    [b'ccc', b'Chamicuro'],
                    [b'ccd', b'Creole, Cafundo'],
                    [b'cce', b'Chopi'],
                    [b'ccg', b'Daka, Samba'],
                    [b'cch', b'Atsam'],
                    [b'ccj', b'Kasanga'],
                    [b'ccl', b'Cutchi-Swahili'],
                    [b'ccm', b'Creole Malay, Malaccan'],
                    [b'cco', b'Chinantec, Comaltepec'],
                    [b'ccp', b'Chakma'],
                    [b'ccr', b'Cacaopera'],
                    [b'cda', b'Choni'],
                    [b'cde', b'Chenchu'],
                    [b'cdf', b'Chiru'],
                    [b'cdg', b'Chamari'],
                    [b'cdh', b'Chambeali'],
                    [b'cdi', b'Chodri'],
                    [b'cdj', b'Churahi'],
                    [b'cdm', b'Chepang'],
                    [b'cdn', b'Chaudangsi'],
                    [b'cdo', b'Chinese, Min Dong'],
                    [b'cdr', b'Cinda-Regi-Tiyal'],
                    [b'cds', b'Chadian Sign Language'],
                    [b'cdy', b'Chadong'],
                    [b'cdz', b'Koda'],
                    [b'cea', b'Chehalis, Lower'],
                    [b'ceb', b'Cebuano'],
                    [b'ceg', b'Chamacoco'],
                    [b'cek', b'Chin, Eastern Khumi'],
                    [b'cen', b'Cen'],
                    [b'ces', b'Czech'],
                    [b'cet', 'Cent\xfa\xfam'],
                    [b'cfa', b'Dijim-Bwilim'],
                    [b'cfd', b'Cara'],
                    [b'cfg', b'Como Karim'],
                    [b'cfm', b'Chin, Falam'],
                    [b'cga', b'Changriwa'],
                    [b'cgc', b'Kagayanen'],
                    [b'cgg', b'Chiga'],
                    [b'cgk', b'Chocangacakha'],
                    [b'cha', b'Chamorro'],
                    [b'chb', b'Chibcha'],
                    [b'chc', b'Catawba'],
                    [b'chd', b'Chontal, Highland Oaxaca'],
                    [b'che', b'Chechen'],
                    [b'chf', b'Chontal, Tabasco'],
                    [b'chg', b'Chagatai'],
                    [b'chh', b'Chinook'],
                    [b'chj', 'Chinantec, Ojitl\xe1n'],
                    [b'chk', b'Chuukese'],
                    [b'chl', b'Cahuilla'],
                    [b'chm', b'Mari (Russia)'],
                    [b'chn', b'Chinook jargon'],
                    [b'cho', b'Choctaw'],
                    [b'chp', b'Chipewyan'],
                    [b'chq', b'Chinantec, Quiotepec'],
                    [b'chr', b'Cherokee'],
                    [b'cht', 'Chol\xf3n'],
                    [b'chu', b'Slavic, Church'],
                    [b'chv', b'Chuvash'],
                    [b'chw', b'Chuwabu'],
                    [b'chx', b'Chantyal'],
                    [b'chy', b'Cheyenne'],
                    [b'chz', 'Chinantec, Ozumac\xedn'],
                    [b'cia', b'Cia-Cia'],
                    [b'cib', b'Gbe, Ci'],
                    [b'cic', b'Chickasaw'],
                    [b'cid', b'Chimariko'],
                    [b'cie', b'Cineni'],
                    [b'cih', b'Chinali'],
                    [b'cik', b'Kinnauri, Chitkuli'],
                    [b'cim', b'Cimbrian'],
                    [b'cin', b'Cinta Larga'],
                    [b'cip', b'Chiapanec'],
                    [b'cir', b'Tiri'],
                    [b'ciw', b'Chippewa'],
                    [b'ciy', b'Chaima'],
                    [b'cja', b'Cham, Western'],
                    [b'cje', b'Chru'],
                    [b'cjh', b'Chehalis, Upper'],
                    [b'cji', b'Chamalal'],
                    [b'cjk', b'Chokwe'],
                    [b'cjm', b'Cham, Eastern'],
                    [b'cjn', b'Chenapian'],
                    [b'cjo', 'Ash\xe9ninka Pajonal'],
                    [b'cjp', 'Cab\xe9car'],
                    [b'cjs', b'Shor'],
                    [b'cjv', b'Chuave'],
                    [b'cjy', b'Chinese, Jinyu'],
                    [b'ckb', b'Kurdish, Central'],
                    [b'ckh', b'Chak'],
                    [b'ckl', b'Cibak'],
                    [b'ckn', b'Chin, Kaang'],
                    [b'cko', b'Anufo'],
                    [b'ckq', b'Kajakse'],
                    [b'ckr', b'Kairak'],
                    [b'cks', b'Tayo'],
                    [b'ckt', b'Chukot'],
                    [b'cku', b'Koasati'],
                    [b'ckv', b'Kavalan'],
                    [b'ckx', b'Caka'],
                    [b'cky', b'Cakfem-Mushere'],
                    [b'ckz', 'Cakchiquel-Quich\xe9 Mixed Language'],
                    [b'cla', b'Ron'],
                    [b'clc', b'Chilcotin'],
                    [b'cld', b'Neo-Aramaic, Chaldean'],
                    [b'cle', b'Chinantec, Lealao'],
                    [b'clh', b'Chilisso'],
                    [b'cli', b'Chakali'],
                    [b'clj', b'Chin, Laitu'],
                    [b'clk', b'Idu-Mishmi'],
                    [b'cll', b'Chala'],
                    [b'clm', b'Clallam'],
                    [b'clo', b'Chontal, Lowland Oaxaca'],
                    [b'clt', b'Chin, Lautu'],
                    [b'clu', b'Caluyanun'],
                    [b'clw', b'Chulym'],
                    [b'cly', b'Chatino, Eastern Highland'],
                    [b'cma', b'Maa'],
                    [b'cme', b'Cerma'],
                    [b'cmg', b'Mongolian, Classical'],
                    [b'cmi', 'Ember\xe1-Cham\xed'],
                    [b'cml', b'Campalagian'],
                    [b'cmm', b'Michigamea'],
                    [b'cmn', b'Chinese, Mandarin'],
                    [b'cmo', b'Mnong, Central'],
                    [b'cmr', b'Chin, Mro-Khimi'],
                    [b'cms', b'Messapic'],
                    [b'cmt', b'Camtho'],
                    [b'cna', b'Changthang'],
                    [b'cnb', b'Chin, Chinbon'],
                    [b'cnc', 'C\xf4\xf4ng'],
                    [b'cng', b'Qiang, Northern'],
                    [b'cnh', b'Chin, Haka'],
                    [b'cni', 'Ash\xe1ninka'],
                    [b'cnk', b'Chin, Khumi'],
                    [b'cnl', b'Chinantec, Lalana'],
                    [b'cno', b'Con'],
                    [b'cns', b'Asmat, Central'],
                    [b'cnt', b'Chinantec, Tepetotutla'],
                    [b'cnu', b'Chenoua'],
                    [b'cnw', b'Chin, Ngawn'],
                    [b'cnx', b'Cornish, Middle'],
                    [b'coa', b'Malay, Cocos Islands'],
                    [b'cob', b'Chicomuceltec'],
                    [b'coc', b'Cocopa'],
                    [b'cod', b'Cocama-Cocamilla'],
                    [b'coe', b'Koreguaje'],
                    [b'cof', b'Colorado'],
                    [b'cog', b'Chong'],
                    [b'coh', b'Chonyi-Dzihana-Kauma'],
                    [b'coj', b'Cochimi'],
                    [b'cok', b'Cora, Santa Teresa'],
                    [b'col', b'Columbia-Wenatchi'],
                    [b'com', b'Comanche'],
                    [b'con', 'Cof\xe1n'],
                    [b'coo', b'Comox'],
                    [b'cop', b'Coptic'],
                    [b'coq', b'Coquille'],
                    [b'cor', b'Cornish'],
                    [b'cos', b'Corsican'],
                    [b'cot', b'Caquinte'],
                    [b'cou', b'Wamey'],
                    [b'cov', b'Cao Miao'],
                    [b'cow', b'Cowlitz'],
                    [b'cox', b'Nanti'],
                    [b'coy', b'Coyaima'],
                    [b'coz', b'Chochotec'],
                    [b'cpa', b'Chinantec, Palantla'],
                    [b'cpb', 'Ash\xe9ninka, Ucayali-Yur\xfaa'],
                    [b'cpc', 'Ajy\xedninka Apurucayali'],
                    [b'cpg', b'Greek, Cappadocian'],
                    [b'cpi', b'Pidgin English, Chinese'],
                    [b'cpn', b'Cherepon'],
                    [b'cpo', b'Kpeego'],
                    [b'cps', b'Capiznon'],
                    [b'cpu', 'Ash\xe9ninka, Pichis'],
                    [b'cpx', b'Chinese, Pu-Xian'],
                    [b'cpy', 'Ash\xe9ninka, South Ucayali'],
                    [b'cqd', b'Miao, Chuanqiandian Cluster'],
                    [b'cqu', b'Quechua, Chilean'],
                    [b'cra', b'Chara'],
                    [b'crb', b'Carib, Island'],
                    [b'crc', b'Lonwolwol'],
                    [b'crd', b"Coeur d'Alene"],
                    [b'cre', b'Cree'],
                    [b'crf', b'Caramanta'],
                    [b'crg', b'Michif'],
                    [b'crh', b'Tatar, Crimean'],
                    [b'cri', 'S\xe3otomense'],
                    [b'crj', b'Cree, Southern East'],
                    [b'crk', b'Cree, Plains'],
                    [b'crl', b'Cree, Northern East'],
                    [b'crm', b'Cree, Moose'],
                    [b'crn', b'Cora, El Nayar'],
                    [b'cro', b'Crow'],
                    [b'crq', b"Chorote, Iyo'wujwa"],
                    [b'crr', b'Algonquian, Carolina'],
                    [b'crs', b'Creole French, Seselwa'],
                    [b'crt', b"Chorote, Iyojwa'ja"],
                    [b'crv', b'Chaura'],
                    [b'crw', b'Chrau'],
                    [b'crx', b'Carrier'],
                    [b'cry', b'Cori'],
                    [b'crz', 'Cruze\xf1o'],
                    [b'csa', b'Chinantec, Chiltepec'],
                    [b'csb', b'Kashubian'],
                    [b'csc', b'Catalan Sign Language'],
                    [b'csd', b'Chiangmai Sign Language'],
                    [b'cse', b'Czech Sign Language'],
                    [b'csf', b'Cuba Sign Language'],
                    [b'csg', b'Chilean Sign Language'],
                    [b'csh', b'Chin, Asho'],
                    [b'csi', b'Miwok, Coast'],
                    [b'csj', b'Chin, Songlai'],
                    [b'csk', b'Jola-Kasa'],
                    [b'csl', b'Chinese Sign Language'],
                    [b'csm', b'Miwok, Central Sierra'],
                    [b'csn', b'Colombian Sign Language'],
                    [b'cso', b'Chinantec, Sochiapam'],
                    [b'csq', b'Croatia Sign Language'],
                    [b'csr', b'Costa Rican Sign Language'],
                    [b'css', b'Ohlone, Southern'],
                    [b'cst', b'Ohlone, Northern'],
                    [b'csv', b'Chin, Sumtu'],
                    [b'csw', b'Cree, Swampy'],
                    [b'csy', b'Chin, Siyin'],
                    [b'csz', b'Coos'],
                    [b'cta', b'Chatino, Tataltepec'],
                    [b'ctc', b'Chetco'],
                    [b'ctd', b'Chin, Tedim'],
                    [b'cte', b'Chinantec, Tepinapa'],
                    [b'ctg', b'Chittagonian'],
                    [b'cth', b'Chin, Thaiphum'],
                    [b'ctl', b'Chinantec, Tlacoatzintepec'],
                    [b'ctm', b'Chitimacha'],
                    [b'ctn', b'Chhintange'],
                    [b'cto', 'Ember\xe1-Cat\xedo'],
                    [b'ctp', b'Chatino, Western Highland'],
                    [b'cts', b'Bikol, Northern Catanduanes'],
                    [b'ctt', b'Chetti, Wayanad'],
                    [b'ctu', b'Chol'],
                    [b'ctz', b'Chatino, Zacatepec'],
                    [b'cua', b'Cua'],
                    [b'cub', b'Cubeo'],
                    [b'cuc', b'Chinantec, Usila'],
                    [b'cug', b'Cung'],
                    [b'cuh', b'Chuka'],
                    [b'cui', b'Cuiba'],
                    [b'cuj', b'Mashco Piro'],
                    [b'cuk', b'Kuna, San Blas'],
                    [b'cul', b'Culina'],
                    [b'cum', b'Cumeral'],
                    [b'cuo', b'Cumanagoto'],
                    [b'cup', 'Cupe\xf1o'],
                    [b'cuq', b'Cun'],
                    [b'cur', b'Chhulung'],
                    [b'cut', b'Cuicatec, Teutila'],
                    [b'cuu', b'Tai Ya'],
                    [b'cuv', b'Cuvok'],
                    [b'cuw', b'Chukwa'],
                    [b'cux', b'Cuicatec, Tepeuxila'],
                    [b'cvg', b'Chug'],
                    [b'cvn', b'Chinantec, Valle Nacional'],
                    [b'cwa', b'Kabwa'],
                    [b'cwb', b'Maindo'],
                    [b'cwd', b'Cree, Woods'],
                    [b'cwe', b'Kwere'],
                    [b'cwg', b'Chewong'],
                    [b'cwt', b'Kuwaataay'],
                    [b'cya', b'Chatino, Nopala'],
                    [b'cyb', b'Cayubaba'],
                    [b'cym', b'Welsh'],
                    [b'cyo', b'Cuyonon'],
                    [b'czh', b'Chinese, Huizhou'],
                    [b'czk', b'Knaanic'],
                    [b'czn', b'Chatino, Zenzontepec'],
                    [b'czo', b'Chinese, Min Zhong'],
                    [b'czt', b'Chin, Zotung'],
                    [b'daa', 'Dangal\xe9at'],
                    [b'dac', b'Dambi'],
                    [b'dad', b'Marik'],
                    [b'dae', b'Duupa'],
                    [b'dag', b'Dagbani'],
                    [b'dah', b'Gwahatike'],
                    [b'dai', b'Day'],
                    [b'daj', b'Daju, Dar Fur'],
                    [b'dak', b'Dakota'],
                    [b'dal', b'Dahalo'],
                    [b'dam', b'Damakawa'],
                    [b'dan', b'Danish'],
                    [b'dao', b'Chin, Daai'],
                    [b'daq', b'Maria, Dandami'],
                    [b'dar', b'Dargwa'],
                    [b'das', b'Daho-Doo'],
                    [b'dau', b'Daju, Dar Sila'],
                    [b'dav', b'Taita'],
                    [b'daw', b'Davawenyo'],
                    [b'dax', b'Dayi'],
                    [b'daz', b'Dao'],
                    [b'dba', b'Bangime'],
                    [b'dbb', b'Deno'],
                    [b'dbd', b'Dadiya'],
                    [b'dbe', b'Dabe'],
                    [b'dbf', b'Edopi'],
                    [b'dbg', b'Dogon, Dogul Dom'],
                    [b'dbi', b'Doka'],
                    [b'dbj', b"Ida'an"],
                    [b'dbl', b'Dyirbal'],
                    [b'dbm', b'Duguri'],
                    [b'dbn', b'Duriankere'],
                    [b'dbo', b'Dulbu'],
                    [b'dbp', b'Duwai'],
                    [b'dbq', b'Daba'],
                    [b'dbr', b'Dabarre'],
                    [b'dbt', b'Dogon, Ben Tey'],
                    [b'dbu', b'Dogon, Bondum Dom'],
                    [b'dbv', b'Dungu'],
                    [b'dbw', b'Dogon, Bankan Tey'],
                    [b'dby', b'Dibiyaso'],
                    [b'dcc', b'Deccan'],
                    [b'dcr', b'Negerhollands'],
                    [b'dda', b'Dadi Dadi'],
                    [b'ddd', b'Dongotono'],
                    [b'dde', b'Doondo'],
                    [b'ddg', b'Fataluku'],
                    [b'ddi', b'Goodenough, West'],
                    [b'ddj', b'Jaru'],
                    [b'ddn', b'Dendi (Benin)'],
                    [b'ddo', b'Dido'],
                    [b'ddr', b'Dhudhuroa'],
                    [b'dds', b'Dogon, Donno So'],
                    [b'ddw', b'Dawera-Daweloor'],
                    [b'dec', b'Dagik'],
                    [b'ded', b'Dedua'],
                    [b'dee', b'Dewoin'],
                    [b'def', b'Dezfuli'],
                    [b'deg', b'Degema'],
                    [b'deh', b'Dehwari'],
                    [b'dei', b'Demisa'],
                    [b'dek', b'Dek'],
                    [b'del', b'Delaware'],
                    [b'dem', b'Dem'],
                    [b'den', b'Slave (Athapascan)'],
                    [b'dep', b'Delaware, Pidgin'],
                    [b'deq', b'Dendi (Central African Republic)'],
                    [b'der', b'Deori'],
                    [b'des', b'Desano'],
                    [b'deu', b'German'],
                    [b'dev', b'Domung'],
                    [b'dez', b'Dengese'],
                    [b'dga', b'Dagaare, Southern'],
                    [b'dgb', b'Dogon, Bunoge'],
                    [b'dgc', b'Agta, Casiguran Dumagat'],
                    [b'dgd', b'Dagaari Dioula'],
                    [b'dge', b'Degenan'],
                    [b'dgg', b'Doga'],
                    [b'dgh', b'Dghwede'],
                    [b'dgi', b'Dagara, Northern'],
                    [b'dgk', b'Dagba'],
                    [b'dgl', b'Andaandi'],
                    [b'dgn', b'Dagoman'],
                    [b'dgo', b'Dogri (individual language)'],
                    [b'dgr', b'Dogrib'],
                    [b'dgs', b'Dogoso'],
                    [b'dgt', b"Ndra'ngith"],
                    [b'dgu', b'Degaru'],
                    [b'dgw', b'Daungwurrung'],
                    [b'dgx', b'Doghoro'],
                    [b'dgz', b'Daga'],
                    [b'dhd', b'Dhundari'],
                    [b'dhg', b'Djangu'],
                    [b'dhi', b'Dhimal'],
                    [b'dhl', b'Dhalandji'],
                    [b'dhm', b'Zemba'],
                    [b'dhn', b'Dhanki'],
                    [b'dho', b'Dhodia'],
                    [b'dhr', b'Dhargari'],
                    [b'dhs', b'Dhaiso'],
                    [b'dhu', b'Dhurga'],
                    [b'dhv', b'Dehu'],
                    [b'dhw', b'Dhanwar (Nepal)'],
                    [b'dhx', b'Dhungaloo'],
                    [b'dia', b'Dia'],
                    [b'dib', b'Dinka, South Central'],
                    [b'dic', b'Dida, Lakota'],
                    [b'did', b'Didinga'],
                    [b'dif', b'Dieri'],
                    [b'dig', b'Digo'],
                    [b'dih', b'Kumiai'],
                    [b'dii', b'Dimbong'],
                    [b'dij', b'Dai'],
                    [b'dik', b'Dinka, Southwestern'],
                    [b'dil', b'Dilling'],
                    [b'dim', b'Dime'],
                    [b'din', b'Dinka'],
                    [b'dio', b'Dibo'],
                    [b'dip', b'Dinka, Northeastern'],
                    [b'diq', b'Dimli (individual language)'],
                    [b'dir', b'Dirim'],
                    [b'dis', b'Dimasa'],
                    [b'dit', b'Dirari'],
                    [b'diu', b'Diriku'],
                    [b'div', b'Dhivehi'],
                    [b'diw', b'Dinka, Northwestern'],
                    [b'dix', b'Dixon Reef'],
                    [b'diy', b'Diuwe'],
                    [b'diz', b'Ding'],
                    [b'dja', b'Djadjawurrung'],
                    [b'djb', b'Djinba'],
                    [b'djc', b'Daju, Dar Daju'],
                    [b'djd', b'Djamindjung'],
                    [b'dje', b'Zarma'],
                    [b'djf', b'Djangun'],
                    [b'dji', b'Djinang'],
                    [b'djj', b'Djeebbana'],
                    [b'djk', b'Eastern Maroon Creole'],
                    [b'djm', b'Dogon, Jamsay'],
                    [b'djn', b'Djauan'],
                    [b'djo', b'Jangkang'],
                    [b'djr', b'Djambarrpuyngu'],
                    [b'dju', b'Kapriman'],
                    [b'djw', b'Djawi'],
                    [b'dka', b'Dakpakha'],
                    [b'dkk', b'Dakka'],
                    [b'dkr', b'Kuijau'],
                    [b'dks', b'Dinka, Southeastern'],
                    [b'dkx', b'Mazagway'],
                    [b'dlg', b'Dolgan'],
                    [b'dlk', b'Dahalik'],
                    [b'dlm', b'Dalmatian'],
                    [b'dln', b'Darlong'],
                    [b'dma', b'Duma'],
                    [b'dmb', b'Dogon, Mombo'],
                    [b'dmc', b'Gavak'],
                    [b'dmd', b'Madhi Madhi'],
                    [b'dme', b'Dugwor'],
                    [b'dmg', b'Kinabatangan, Upper'],
                    [b'dmk', b'Domaaki'],
                    [b'dml', b'Dameli'],
                    [b'dmm', b'Dama'],
                    [b'dmo', b'Kemedzung'],
                    [b'dmr', b'Damar, East'],
                    [b'dms', b'Dampelas'],
                    [b'dmu', b'Dubu'],
                    [b'dmv', b'Dumpas'],
                    [b'dmw', b'Mudburra'],
                    [b'dmx', b'Dema'],
                    [b'dmy', b'Demta'],
                    [b'dna', b'Dani, Upper Grand Valley'],
                    [b'dnd', b'Daonda'],
                    [b'dne', b'Ndendeule'],
                    [b'dng', b'Dungan'],
                    [b'dni', b'Dani, Lower Grand Valley'],
                    [b'dnj', b'Dan'],
                    [b'dnk', b'Dengka'],
                    [b'dnn', 'Dz\xf9\xf9ngoo'],
                    [b'dnr', b'Danaru'],
                    [b'dnt', b'Dani, Mid Grand Valley'],
                    [b'dnu', b'Danau'],
                    [b'dnv', b'Danu'],
                    [b'dnw', b'Dani, Western'],
                    [b'dny', 'Den\xed'],
                    [b'doa', b'Dom'],
                    [b'dob', b'Dobu'],
                    [b'doc', b'Dong, Northern'],
                    [b'doe', b'Doe'],
                    [b'dof', b'Domu'],
                    [b'doh', b'Dong'],
                    [b'doi', b'Dogri (macrolanguage)'],
                    [b'dok', b'Dondo'],
                    [b'dol', b'Doso'],
                    [b'don', b'Toura (Papua New Guinea)'],
                    [b'doo', b'Dongo'],
                    [b'dop', b'Lukpa'],
                    [b'doq', b'Dominican Sign Language'],
                    [b'dor', b"Dori'o"],
                    [b'dos', 'Dogos\xe9'],
                    [b'dot', b'Dass'],
                    [b'dov', b'Dombe'],
                    [b'dow', b'Doyayo'],
                    [b'dox', b'Bussa'],
                    [b'doy', b'Dompo'],
                    [b'doz', b'Dorze'],
                    [b'dpp', b'Papar'],
                    [b'drb', b'Dair'],
                    [b'drc', b'Minderico'],
                    [b'drd', b'Darmiya'],
                    [b'dre', b'Dolpo'],
                    [b'drg', b'Rungus'],
                    [b'dri', b"C'lela"],
                    [b'drl', b'Paakantyi'],
                    [b'drn', b'Damar, West'],
                    [b'dro', b'Melanau, Daro-Matu'],
                    [b'drq', b'Dura'],
                    [b'drr', b'Dororo'],
                    [b'drs', b'Gedeo'],
                    [b'drt', b'Drents'],
                    [b'dru', b'Rukai'],
                    [b'dry', b'Darai'],
                    [b'dsb', b'Sorbian, Lower'],
                    [b'dse', b'Dutch Sign Language'],
                    [b'dsh', b'Daasanach'],
                    [b'dsi', b'Disa'],
                    [b'dsl', b'Danish Sign Language'],
                    [b'dsn', b'Dusner'],
                    [b'dso', b'Desiya'],
                    [b'dsq', b'Tadaksahak'],
                    [b'dta', b'Daur'],
                    [b'dtb', b'Kadazan, Labuk-Kinabatangan'],
                    [b'dtd', b'Ditidaht'],
                    [b'dth', b'Adithinngithigh'],
                    [b'dti', b'Dogon, Ana Tinga'],
                    [b'dtk', b'Dogon, Tene Kan'],
                    [b'dtm', b'Dogon, Tomo Kan'],
                    [b'dto', b'Dogon, Tommo So'],
                    [b'dtp', b'Dusun, Central'],
                    [b'dtr', b'Lotud'],
                    [b'dts', b'Dogon, Toro So'],
                    [b'dtt', b'Dogon, Toro Tegu'],
                    [b'dtu', b'Dogon, Tebul Ure'],
                    [b'dty', b'Dotyali'],
                    [b'dua', b'Duala'],
                    [b'dub', b'Dubli'],
                    [b'duc', b'Duna'],
                    [b'dud', b'Hun-Saare'],
                    [b'due', b'Agta, Umiray Dumaget'],
                    [b'duf', b'Dumbea'],
                    [b'dug', b'Duruma'],
                    [b'duh', b'Dungra Bhil'],
                    [b'dui', b'Dumun'],
                    [b'duj', b'Dhuwal'],
                    [b'duk', b'Uyajitaya'],
                    [b'dul', b'Agta, Alabat Island'],
                    [b'dum', b'Dutch, Middle (ca. 1050-1350)'],
                    [b'dun', b'Dusun Deyah'],
                    [b'duo', b'Agta, Dupaninan'],
                    [b'dup', b'Duano'],
                    [b'duq', b'Dusun Malang'],
                    [b'dur', b'Dii'],
                    [b'dus', b'Dumi'],
                    [b'duu', b'Drung'],
                    [b'duv', b'Duvle'],
                    [b'duw', b'Dusun Witu'],
                    [b'dux', b'Duungooma'],
                    [b'duy', b'Agta, Dicamay'],
                    [b'duz', b'Duli'],
                    [b'dva', b'Duau'],
                    [b'dwa', b'Diri'],
                    [b'dwr', b'Dawro'],
                    [b'dws', b'Dutton World Speedwords'],
                    [b'dww', b'Dawawa'],
                    [b'dya', b'Dyan'],
                    [b'dyb', b'Dyaberdyaber'],
                    [b'dyd', b'Dyugun'],
                    [b'dyg', b'Agta, Villa Viciosa'],
                    [b'dyi', b'Senoufo, Djimini'],
                    [b'dym', b'Dogon, Yanda Dom'],
                    [b'dyn', b'Dyangadi'],
                    [b'dyo', b'Jola-Fonyi'],
                    [b'dyu', b'Dyula'],
                    [b'dyy', b'Dyaabugay'],
                    [b'dza', b'Tunzu'],
                    [b'dzd', b'Daza'],
                    [b'dze', b'Djiwarli'],
                    [b'dzg', b'Dazaga'],
                    [b'dzl', b'Dzalakha'],
                    [b'dzn', b'Dzando'],
                    [b'dzo', b'Dzongkha'],
                    [b'eaa', b'Karenggapa'],
                    [b'ebg', b'Ebughu'],
                    [b'ebk', b'Bontok, Eastern'],
                    [b'ebo', b'Teke-Ebo'],
                    [b'ebr', 'Ebri\xe9'],
                    [b'ebu', b'Embu'],
                    [b'ecr', b'Eteocretan'],
                    [b'ecs', b'Ecuadorian Sign Language'],
                    [b'ecy', b'Eteocypriot'],
                    [b'eee', b'E'],
                    [b'efa', b'Efai'],
                    [b'efe', b'Efe'],
                    [b'efi', b'Efik'],
                    [b'ega', b'Ega'],
                    [b'egl', b'Emilian'],
                    [b'ego', b'Eggon'],
                    [b'egy', b'Egyptian (Ancient)'],
                    [b'ehu', b'Ehueun'],
                    [b'eip', b'Eipomek'],
                    [b'eit', b'Eitiep'],
                    [b'eiv', b'Askopan'],
                    [b'eja', b'Ejamat'],
                    [b'eka', b'Ekajuk'],
                    [b'ekc', b'Karnic, Eastern'],
                    [b'eke', b'Ekit'],
                    [b'ekg', b'Ekari'],
                    [b'eki', b'Eki'],
                    [b'ekk', b'Estonian, Standard'],
                    [b'ekl', b'Kol (Bangladesh)'],
                    [b'ekm', b'Elip'],
                    [b'eko', b'Koti'],
                    [b'ekp', b'Ekpeye'],
                    [b'ekr', b'Yace'],
                    [b'eky', b'Kayah, Eastern'],
                    [b'ele', b'Elepi'],
                    [b'elh', b'El Hugeirat'],
                    [b'eli', b'Nding'],
                    [b'elk', b'Elkei'],
                    [b'ell', b'Greek, Modern (1453-)'],
                    [b'elm', b'Eleme'],
                    [b'elo', b'El Molo'],
                    [b'elu', b'Elu'],
                    [b'elx', b'Elamite'],
                    [b'ema', b'Emai-Iuleha-Ora'],
                    [b'emb', b'Embaloh'],
                    [b'eme', b'Emerillon'],
                    [b'emg', b'Meohang, Eastern'],
                    [b'emi', b'Mussau-Emira'],
                    [b'emk', b'Maninkakan, Eastern'],
                    [b'emm', b'Mamulique'],
                    [b'emn', b'Eman'],
                    [b'emo', b'Emok'],
                    [b'emp', 'Ember\xe1, Northern'],
                    [b'ems', b'Yupik, Pacific Gulf'],
                    [b'emu', b'Muria, Eastern'],
                    [b'emw', b'Emplawas'],
                    [b'emx', b'Erromintxela'],
                    [b'emy', b'Mayan, Epigraphic'],
                    [b'ena', b'Apali'],
                    [b'enb', b'Markweeta'],
                    [b'enc', b'En'],
                    [b'end', b'Ende'],
                    [b'enf', b'Enets, Forest'],
                    [b'eng', b'English'],
                    [b'enh', b'Enets, Tundra'],
                    [b'enm', b'English, Middle (1100-1500)'],
                    [b'enn', b'Engenni'],
                    [b'eno', b'Enggano'],
                    [b'enq', b'Enga'],
                    [b'enr', b'Emumu'],
                    [b'enu', b'Enu'],
                    [b'env', b'Enwan (Edu State)'],
                    [b'enw', b'Enwan (Akwa Ibom State)'],
                    [b'eot', "Beti (C\xf4te d'Ivoire)"],
                    [b'epi', b'Epie'],
                    [b'epo', b'Esperanto'],
                    [b'era', b'Eravallan'],
                    [b'erg', b'Sie'],
                    [b'erh', b'Eruwa'],
                    [b'eri', b'Ogea'],
                    [b'erk', b'Efate, South'],
                    [b'ero', b'Horpa'],
                    [b'err', b'Erre'],
                    [b'ers', b'Ersu'],
                    [b'ert', b'Eritai'],
                    [b'erw', b'Erokwanas'],
                    [b'ese', b'Ese Ejja'],
                    [b'esh', b'Eshtehardi'],
                    [b'esi', b'Inupiatun, North Alaskan'],
                    [b'esk', b'Inupiatun, Northwest Alaska'],
                    [b'esl', b'Egypt Sign Language'],
                    [b'esm', b'Esuma'],
                    [b'esn', b'Salvadoran Sign Language'],
                    [b'eso', b'Estonian Sign Language'],
                    [b'esq', b'Esselen'],
                    [b'ess', b'Yupik, Central Siberian'],
                    [b'est', b'Estonian'],
                    [b'esu', b'Yupik, Central'],
                    [b'etb', b'Etebi'],
                    [b'etc', b'Etchemin'],
                    [b'eth', b'Ethiopian Sign Language'],
                    [b'etn', b'Eton (Vanuatu)'],
                    [b'eto', b'Eton (Cameroon)'],
                    [b'etr', b'Edolo'],
                    [b'ets', b'Yekhee'],
                    [b'ett', b'Etruscan'],
                    [b'etu', b'Ejagham'],
                    [b'etx', b'Eten'],
                    [b'etz', b'Semimi'],
                    [b'eus', b'Basque'],
                    [b'eve', b'Even'],
                    [b'evh', b'Uvbie'],
                    [b'evn', b'Evenki'],
                    [b'ewe', b'Ewe'],
                    [b'ewo', b'Ewondo'],
                    [b'ext', b'Extremaduran'],
                    [b'eya', b'Eyak'],
                    [b'eyo', b'Keiyo'],
                    [b'eza', b'Ezaa'],
                    [b'eze', b'Uzekwe'],
                    [b'faa', b'Fasu'],
                    [b'fab', b"Fa d'Ambu"],
                    [b'fad', b'Wagi'],
                    [b'faf', b'Fagani'],
                    [b'fag', b'Finongan'],
                    [b'fah', b'Fali, Baissa'],
                    [b'fai', b'Faiwol'],
                    [b'faj', b'Faita'],
                    [b'fak', b'Fang (Cameroon)'],
                    [b'fal', b'Fali, South'],
                    [b'fam', b'Fam'],
                    [b'fan', b'Fang (Equatorial Guinea)'],
                    [b'fao', b'Faroese'],
                    [b'fap', b'Palor'],
                    [b'far', b'Fataleka'],
                    [b'fas', b'Persian'],
                    [b'fat', b'Fanti'],
                    [b'fau', b'Fayu'],
                    [b'fax', b'Fala'],
                    [b'fay', b'Fars, Southwestern'],
                    [b'faz', b'Fars, Northwestern'],
                    [b'fbl', b'Bikol, West Albay'],
                    [b'fcs', b'Quebec Sign Language'],
                    [b'fer', b'Feroge'],
                    [b'ffi', b'Foia Foia'],
                    [b'ffm', b'Fulfulde, Maasina'],
                    [b'fgr', b'Fongoro'],
                    [b'fia', b'Nobiin'],
                    [b'fie', b'Fyer'],
                    [b'fij', b'Fijian'],
                    [b'fil', b'Filipino'],
                    [b'fin', b'Finnish'],
                    [b'fip', b'Fipa'],
                    [b'fir', b'Firan'],
                    [b'fit', b'Finnish, Tornedalen'],
                    [b'fiw', b'Fiwaga'],
                    [b'fkk', 'Kirya-Konz\u0259l'],
                    [b'fkv', b'Finnish, Kven'],
                    [b'fla', b"Kalispel-Pend d'Oreille"],
                    [b'flh', b'Foau'],
                    [b'fli', b'Fali'],
                    [b'fll', b'Fali, North'],
                    [b'fln', b'Flinders Island'],
                    [b'flr', b'Fuliiru'],
                    [b'fly', b'Tsotsitaal'],
                    [b'fmp', b"Fe'fe'"],
                    [b'fmu', b'Muria, Far Western'],
                    [b'fng', b'Fanagalo'],
                    [b'fni', b'Fania'],
                    [b'fod', b'Foodo'],
                    [b'foi', b'Foi'],
                    [b'fom', b'Foma'],
                    [b'fon', b'Fon'],
                    [b'for', b'Fore'],
                    [b'fos', b'Siraya'],
                    [b'fpe', b'Creole English, Fernando Po'],
                    [b'fqs', b'Fas'],
                    [b'fra', b'French'],
                    [b'frc', b'French, Cajun'],
                    [b'frd', b'Fordata'],
                    [b'frk', b'Frankish'],
                    [b'frm', b'French, Middle (ca. 1400-1600)'],
                    [b'fro', b'French, Old (842-ca. 1400)'],
                    [b'frp', b'Arpitan'],
                    [b'frq', b'Forak'],
                    [b'frr', b'Frisian, Northern'],
                    [b'frs', b'Frisian, Eastern'],
                    [b'frt', b'Fortsenal'],
                    [b'fry', b'Frisian, Western'],
                    [b'fse', b'Finnish Sign Language'],
                    [b'fsl', b'French Sign Language'],
                    [b'fss', b'Finland-Swedish Sign Language'],
                    [b'fub', b'Fulfulde, Adamawa'],
                    [b'fuc', b'Pulaar'],
                    [b'fud', b'Futuna, East'],
                    [b'fue', b'Fulfulde, Borgu'],
                    [b'fuf', b'Pular'],
                    [b'fuh', b'Fulfulde, Western Niger'],
                    [b'fui', b'Fulfulde, Bagirmi'],
                    [b'fuj', b'Ko'],
                    [b'ful', b'Fulah'],
                    [b'fum', b'Fum'],
                    [b'fun', 'Fulni\xf4'],
                    [b'fuq', b'Fulfulde, Central-Eastern Niger'],
                    [b'fur', b'Friulian'],
                    [b'fut', b'Futuna-Aniwa'],
                    [b'fuu', b'Furu'],
                    [b'fuv', b'Fulfulde, Nigerian'],
                    [b'fuy', b'Fuyug'],
                    [b'fvr', b'Fur'],
                    [b'fwa', 'Fw\xe2i'],
                    [b'fwe', b'Fwe'],
                    [b'gaa', b'Ga'],
                    [b'gab', b'Gabri'],
                    [b'gac', b'Great Andamanese, Mixed'],
                    [b'gad', b'Gaddang'],
                    [b'gae', b'Guarequena'],
                    [b'gaf', b'Gende'],
                    [b'gag', b'Gagauz'],
                    [b'gah', b'Alekano'],
                    [b'gai', b'Borei'],
                    [b'gaj', b'Gadsup'],
                    [b'gak', b'Gamkonora'],
                    [b'gal', b'Galolen'],
                    [b'gam', b'Kandawo'],
                    [b'gan', b'Chinese, Gan'],
                    [b'gao', b'Gants'],
                    [b'gap', b'Gal'],
                    [b'gaq', b"Gata'"],
                    [b'gar', b'Galeya'],
                    [b'gas', b'Garasia, Adiwasi'],
                    [b'gat', b'Kenati'],
                    [b'gau', b'Gadaba, Mudhili'],
                    [b'gaw', b'Nobonob'],
                    [b'gax', b'Oromo, Borana-Arsi-Guji'],
                    [b'gay', b'Gayo'],
                    [b'gaz', b'Oromo, West Central'],
                    [b'gba', b'Gbaya (Central African Republic)'],
                    [b'gbb', b'Kaytetye'],
                    [b'gbd', b'Karadjeri'],
                    [b'gbe', b'Niksek'],
                    [b'gbf', b'Gaikundi'],
                    [b'gbg', b'Gbanziri'],
                    [b'gbh', b'Gbe, Defi'],
                    [b'gbi', b'Galela'],
                    [b'gbj', b'Gadaba, Bodo'],
                    [b'gbk', b'Gaddi'],
                    [b'gbl', b'Gamit'],
                    [b'gbm', b'Garhwali'],
                    [b'gbn', b"Mo'da"],
                    [b'gbo', b'Grebo, Northern'],
                    [b'gbp', b'Gbaya-Bossangoa'],
                    [b'gbq', b'Gbaya-Bozoum'],
                    [b'gbr', b'Gbagyi'],
                    [b'gbs', b'Gbe, Gbesi'],
                    [b'gbu', b'Gagadu'],
                    [b'gbv', b'Gbanu'],
                    [b'gbw', b'Gabi-Gabi'],
                    [b'gbx', b'Gbe, Eastern Xwla'],
                    [b'gby', b'Gbari'],
                    [b'gbz', b'Dari, Zoroastrian'],
                    [b'gcc', b'Mali'],
                    [b'gcd', b'Ganggalida'],
                    [b'gce', b'Galice'],
                    [b'gcf', b'Creole French, Guadeloupean'],
                    [b'gcl', b'Creole English, Grenadian'],
                    [b'gcn', b'Gaina'],
                    [b'gcr', b'Creole French, Guianese'],
                    [b'gct', b'German, Colonia Tovar'],
                    [b'gda', b'Lohar, Gade'],
                    [b'gdb', b'Gadaba, Pottangi Ollar'],
                    [b'gdc', b'Gugu Badhun'],
                    [b'gdd', b'Gedaged'],
                    [b'gde', b'Gude'],
                    [b'gdf', b'Guduf-Gava'],
                    [b'gdg', b"Ga'dang"],
                    [b'gdh', b'Gadjerawang'],
                    [b'gdi', b'Gundi'],
                    [b'gdj', b'Gurdjar'],
                    [b'gdk', b'Gadang'],
                    [b'gdl', b'Dirasha'],
                    [b'gdm', b'Laal'],
                    [b'gdn', b'Umanakaina'],
                    [b'gdo', b'Ghodoberi'],
                    [b'gdq', b'Mehri'],
                    [b'gdr', b'Wipi'],
                    [b'gds', b'Ghandruk Sign Language'],
                    [b'gdt', b'Kungardutyi'],
                    [b'gdu', b'Gudu'],
                    [b'gdx', b'Godwari'],
                    [b'gea', b'Geruma'],
                    [b'geb', b'Kire'],
                    [b'gec', b'Grebo, Gboloo'],
                    [b'ged', b'Gade'],
                    [b'geg', b'Gengle'],
                    [b'geh', b'German, Hutterite'],
                    [b'gei', b'Gebe'],
                    [b'gej', b'Gen'],
                    [b'gek', b'Yiwom'],
                    [b'gel', b"ut-Ma'in"],
                    [b'geq', b'Geme'],
                    [b'ges', b'Geser-Gorom'],
                    [b'gew', b'Gera'],
                    [b'gex', b'Garre'],
                    [b'gey', b'Enya'],
                    [b'gez', b'Geez'],
                    [b'gfk', b'Patpatar'],
                    [b'gft', b'Gafat'],
                    [b'gfx', b'!Xung, Mangetti Dune'],
                    [b'gga', b'Gao'],
                    [b'ggb', b'Gbii'],
                    [b'ggd', b'Gugadj'],
                    [b'gge', b'Guragone'],
                    [b'ggg', b'Gurgula'],
                    [b'ggk', b'Kungarakany'],
                    [b'ggl', b'Ganglau'],
                    [b'ggm', b'Gugu Mini'],
                    [b'ggn', b'Gurung, Eastern'],
                    [b'ggo', b'Gondi, Southern'],
                    [b'ggt', b'Gitua'],
                    [b'ggu', b'Gagu'],
                    [b'ggw', b'Gogodala'],
                    [b'gha', 'Ghadam\xe8s'],
                    [b'ghc', b'Gaelic, Hiberno-Scottish'],
                    [b'ghe', b'Ghale, Southern'],
                    [b'ghh', b'Ghale, Northern'],
                    [b'ghk', b'Karen, Geko'],
                    [b'ghl', b'Ghulfan'],
                    [b'ghn', b'Ghanongga'],
                    [b'gho', b'Ghomara'],
                    [b'ghr', b'Ghera'],
                    [b'ghs', b'Guhu-Samane'],
                    [b'ght', b'Kuke'],
                    [b'gia', b'Kitja'],
                    [b'gib', b'Gibanawa'],
                    [b'gic', b'Gail'],
                    [b'gid', b'Gidar'],
                    [b'gig', b'Goaria'],
                    [b'gih', b'Githabul'],
                    [b'gil', b'Gilbertese'],
                    [b'gim', b'Gimi (Eastern Highlands)'],
                    [b'gin', b'Hinukh'],
                    [b'gip', b'Gimi (West New Britain)'],
                    [b'giq', b'Gelao, Green'],
                    [b'gir', b'Gelao, Red'],
                    [b'gis', b'Giziga, North'],
                    [b'git', b'Gitxsan'],
                    [b'giu', b'Mulao'],
                    [b'giw', b'Gelao, White'],
                    [b'gix', b'Gilima'],
                    [b'giy', b'Giyug'],
                    [b'giz', b'Giziga, South'],
                    [b'gji', b'Geji'],
                    [b'gjk', b'Koli, Kachi'],
                    [b'gjm', b'Gunditjmara'],
                    [b'gjn', b'Gonja'],
                    [b'gju', b'Gujari'],
                    [b'gka', b'Guya'],
                    [b'gke', b'Ndai'],
                    [b'gkn', b'Gokana'],
                    [b'gko', b'Kok-Nar'],
                    [b'gkp', b'Kpelle, Guinea'],
                    [b'gla', b'Gaelic, Scottish'],
                    [b'glc', b'Bon Gula'],
                    [b'gld', b'Nanai'],
                    [b'gle', b'Irish'],
                    [b'glg', b'Galician'],
                    [b'glh', b'Pashayi, Northwest'],
                    [b'gli', b'Guliguli'],
                    [b'glj', b'Gula Iro'],
                    [b'glk', b'Gilaki'],
                    [b'gll', b'Garlali'],
                    [b'glo', b'Galambu'],
                    [b'glr', b'Glaro-Twabo'],
                    [b'glu', b'Gula (Chad)'],
                    [b'glv', b'Manx'],
                    [b'glw', b'Glavda'],
                    [b'gly', b'Gule'],
                    [b'gma', b'Gambera'],
                    [b'gmb', b"Gula'alaa"],
                    [b'gmd', 'M\xe1ghd\xec'],
                    [b'gmh', b'German, Middle High (ca. 1050-1500)'],
                    [b'gml', b'German, Middle Low'],
                    [b'gmm', b'Gbaya-Mbodomo'],
                    [b'gmn', b'Gimnime'],
                    [b'gmu', b'Gumalu'],
                    [b'gmv', b'Gamo'],
                    [b'gmx', b'Magoma'],
                    [b'gmy', b'Greek, Mycenaean'],
                    [b'gmz', b'Mgbolizhia'],
                    [b'gna', b'Kaansa'],
                    [b'gnb', b'Gangte'],
                    [b'gnc', b'Guanche'],
                    [b'gnd', b'Zulgo-Gemzek'],
                    [b'gne', b'Ganang'],
                    [b'gng', b'Ngangam'],
                    [b'gnh', b'Lere'],
                    [b'gni', b'Gooniyandi'],
                    [b'gnk', b'//Gana'],
                    [b'gnl', b'Gangulu'],
                    [b'gnm', b'Ginuman'],
                    [b'gnn', b'Gumatj'],
                    [b'gno', b'Gondi, Northern'],
                    [b'gnq', b'Gana'],
                    [b'gnr', b'Gureng Gureng'],
                    [b'gnt', b'Guntai'],
                    [b'gnu', b'Gnau'],
                    [b'gnw', 'Guaran\xed, Western Bolivian'],
                    [b'gnz', b'Ganzi'],
                    [b'goa', b'Guro'],
                    [b'gob', b'Playero'],
                    [b'goc', b'Gorakor'],
                    [b'god', 'Godi\xe9'],
                    [b'goe', b'Gongduk'],
                    [b'gof', b'Gofa'],
                    [b'gog', b'Gogo'],
                    [b'goh', b'German, Old High (ca. 750-1050)'],
                    [b'goi', b'Gobasi'],
                    [b'goj', b'Gowlan'],
                    [b'gok', b'Gowli'],
                    [b'gol', b'Gola'],
                    [b'gom', b'Konkani, Goan'],
                    [b'gon', b'Gondi'],
                    [b'goo', b'Gone Dau'],
                    [b'gop', b'Yeretuar'],
                    [b'goq', b'Gorap'],
                    [b'gor', b'Gorontalo'],
                    [b'gos', b'Gronings'],
                    [b'got', b'Gothic'],
                    [b'gou', b'Gavar'],
                    [b'gow', b'Gorowa'],
                    [b'gox', b'Gobu'],
                    [b'goy', b'Goundo'],
                    [b'goz', b'Gozarkhani'],
                    [b'gpa', b'Gupa-Abawa'],
                    [b'gpe', b'Pidgin English, Ghanaian'],
                    [b'gpn', b'Taiap'],
                    [b'gqa', b"Ga'anda"],
                    [b'gqi', b'Guiqiong'],
                    [b'gqn', b'Guana (Brazil)'],
                    [b'gqr', b'Gor'],
                    [b'gqu', b'Qau'],
                    [b'gra', b'Garasia, Rajput'],
                    [b'grb', b'Grebo'],
                    [b'grc', b'Greek, Ancient (to 1453)'],
                    [b'grd', b'Guruntum-Mbaaru'],
                    [b'grg', b'Madi'],
                    [b'grh', b'Gbiri-Niragu'],
                    [b'gri', b'Ghari'],
                    [b'grj', b'Grebo, Southern'],
                    [b'grm', b'Kota Marudu Talantang'],
                    [b'grn', b'Guarani'],
                    [b'gro', b'Groma'],
                    [b'grq', b'Gorovu'],
                    [b'grr', b'Taznatit'],
                    [b'grs', b'Gresi'],
                    [b'grt', b'Garo'],
                    [b'gru', b'Kistane'],
                    [b'grv', b'Grebo, Central'],
                    [b'grw', b'Gweda'],
                    [b'grx', b'Guriaso'],
                    [b'gry', b'Grebo, Barclayville'],
                    [b'grz', b'Guramalum'],
                    [b'gse', b'Ghanaian Sign Language'],
                    [b'gsg', b'German Sign Language'],
                    [b'gsl', b'Gusilay'],
                    [b'gsm', b'Guatemalan Sign Language'],
                    [b'gsn', b'Gusan'],
                    [b'gso', b'Gbaya, Southwest'],
                    [b'gsp', b'Wasembo'],
                    [b'gss', b'Greek Sign Language'],
                    [b'gsw', b'German, Swiss'],
                    [b'gta', 'Guat\xf3'],
                    [b'gti', b'Gbati-ri'],
                    [b'gtu', b'Aghu-Tharnggala'],
                    [b'gua', b'Shiki'],
                    [b'gub', 'Guajaj\xe1ra'],
                    [b'guc', b'Wayuu'],
                    [b'gud', 'Dida, Yocobou\xe9'],
                    [b'gue', b'Gurinji'],
                    [b'guf', b'Gupapuyngu'],
                    [b'gug', 'Guaran\xed, Paraguayan'],
                    [b'guh', b'Guahibo'],
                    [b'gui', 'Guaran\xed, Eastern Bolivian'],
                    [b'guj', b'Gujarati'],
                    [b'guk', b'Gumuz'],
                    [b'gul', b'Creole English, Sea Island'],
                    [b'gum', b'Guambiano'],
                    [b'gun', 'Guaran\xed, Mby\xe1'],
                    [b'guo', b'Guayabero'],
                    [b'gup', b'Gunwinggu'],
                    [b'guq', 'Ach\xe9'],
                    [b'gur', b'Farefare'],
                    [b'gus', b'Guinean Sign Language'],
                    [b'gut', 'Mal\xe9ku Ja\xedka'],
                    [b'guu', 'Yanomam\xf6'],
                    [b'guv', b'Gey'],
                    [b'guw', b'Gun'],
                    [b'gux', 'Gourmanch\xe9ma'],
                    [b'guz', b'Gusii'],
                    [b'gva', b'Guana (Paraguay)'],
                    [b'gvc', b'Guanano'],
                    [b'gve', b'Duwet'],
                    [b'gvf', b'Golin'],
                    [b'gvj', 'Guaj\xe1'],
                    [b'gvl', b'Gulay'],
                    [b'gvm', b'Gurmana'],
                    [b'gvn', b'Kuku-Yalanji'],
                    [b'gvo', 'Gavi\xe3o Do Jiparan\xe1'],
                    [b'gvp', 'Gavi\xe3o, Par\xe1'],
                    [b'gvr', b'Gurung, Western'],
                    [b'gvs', b'Gumawana'],
                    [b'gvy', b'Guyani'],
                    [b'gwa', b'Mbato'],
                    [b'gwb', b'Gwa'],
                    [b'gwc', b'Kalami'],
                    [b'gwd', b'Gawwada'],
                    [b'gwe', b'Gweno'],
                    [b'gwf', b'Gowro'],
                    [b'gwg', b'Moo'],
                    [b'gwi', 'Gwich\u02bcin'],
                    [b'gwj', b'/Gwi'],
                    [b'gwm', b'Awngthim'],
                    [b'gwn', b'Gwandara'],
                    [b'gwr', b'Gwere'],
                    [b'gwt', b'Gawar-Bati'],
                    [b'gwu', b'Guwamu'],
                    [b'gww', b'Kwini'],
                    [b'gwx', b'Gua'],
                    [b'gxx', 'W\xe8 Southern'],
                    [b'gya', b'Gbaya, Northwest'],
                    [b'gyb', b'Garus'],
                    [b'gyd', b'Kayardild'],
                    [b'gye', b'Gyem'],
                    [b'gyf', b'Gungabula'],
                    [b'gyg', b'Gbayi'],
                    [b'gyi', b'Gyele'],
                    [b'gyl', b'Gayil'],
                    [b'gym', 'Ng\xe4bere'],
                    [b'gyn', b'Creole English, Guyanese'],
                    [b'gyr', b'Guarayu'],
                    [b'gyy', b'Gunya'],
                    [b'gza', b'Ganza'],
                    [b'gzi', b'Gazi'],
                    [b'gzn', b'Gane'],
                    [b'haa', b'Han'],
                    [b'hab', b'Hanoi Sign Language'],
                    [b'hac', b'Gurani'],
                    [b'had', b'Hatam'],
                    [b'hae', b'Oromo, Eastern'],
                    [b'haf', b'Haiphong Sign Language'],
                    [b'hag', b'Hanga'],
                    [b'hah', b'Hahon'],
                    [b'hai', b'Haida'],
                    [b'haj', b'Hajong'],
                    [b'hak', b'Chinese, Hakka'],
                    [b'hal', b'Halang'],
                    [b'ham', b'Hewa'],
                    [b'han', b'Hangaza'],
                    [b'hao', 'Hak\xf6'],
                    [b'hap', b'Hupla'],
                    [b'haq', b'Ha'],
                    [b'har', b'Harari'],
                    [b'has', b'Haisla'],
                    [b'hat', b'Haitian'],
                    [b'hau', b'Hausa'],
                    [b'hav', b'Havu'],
                    [b'haw', b'Hawaiian'],
                    [b'hax', b'Haida, Southern'],
                    [b'hay', b'Haya'],
                    [b'haz', b'Hazaragi'],
                    [b'hba', b'Hamba'],
                    [b'hbb', b'Huba'],
                    [b'hbn', b'Heiban'],
                    [b'hbo', b'Hebrew, Ancient'],
                    [b'hbs', b'Serbo-Croatian'],
                    [b'hbu', b'Habu'],
                    [b'hca', b'Creole Hindi, Andaman'],
                    [b'hch', b'Huichol'],
                    [b'hdn', b'Haida, Northern'],
                    [b'hds', b'Honduras Sign Language'],
                    [b'hdy', b'Hadiyya'],
                    [b'hea', b'Miao, Northern Qiandong'],
                    [b'heb', b'Hebrew'],
                    [b'hed', 'Herd\xe9'],
                    [b'heg', b'Helong'],
                    [b'heh', b'Hehe'],
                    [b'hei', b'Heiltsuk'],
                    [b'hem', b'Hemba'],
                    [b'her', b'Herero'],
                    [b'hgm', b'Hai//om'],
                    [b'hgw', b'Haigwai'],
                    [b'hhi', b'Hoia Hoia'],
                    [b'hhr', b'Kerak'],
                    [b'hhy', b'Hoyahoya'],
                    [b'hia', b'Lamang'],
                    [b'hib', b'Hibito'],
                    [b'hid', b'Hidatsa'],
                    [b'hif', b'Hindi, Fiji'],
                    [b'hig', b'Kamwe'],
                    [b'hih', b'Pamosu'],
                    [b'hii', b'Hinduri'],
                    [b'hij', b'Hijuk'],
                    [b'hik', b'Seit-Kaitetu'],
                    [b'hil', b'Hiligaynon'],
                    [b'hin', b'Hindi'],
                    [b'hio', b'Tsoa'],
                    [b'hir', 'Himarim\xe3'],
                    [b'hit', b'Hittite'],
                    [b'hiw', b'Hiw'],
                    [b'hix', 'Hixkary\xe1na'],
                    [b'hji', b'Haji'],
                    [b'hka', b'Kahe'],
                    [b'hke', b'Hunde'],
                    [b'hkk', b'Hunjara-Kaina Ke'],
                    [b'hks', b'Hong Kong Sign Language'],
                    [b'hla', b'Halia'],
                    [b'hlb', b'Halbi'],
                    [b'hld', b'Halang Doan'],
                    [b'hle', b'Hlersu'],
                    [b'hlt', b'Chin, Matu'],
                    [b'hlu', b'Luwian, Hieroglyphic'],
                    [b'hma', b'Hmong, Southern Mashan'],
                    [b'hmb', b'Songhay, Humburi Senni'],
                    [b'hmc', b'Hmong, Central Huishui'],
                    [b'hmd', b'Miao, Large Flowery'],
                    [b'hme', b'Hmong, Eastern Huishui'],
                    [b'hmf', b'Hmong Don'],
                    [b'hmg', b'Hmong, Southwestern Guiyang'],
                    [b'hmh', b'Hmong, Southwestern Huishui'],
                    [b'hmi', b'Hmong, Northern Huishui'],
                    [b'hmj', b'Ge'],
                    [b'hmk', b'Maek'],
                    [b'hml', b'Hmong, Luopohe'],
                    [b'hmm', b'Hmong, Central Mashan'],
                    [b'hmn', b'Hmong'],
                    [b'hmo', b'Hiri Motu'],
                    [b'hmp', b'Hmong, Northern Mashan'],
                    [b'hmq', b'Miao, Eastern Qiandong'],
                    [b'hmr', b'Hmar'],
                    [b'hms', b'Miao, Southern Qiandong'],
                    [b'hmt', b'Hamtai'],
                    [b'hmu', b'Hamap'],
                    [b'hmv', 'Hmong D\xf4'],
                    [b'hmw', b'Hmong, Western Mashan'],
                    [b'hmy', b'Hmong, Southern Guiyang'],
                    [b'hmz', b'Hmong Shua'],
                    [b'hna', b'Mina (Cameroon)'],
                    [b'hnd', b'Hindko, Southern'],
                    [b'hne', b'Chhattisgarhi'],
                    [b'hnh', b'//Ani'],
                    [b'hni', b'Hani'],
                    [b'hnj', b'Hmong Njua'],
                    [b'hnn', b'Hanunoo'],
                    [b'hno', b'Hindko, Northern'],
                    [b'hns', b'Hindustani, Caribbean'],
                    [b'hnu', b'Hung'],
                    [b'hoa', b'Hoava'],
                    [b'hob', b'Mari (Madang Province)'],
                    [b'hoc', b'Ho'],
                    [b'hod', b'Holma'],
                    [b'hoe', b'Horom'],
                    [b'hoh', 'Hoby\xf3t'],
                    [b'hoi', b'Holikachuk'],
                    [b'hoj', b'Hadothi'],
                    [b'hol', b'Holu'],
                    [b'hom', b'Homa'],
                    [b'hoo', b'Holoholo'],
                    [b'hop', b'Hopi'],
                    [b'hor', b'Horo'],
                    [b'hos', b'Ho Chi Minh City Sign Language'],
                    [b'hot', b'Hote'],
                    [b'hov', b'Hovongan'],
                    [b'how', b'Honi'],
                    [b'hoy', b'Holiya'],
                    [b'hoz', b'Hozo'],
                    [b'hpo', b'Hpon'],
                    [b'hps', b"Hawai'i Pidgin Sign Language"],
                    [b'hra', b'Hrangkhol'],
                    [b'hrc', b'Niwer Mil'],
                    [b'hre', b'Hre'],
                    [b'hrk', b'Haruku'],
                    [b'hrm', b'Miao, Horned'],
                    [b'hro', b'Haroi'],
                    [b'hrp', b'Nhirrpi'],
                    [b'hrt', 'H\xe9rtevin'],
                    [b'hru', b'Hruso'],
                    [b'hrv', b'Croatian'],
                    [b'hrw', b'Warwar Feni'],
                    [b'hrx', b'Hunsrik'],
                    [b'hrz', b'Harzani'],
                    [b'hsb', b'Sorbian, Upper'],
                    [b'hsh', b'Hungarian Sign Language'],
                    [b'hsl', b'Hausa Sign Language'],
                    [b'hsn', b'Chinese, Xiang'],
                    [b'hss', b'Harsusi'],
                    [b'hti', b'Hoti'],
                    [b'hto', b'Huitoto, Minica'],
                    [b'hts', b'Hadza'],
                    [b'htu', b'Hitu'],
                    [b'htx', b'Hittite, Middle'],
                    [b'hub', b'Huambisa'],
                    [b'huc', b'=/Hua'],
                    [b'hud', b'Huaulu'],
                    [b'hue', b'Huave, San Francisco Del Mar'],
                    [b'huf', b'Humene'],
                    [b'hug', b'Huachipaeri'],
                    [b'huh', b'Huilliche'],
                    [b'hui', b'Huli'],
                    [b'huj', b'Hmong, Northern Guiyang'],
                    [b'huk', b'Hulung'],
                    [b'hul', b'Hula'],
                    [b'hum', b'Hungana'],
                    [b'hun', b'Hungarian'],
                    [b'huo', b'Hu'],
                    [b'hup', b'Hupa'],
                    [b'huq', b'Tsat'],
                    [b'hur', b'Halkomelem'],
                    [b'hus', b'Huastec'],
                    [b'hut', b'Humla'],
                    [b'huu', b'Huitoto, Murui'],
                    [b'huv', b'Huave, San Mateo Del Mar'],
                    [b'huw', b'Hukumina'],
                    [b'hux', 'Huitoto, N\xfcpode'],
                    [b'huy', 'Hulaul\xe1'],
                    [b'huz', b'Hunzib'],
                    [b'hvc', b'Haitian Vodoun Culture Language'],
                    [b'hve', b'Huave, San Dionisio Del Mar'],
                    [b'hvk', b'Haveke'],
                    [b'hvn', b'Sabu'],
                    [b'hvv', 'Huave, Santa Mar\xeda Del Mar'],
                    [b'hwa', 'Wan\xe9'],
                    [b'hwc', b"Creole English, Hawai'i"],
                    [b'hwo', b'Hwana'],
                    [b'hya', b'Hya'],
                    [b'hye', b'Armenian'],
                    [b'iai', b'Iaai'],
                    [b'ian', b'Iatmul'],
                    [b'iap', b'Iapama'],
                    [b'iar', b'Purari'],
                    [b'iba', b'Iban'],
                    [b'ibb', b'Ibibio'],
                    [b'ibd', b'Iwaidja'],
                    [b'ibe', b'Akpes'],
                    [b'ibg', b'Ibanag'],
                    [b'ibl', b'Ibaloi'],
                    [b'ibm', b'Agoi'],
                    [b'ibn', b'Ibino'],
                    [b'ibo', b'Igbo'],
                    [b'ibr', b'Ibuoro'],
                    [b'ibu', b'Ibu'],
                    [b'iby', b'Ibani'],
                    [b'ica', b'Ede Ica'],
                    [b'ich', b'Etkywan'],
                    [b'icl', b'Icelandic Sign Language'],
                    [b'icr', b'Creole English, Islander'],
                    [b'ida', b'Idakho-Isukha-Tiriki'],
                    [b'idb', b'Indo-Portuguese'],
                    [b'idc', b'Idon'],
                    [b'idd', b'Ede Idaca'],
                    [b'ide', b'Idere'],
                    [b'idi', b'Idi'],
                    [b'ido', b'Ido'],
                    [b'idr', b'Indri'],
                    [b'ids', b'Idesa'],
                    [b'idt', 'Idat\xe9'],
                    [b'idu', b'Idoma'],
                    [b'ifa', b'Ifugao, Amganad'],
                    [b'ifb', b'Ifugao, Batad'],
                    [b'ife', 'If\xe8'],
                    [b'iff', b'Ifo'],
                    [b'ifk', b'Ifugao, Tuwali'],
                    [b'ifm', b'Teke-Fuumu'],
                    [b'ifu', b'Ifugao, Mayoyao'],
                    [b'ify', b'Kallahan, Keley-I'],
                    [b'igb', b'Ebira'],
                    [b'ige', b'Igede'],
                    [b'igg', b'Igana'],
                    [b'igl', b'Igala'],
                    [b'igm', b'Kanggape'],
                    [b'ign', b'Ignaciano'],
                    [b'igo', b'Isebe'],
                    [b'igs', b'Interglossa'],
                    [b'igw', b'Igwe'],
                    [b'ihb', b'Iha Based Pidgin'],
                    [b'ihi', b'Ihievbe'],
                    [b'ihp', b'Iha'],
                    [b'ihw', b'Bidhawal'],
                    [b'iii', b'Yi, Sichuan'],
                    [b'iin', b'Thiin'],
                    [b'ijc', b'Izon'],
                    [b'ije', b'Biseni'],
                    [b'ijj', b'Ede Ije'],
                    [b'ijn', b'Kalabari'],
                    [b'ijs', b'Ijo, Southeast'],
                    [b'ike', b'Inuktitut, Eastern Canadian'],
                    [b'iki', b'Iko'],
                    [b'ikk', b'Ika'],
                    [b'ikl', b'Ikulu'],
                    [b'iko', b'Olulumo-Ikom'],
                    [b'ikp', b'Ikpeshi'],
                    [b'ikr', b'Ikaranggal'],
                    [b'ikt', b'Inuinnaqtun'],
                    [b'iku', b'Inuktitut'],
                    [b'ikv', b'Iku-Gora-Ankwa'],
                    [b'ikw', b'Ikwere'],
                    [b'ikx', b'Ik'],
                    [b'ikz', b'Ikizu'],
                    [b'ila', b'Ile Ape'],
                    [b'ilb', b'Ila'],
                    [b'ile', b'Interlingue'],
                    [b'ilg', b'Garig-Ilgar'],
                    [b'ili', b'Ili Turki'],
                    [b'ilk', b'Ilongot'],
                    [b'ill', b'Iranun'],
                    [b'ilo', b'Iloko'],
                    [b'ils', b'International Sign'],
                    [b'ilu', b"Ili'uun"],
                    [b'ilv', b'Ilue'],
                    [b'ima', b'Malasar, Mala'],
                    [b'ime', b'Imeraguen'],
                    [b'imi', b'Anamgura'],
                    [b'iml', b'Miluk'],
                    [b'imn', b'Imonda'],
                    [b'imo', b'Imbongu'],
                    [b'imr', b'Imroing'],
                    [b'ims', b'Marsian'],
                    [b'imy', b'Milyan'],
                    [
                        b'ina',
                        b'Interlingua (International Auxiliary Language Association)'
                    ],
                    [b'inb', b'Inga'],
                    [b'ind', b'Indonesian'],
                    [b'ing', b"Degexit'an"],
                    [b'inh', b'Ingush'],
                    [b'inj', b'Inga, Jungle'],
                    [b'inl', b'Indonesian Sign Language'],
                    [b'inm', b'Minaean'],
                    [b'inn', b'Isinai'],
                    [b'ino', b'Inoke-Yate'],
                    [b'inp', 'I\xf1apari'],
                    [b'ins', b'Indian Sign Language'],
                    [b'int', b'Intha'],
                    [b'inz', 'Inese\xf1o'],
                    [b'ior', b'Inor'],
                    [b'iou', b'Tuma-Irumu'],
                    [b'iow', b'Iowa-Oto'],
                    [b'ipi', b'Ipili'],
                    [b'ipk', b'Inupiaq'],
                    [b'ipo', b'Ipiko'],
                    [b'iqu', b'Iquito'],
                    [b'iqw', b'Ikwo'],
                    [b'ire', b'Iresim'],
                    [b'irh', b'Irarutu'],
                    [b'iri', b'Irigwe'],
                    [b'irk', b'Iraqw'],
                    [b'irn', 'Ir\xe1ntxe'],
                    [b'irr', b'Ir'],
                    [b'iru', b'Irula'],
                    [b'irx', b'Kamberau'],
                    [b'iry', b'Iraya'],
                    [b'isa', b'Isabi'],
                    [b'isc', b'Isconahua'],
                    [b'isd', b'Isnag'],
                    [b'ise', b'Italian Sign Language'],
                    [b'isg', b'Irish Sign Language'],
                    [b'ish', b'Esan'],
                    [b'isi', b'Nkem-Nkum'],
                    [b'isk', b'Ishkashimi'],
                    [b'isl', b'Icelandic'],
                    [b'ism', b'Masimasi'],
                    [b'isn', b'Isanzu'],
                    [b'iso', b'Isoko'],
                    [b'isr', b'Israeli Sign Language'],
                    [b'ist', b'Istriot'],
                    [b'isu', b'Isu (Menchum Division)'],
                    [b'ita', b'Italian'],
                    [b'itb', b'Itneg, Binongan'],
                    [b'ite', b'Itene'],
                    [b'iti', b'Itneg, Inlaod'],
                    [b'itk', b'Judeo-Italian'],
                    [b'itl', b'Itelmen'],
                    [b'itm', b'Itu Mbon Uzo'],
                    [b'ito', b'Itonama'],
                    [b'itr', b'Iteri'],
                    [b'its', b'Isekiri'],
                    [b'itt', b'Itneg, Maeng'],
                    [b'itv', b'Itawit'],
                    [b'itw', b'Ito'],
                    [b'itx', b'Itik'],
                    [b'ity', b'Itneg, Moyadan'],
                    [b'itz', 'Itz\xe1'],
                    [b'ium', b'Mien, Iu'],
                    [b'ivb', b'Ibatan'],
                    [b'ivv', b'Ivatan'],
                    [b'iwk', b'I-Wak'],
                    [b'iwm', b'Iwam'],
                    [b'iwo', b'Iwur'],
                    [b'iws', b'Iwam, Sepik'],
                    [b'ixc', b'Ixcatec'],
                    [b'ixl', b'Ixil'],
                    [b'iya', b'Iyayu'],
                    [b'iyo', b'Mesaka'],
                    [b'iyx', b'Yaka (Congo)'],
                    [b'izh', b'Ingrian'],
                    [b'izr', b'Izere'],
                    [b'izz', b'Izii'],
                    [b'jaa', 'Jamamad\xed'],
                    [b'jab', b'Hyam'],
                    [b'jac', b"Popti'"],
                    [b'jad', b'Jahanka'],
                    [b'jae', b'Yabem'],
                    [b'jaf', b'Jara'],
                    [b'jah', b'Jah Hut'],
                    [b'jaj', b'Zazao'],
                    [b'jak', b'Jakun'],
                    [b'jal', b'Yalahatan'],
                    [b'jam', b'Creole English, Jamaican'],
                    [b'jan', b'Jandai'],
                    [b'jao', b'Yanyuwa'],
                    [b'jaq', b'Yaqay'],
                    [b'jas', b'Javanese, New Caledonian'],
                    [b'jat', b'Jakati'],
                    [b'jau', b'Yaur'],
                    [b'jav', b'Javanese'],
                    [b'jax', b'Malay, Jambi'],
                    [b'jay', b'Yan-nhangu'],
                    [b'jaz', b'Jawe'],
                    [b'jbe', b'Judeo-Berber'],
                    [b'jbi', b'Badjiri'],
                    [b'jbj', b'Arandai'],
                    [b'jbk', b'Barikewa'],
                    [b'jbn', b'Nafusi'],
                    [b'jbo', b'Lojban'],
                    [b'jbr', b'Jofotek-Bromnya'],
                    [b'jbt', 'Jabut\xed'],
                    [b'jbu', b'Jukun Takum'],
                    [b'jbw', b'Yawijibaya'],
                    [b'jcs', b'Jamaican Country Sign Language'],
                    [b'jct', b'Krymchak'],
                    [b'jda', b'Jad'],
                    [b'jdg', b'Jadgali'],
                    [b'jdt', b'Judeo-Tat'],
                    [b'jeb', b'Jebero'],
                    [b'jee', b'Jerung'],
                    [b'jeg', b'Jeng'],
                    [b'jeh', b'Jeh'],
                    [b'jei', b'Yei'],
                    [b'jek', b'Jeri Kuo'],
                    [b'jel', b'Yelmek'],
                    [b'jen', b'Dza'],
                    [b'jer', b'Jere'],
                    [b'jet', b'Manem'],
                    [b'jeu', b'Jonkor Bourmataguil'],
                    [b'jgb', b'Ngbee'],
                    [b'jge', b'Judeo-Georgian'],
                    [b'jgk', b'Gwak'],
                    [b'jgo', b'Ngomba'],
                    [b'jhi', b'Jehai'],
                    [b'jhs', b'Jhankot Sign Language'],
                    [b'jia', b'Jina'],
                    [b'jib', b'Jibu'],
                    [b'jic', b'Tol'],
                    [b'jid', b'Bu'],
                    [b'jie', b'Jilbe'],
                    [b'jig', b'Djingili'],
                    [b'jih', b'sTodsde'],
                    [b'jii', b'Jiiddu'],
                    [b'jil', b'Jilim'],
                    [b'jim', b'Jimi (Cameroon)'],
                    [b'jio', b'Jiamao'],
                    [b'jiq', b'Guanyinqiao'],
                    [b'jit', b'Jita'],
                    [b'jiu', b'Jinuo, Youle'],
                    [b'jiv', b'Shuar'],
                    [b'jiy', b'Jinuo, Buyuan'],
                    [b'jjr', b'Bankal'],
                    [b'jkm', b'Karen, Mobwa'],
                    [b'jko', b'Kubo'],
                    [b'jkp', b'Karen, Paku'],
                    [b'jkr', b'Koro (India)'],
                    [b'jku', b'Labir'],
                    [b'jle', b'Ngile'],
                    [b'jls', b'Jamaican Sign Language'],
                    [b'jma', b'Dima'],
                    [b'jmb', b'Zumbun'],
                    [b'jmc', b'Machame'],
                    [b'jmd', b'Yamdena'],
                    [b'jmi', b'Jimi (Nigeria)'],
                    [b'jml', b'Jumli'],
                    [b'jmn', b'Naga, Makuri'],
                    [b'jmr', b'Kamara'],
                    [b'jms', b'Mashi (Nigeria)'],
                    [b'jmw', b'Mouwase'],
                    [b'jmx', b'Mixtec, Western Juxtlahuaca'],
                    [b'jna', b'Jangshung'],
                    [b'jnd', b'Jandavra'],
                    [b'jng', b'Yangman'],
                    [b'jni', b'Janji'],
                    [b'jnj', b'Yemsa'],
                    [b'jnl', b'Rawat'],
                    [b'jns', b'Jaunsari'],
                    [b'job', b'Joba'],
                    [b'jod', b'Wojenaka'],
                    [b'jor', 'Jor\xe1'],
                    [b'jos', b'Jordanian Sign Language'],
                    [b'jow', b'Jowulu'],
                    [b'jpa', b'Aramaic, Jewish Palestinian'],
                    [b'jpn', b'Japanese'],
                    [b'jpr', b'Judeo-Persian'],
                    [b'jqr', b'Jaqaru'],
                    [b'jra', b'Jarai'],
                    [b'jrb', b'Judeo-Arabic'],
                    [b'jrr', b'Jiru'],
                    [b'jrt', b'Jorto'],
                    [b'jru', 'Japrer\xeda'],
                    [b'jsl', b'Japanese Sign Language'],
                    [b'jua', 'J\xfama'],
                    [b'jub', b'Wannu'],
                    [b'juc', b'Jurchen'],
                    [b'jud', b'Worodougou'],
                    [b'juh', 'H\xf5ne'],
                    [b'jui', b'Ngadjuri'],
                    [b'juk', b'Wapan'],
                    [b'jul', b'Jirel'],
                    [b'jum', b'Jumjum'],
                    [b'jun', b'Juang'],
                    [b'juo', b'Jiba'],
                    [b'jup', 'Hupd\xeb'],
                    [b'jur', 'Jur\xfana'],
                    [b'jus', b'Jumla Sign Language'],
                    [b'jut', b'Jutish'],
                    [b'juu', b'Ju'],
                    [b'juw', 'W\xe3pha'],
                    [b'juy', b'Juray'],
                    [b'jvd', b'Javindo'],
                    [b'jvn', b'Javanese, Caribbean'],
                    [b'jwi', b'Jwira-Pepesa'],
                    [b'jya', b'Jiarong'],
                    [b'jye', b'Arabic, Judeo-Yemeni'],
                    [b'jyy', b'Jaya'],
                    [b'kaa', b'Kara-Kalpak'],
                    [b'kab', b'Kabyle'],
                    [b'kac', b'Kachin'],
                    [b'kad', b'Adara'],
                    [b'kae', b'Ketangalan'],
                    [b'kaf', b'Katso'],
                    [b'kag', b'Kajaman'],
                    [b'kah', b'Kara (Central African Republic)'],
                    [b'kai', b'Karekare'],
                    [b'kaj', b'Jju'],
                    [b'kak', b'Kallahan, Kayapa'],
                    [b'kal', b'Kalaallisut'],
                    [b'kam', b'Kamba (Kenya)'],
                    [b'kan', b'Kannada'],
                    [b'kao', b'Xaasongaxango'],
                    [b'kap', b'Bezhta'],
                    [b'kaq', b'Capanahua'],
                    [b'kas', b'Kashmiri'],
                    [b'kat', b'Georgian'],
                    [b'kau', b'Kanuri'],
                    [b'kav', 'Katuk\xedna'],
                    [b'kaw', b'Kawi'],
                    [b'kax', b'Kao'],
                    [b'kay', 'Kamayur\xe1'],
                    [b'kaz', b'Kazakh'],
                    [b'kba', b'Kalarko'],
                    [b'kbb', 'Kaxui\xe2na'],
                    [b'kbc', 'Kadiw\xe9u'],
                    [b'kbd', b'Kabardian'],
                    [b'kbe', b'Kanju'],
                    [b'kbf', b'Kakauhua'],
                    [b'kbg', b'Khamba'],
                    [b'kbh', 'Cams\xe1'],
                    [b'kbi', b'Kaptiau'],
                    [b'kbj', b'Kari'],
                    [b'kbk', b'Koiari, Grass'],
                    [b'kbl', b'Kanembu'],
                    [b'kbm', b'Iwal'],
                    [b'kbn', b'Kare (Central African Republic)'],
                    [b'kbo', b'Keliko'],
                    [b'kbp', 'Kabiy\xe8'],
                    [b'kbq', b'Kamano'],
                    [b'kbr', b'Kafa'],
                    [b'kbs', b'Kande'],
                    [b'kbt', b'Abadi'],
                    [b'kbu', b'Kabutra'],
                    [b'kbv', b'Dera (Indonesia)'],
                    [b'kbw', b'Kaiep'],
                    [b'kbx', b'Ap Ma'],
                    [b'kby', b'Kanuri, Manga'],
                    [b'kbz', b'Duhwa'],
                    [b'kca', b'Khanty'],
                    [b'kcb', b'Kawacha'],
                    [b'kcc', b'Lubila'],
                    [b'kcd', 'Kanum, Ngk\xe2lmpw'],
                    [b'kce', b'Kaivi'],
                    [b'kcf', b'Ukaan'],
                    [b'kcg', b'Tyap'],
                    [b'kch', b'Vono'],
                    [b'kci', b'Kamantan'],
                    [b'kcj', b'Kobiana'],
                    [b'kck', b'Kalanga'],
                    [b'kcl', b'Kela (Papua New Guinea)'],
                    [b'kcm', b'Gula (Central African Republic)'],
                    [b'kcn', b'Nubi'],
                    [b'kco', b'Kinalakna'],
                    [b'kcp', b'Kanga'],
                    [b'kcq', b'Kamo'],
                    [b'kcr', b'Katla'],
                    [b'kcs', b'Koenoem'],
                    [b'kct', b'Kaian'],
                    [b'kcu', b'Kami (Tanzania)'],
                    [b'kcv', b'Kete'],
                    [b'kcw', b'Kabwari'],
                    [b'kcx', b'Kachama-Ganjule'],
                    [b'kcy', b'Korandje'],
                    [b'kcz', b'Konongo'],
                    [b'kda', b'Worimi'],
                    [b'kdc', b'Kutu'],
                    [b'kdd', b'Yankunytjatjara'],
                    [b'kde', b'Makonde'],
                    [b'kdf', b'Mamusi'],
                    [b'kdg', b'Seba'],
                    [b'kdh', b'Tem'],
                    [b'kdi', b'Kumam'],
                    [b'kdj', b'Karamojong'],
                    [b'kdk', 'Num\xe8\xe8'],
                    [b'kdl', b'Tsikimba'],
                    [b'kdm', b'Kagoma'],
                    [b'kdn', b'Kunda'],
                    [b'kdp', b'Kaningdon-Nindem'],
                    [b'kdq', b'Koch'],
                    [b'kdr', b'Karaim'],
                    [b'kdt', b'Kuy'],
                    [b'kdu', b'Kadaru'],
                    [b'kdw', b'Koneraw'],
                    [b'kdx', b'Kam'],
                    [b'kdy', b'Keder'],
                    [b'kdz', b'Kwaja'],
                    [b'kea', b'Kabuverdianu'],
                    [b'keb', 'K\xe9l\xe9'],
                    [b'kec', b'Keiga'],
                    [b'ked', b'Kerewe'],
                    [b'kee', b'Keres, Eastern'],
                    [b'kef', b'Kpessi'],
                    [b'keg', b'Tese'],
                    [b'keh', b'Keak'],
                    [b'kei', b'Kei'],
                    [b'kej', b'Kadar'],
                    [b'kek', 'Kekch\xed'],
                    [b'kel', b'Kela (Democratic Republic of Congo)'],
                    [b'kem', b'Kemak'],
                    [b'ken', b'Kenyang'],
                    [b'keo', b'Kakwa'],
                    [b'kep', b'Kaikadi'],
                    [b'keq', b'Kamar'],
                    [b'ker', b'Kera'],
                    [b'kes', b'Kugbo'],
                    [b'ket', b'Ket'],
                    [b'keu', b'Akebu'],
                    [b'kev', b'Kanikkaran'],
                    [b'kew', b'Kewa, West'],
                    [b'kex', b'Kukna'],
                    [b'key', b'Kupia'],
                    [b'kez', b'Kukele'],
                    [b'kfa', b'Kodava'],
                    [b'kfb', b'Kolami, Northwestern'],
                    [b'kfc', b'Konda-Dora'],
                    [b'kfd', b'Koraga, Korra'],
                    [b'kfe', b'Kota (India)'],
                    [b'kff', b'Koya'],
                    [b'kfg', b'Kudiya'],
                    [b'kfh', b'Kurichiya'],
                    [b'kfi', b'Kurumba, Kannada'],
                    [b'kfj', b'Kemiehua'],
                    [b'kfk', b'Kinnauri'],
                    [b'kfl', b'Kung'],
                    [b'kfm', b'Khunsari'],
                    [b'kfn', b'Kuk'],
                    [b'kfo', "Koro (C\xf4te d'Ivoire)"],
                    [b'kfp', b'Korwa'],
                    [b'kfq', b'Korku'],
                    [b'kfr', b'Kachchi'],
                    [b'kfs', b'Bilaspuri'],
                    [b'kft', b'Kanjari'],
                    [b'kfu', b'Katkari'],
                    [b'kfv', b'Kurmukar'],
                    [b'kfw', b'Naga, Kharam'],
                    [b'kfx', b'Pahari, Kullu'],
                    [b'kfy', b'Kumaoni'],
                    [b'kfz', 'Koromf\xe9'],
                    [b'kga', b'Koyaga'],
                    [b'kgb', b'Kawe'],
                    [b'kgc', b'Kasseng'],
                    [b'kgd', b'Kataang'],
                    [b'kge', b'Komering'],
                    [b'kgf', b'Kube'],
                    [b'kgg', b'Kusunda'],
                    [b'kgi', b'Selangor Sign Language'],
                    [b'kgj', b'Kham, Gamale'],
                    [b'kgk', 'Kaiw\xe1'],
                    [b'kgl', b'Kunggari'],
                    [b'kgm', 'Karip\xfana'],
                    [b'kgn', b'Karingani'],
                    [b'kgo', b'Krongo'],
                    [b'kgp', b'Kaingang'],
                    [b'kgq', b'Kamoro'],
                    [b'kgr', b'Abun'],
                    [b'kgs', b'Kumbainggar'],
                    [b'kgt', b'Somyev'],
                    [b'kgu', b'Kobol'],
                    [b'kgv', b'Karas'],
                    [b'kgw', b'Karon Dori'],
                    [b'kgx', b'Kamaru'],
                    [b'kgy', b'Kyerung'],
                    [b'kha', b'Khasi'],
                    [b'khb', 'L\xfc'],
                    [b'khc', b'Tukang Besi North'],
                    [b'khd', 'Kanum, B\xe4di'],
                    [b'khe', b'Korowai'],
                    [b'khf', b'Khuen'],
                    [b'khg', b'Tibetan, Khams'],
                    [b'khh', b'Kehu'],
                    [b'khj', b'Kuturmi'],
                    [b'khk', b'Mongolian, Halh'],
                    [b'khl', b'Lusi'],
                    [b'khm', b'Khmer, Central'],
                    [b'khn', b'Khandesi'],
                    [b'kho', b'Khotanese'],
                    [b'khp', b'Kapori'],
                    [b'khq', b'Songhay, Koyra Chiini'],
                    [b'khr', b'Kharia'],
                    [b'khs', b'Kasua'],
                    [b'kht', b'Khamti'],
                    [b'khu', b'Nkhumbi'],
                    [b'khv', b'Khvarshi'],
                    [b'khw', b'Khowar'],
                    [b'khx', b'Kanu'],
                    [b'khy', b'Kele (Democratic Republic of Congo)'],
                    [b'khz', b'Keapara'],
                    [b'kia', b'Kim'],
                    [b'kib', b'Koalib'],
                    [b'kic', b'Kickapoo'],
                    [b'kid', b'Koshin'],
                    [b'kie', b'Kibet'],
                    [b'kif', b'Kham, Eastern Parbate'],
                    [b'kig', b'Kimaama'],
                    [b'kih', b'Kilmeri'],
                    [b'kii', b'Kitsai'],
                    [b'kij', b'Kilivila'],
                    [b'kik', b'Kikuyu'],
                    [b'kil', b'Kariya'],
                    [b'kim', b'Karagas'],
                    [b'kin', b'Kinyarwanda'],
                    [b'kio', b'Kiowa'],
                    [b'kip', b'Kham, Sheshi'],
                    [b'kiq', b'Kosadle'],
                    [b'kir', b'Kirghiz'],
                    [b'kis', b'Kis'],
                    [b'kit', b'Agob'],
                    [b'kiu', b'Kirmanjki (individual language)'],
                    [b'kiv', b'Kimbu'],
                    [b'kiw', b'Kiwai, Northeast'],
                    [b'kix', b'Naga, Khiamniungan'],
                    [b'kiy', b'Kirikiri'],
                    [b'kiz', b'Kisi'],
                    [b'kja', b'Mlap'],
                    [b'kjb', b"Q'anjob'al"],
                    [b'kjc', b'Konjo, Coastal'],
                    [b'kjd', b'Kiwai, Southern'],
                    [b'kje', b'Kisar'],
                    [b'kjf', b'Khalaj'],
                    [b'kjg', b'Khmu'],
                    [b'kjh', b'Khakas'],
                    [b'kji', b'Zabana'],
                    [b'kjj', b'Khinalugh'],
                    [b'kjk', b'Konjo, Highland'],
                    [b'kjl', b'Kham, Western Parbate'],
                    [b'kjm', 'Kh\xe1ng'],
                    [b'kjn', b'Kunjen'],
                    [b'kjo', b'Kinnauri, Harijan'],
                    [b'kjp', b'Karen, Pwo Eastern'],
                    [b'kjq', b'Keres, Western'],
                    [b'kjr', b'Kurudu'],
                    [b'kjs', b'Kewa, East'],
                    [b'kjt', b'Karen, Phrae Pwo'],
                    [b'kju', b'Kashaya'],
                    [b'kjx', b'Ramopa'],
                    [b'kjy', b'Erave'],
                    [b'kjz', b'Bumthangkha'],
                    [b'kka', b'Kakanda'],
                    [b'kkb', b'Kwerisa'],
                    [b'kkc', b'Odoodee'],
                    [b'kkd', b'Kinuku'],
                    [b'kke', b'Kakabe'],
                    [b'kkf', b'Monpa, Kalaktang'],
                    [b'kkg', b'Kalinga, Mabaka Valley'],
                    [b'kkh', 'Kh\xfcn'],
                    [b'kki', b'Kagulu'],
                    [b'kkj', b'Kako'],
                    [b'kkk', b'Kokota'],
                    [b'kkl', b'Yale, Kosarek'],
                    [b'kkm', b'Kiong'],
                    [b'kkn', b'Kon Keu'],
                    [b'kko', b'Karko'],
                    [b'kkp', b'Gugubera'],
                    [b'kkq', b'Kaiku'],
                    [b'kkr', b'Kir-Balar'],
                    [b'kks', b'Giiwo'],
                    [b'kkt', b'Koi'],
                    [b'kku', b'Tumi'],
                    [b'kkv', b'Kangean'],
                    [b'kkw', b'Teke-Kukuya'],
                    [b'kkx', b'Kohin'],
                    [b'kky', b'Guguyimidjir'],
                    [b'kkz', b'Kaska'],
                    [b'kla', b'Klamath-Modoc'],
                    [b'klb', b'Kiliwa'],
                    [b'klc', b'Kolbila'],
                    [b'kld', b'Gamilaraay'],
                    [b'kle', b'Kulung (Nepal)'],
                    [b'klf', b'Kendeje'],
                    [b'klg', b'Tagakaulo'],
                    [b'klh', b'Weliki'],
                    [b'kli', b'Kalumpang'],
                    [b'klj', b'Khalaj, Turkic'],
                    [b'klk', b'Kono (Nigeria)'],
                    [b'kll', b'Kalagan, Kagan'],
                    [b'klm', b'Migum'],
                    [b'kln', b'Kalenjin'],
                    [b'klo', b'Kapya'],
                    [b'klp', b'Kamasa'],
                    [b'klq', b'Rumu'],
                    [b'klr', b'Khaling'],
                    [b'kls', b'Kalasha'],
                    [b'klt', b'Nukna'],
                    [b'klu', b'Klao'],
                    [b'klv', b'Maskelynes'],
                    [b'klw', b'Lindu'],
                    [b'klx', b'Koluwawa'],
                    [b'kly', b'Kalao'],
                    [b'klz', b'Kabola'],
                    [b'kma', b'Konni'],
                    [b'kmb', b'Kimbundu'],
                    [b'kmc', b'Dong, Southern'],
                    [b'kmd', b'Kalinga, Majukayang'],
                    [b'kme', b'Bakole'],
                    [b'kmf', b'Kare (Papua New Guinea)'],
                    [b'kmg', 'K\xe2te'],
                    [b'kmh', b'Kalam'],
                    [b'kmi', b'Kami (Nigeria)'],
                    [b'kmj', b'Kumarbhag Paharia'],
                    [b'kmk', b'Kalinga, Limos'],
                    [b'kml', b'Kalinga, Tanudan'],
                    [b'kmm', b'Kom (India)'],
                    [b'kmn', b'Awtuw'],
                    [b'kmo', b'Kwoma'],
                    [b'kmp', b'Gimme'],
                    [b'kmq', b'Kwama'],
                    [b'kmr', b'Kurdish, Northern'],
                    [b'kms', b'Kamasau'],
                    [b'kmt', b'Kemtuik'],
                    [b'kmu', b'Kanite'],
                    [b'kmv', 'Creole French, Karip\xfana'],
                    [b'kmw', b'Komo (Democratic Republic of Congo)'],
                    [b'kmx', b'Waboda'],
                    [b'kmy', b'Koma'],
                    [b'kmz', b'Khorasani Turkish'],
                    [b'kna', b'Dera (Nigeria)'],
                    [b'knb', b'Kalinga, Lubuagan'],
                    [b'knc', b'Kanuri, Central'],
                    [b'knd', b'Konda'],
                    [b'kne', b'Kankanaey'],
                    [b'knf', b'Mankanya'],
                    [b'kng', b'Koongo'],
                    [b'kni', b'Kanufi'],
                    [b'knj', b'Kanjobal, Western'],
                    [b'knk', b'Kuranko'],
                    [b'knl', b'Keninjal'],
                    [b'knm', 'Kanamar\xed'],
                    [b'knn', b'Konkani (individual language)'],
                    [b'kno', b'Kono (Sierra Leone)'],
                    [b'knp', b'Kwanja'],
                    [b'knq', b'Kintaq'],
                    [b'knr', b'Kaningra'],
                    [b'kns', b'Kensiu'],
                    [b'knt', 'Katuk\xedna, Panoan'],
                    [b'knu', b'Kono (Guinea)'],
                    [b'knv', b'Tabo'],
                    [b'knw', b'Kung-Ekoka'],
                    [b'knx', b'Kendayan'],
                    [b'kny', b'Kanyok'],
                    [b'knz', 'Kalams\xe9'],
                    [b'koa', b'Konomala'],
                    [b'koc', b'Kpati'],
                    [b'kod', b'Kodi'],
                    [b'koe', b'Kacipo-Balesi'],
                    [b'kof', b'Kubi'],
                    [b'kog', b'Cogui'],
                    [b'koh', b'Koyo'],
                    [b'koi', b'Komi-Permyak'],
                    [b'koj', b'Sara Dunjo'],
                    [b'kok', b'Konkani (macrolanguage)'],
                    [b'kol', b'Kol (Papua New Guinea)'],
                    [b'kom', b'Komi'],
                    [b'kon', b'Kongo'],
                    [b'koo', b'Konzo'],
                    [b'kop', b'Waube'],
                    [b'koq', b'Kota (Gabon)'],
                    [b'kor', b'Korean'],
                    [b'kos', b'Kosraean'],
                    [b'kot', b'Lagwan'],
                    [b'kou', b'Koke'],
                    [b'kov', b'Kudu-Camo'],
                    [b'kow', b'Kugama'],
                    [b'kox', b'Coxima'],
                    [b'koy', b'Koyukon'],
                    [b'koz', b'Korak'],
                    [b'kpa', b'Kutto'],
                    [b'kpb', b'Kurumba, Mullu'],
                    [b'kpc', b'Curripaco'],
                    [b'kpd', b'Koba'],
                    [b'kpe', b'Kpelle'],
                    [b'kpf', b'Komba'],
                    [b'kpg', b'Kapingamarangi'],
                    [b'kph', b'Kplang'],
                    [b'kpi', b'Kofei'],
                    [b'kpj', 'Karaj\xe1'],
                    [b'kpk', b'Kpan'],
                    [b'kpl', b'Kpala'],
                    [b'kpm', b'Koho'],
                    [b'kpn', 'Kepkiriw\xe1t'],
                    [b'kpo', b'Ikposo'],
                    [b'kpq', b'Korupun-Sela'],
                    [b'kpr', b'Korafe-Yegha'],
                    [b'kps', b'Tehit'],
                    [b'kpt', b'Karata'],
                    [b'kpu', b'Kafoa'],
                    [b'kpv', b'Komi-Zyrian'],
                    [b'kpw', b'Kobon'],
                    [b'kpx', b'Koiali, Mountain'],
                    [b'kpy', b'Koryak'],
                    [b'kpz', b'Kupsabiny'],
                    [b'kqa', b'Mum'],
                    [b'kqb', b'Kovai'],
                    [b'kqc', b'Doromu-Koki'],
                    [b'kqd', b'Koy Sanjaq Surat'],
                    [b'kqe', b'Kalagan'],
                    [b'kqf', b'Kakabai'],
                    [b'kqg', b'Khe'],
                    [b'kqh', b'Kisankasa'],
                    [b'kqi', b'Koitabu'],
                    [b'kqj', b'Koromira'],
                    [b'kqk', b'Gbe, Kotafon'],
                    [b'kql', b'Kyenele'],
                    [b'kqm', b'Khisa'],
                    [b'kqn', b'Kaonde'],
                    [b'kqo', b'Krahn, Eastern'],
                    [b'kqp', 'Kimr\xe9'],
                    [b'kqq', b'Krenak'],
                    [b'kqr', b'Kimaragang'],
                    [b'kqs', b'Kissi, Northern'],
                    [b'kqt', b'Kadazan, Klias River'],
                    [b'kqu', b'Seroa'],
                    [b'kqv', b'Okolod'],
                    [b'kqw', b'Kandas'],
                    [b'kqx', b'Mser'],
                    [b'kqy', b'Koorete'],
                    [b'kqz', b'Korana'],
                    [b'kra', b'Kumhali'],
                    [b'krb', b'Karkin'],
                    [b'krc', b'Karachay-Balkar'],
                    [b'krd', b'Kairui-Midiki'],
                    [b'kre', 'Panar\xe1'],
                    [b'krf', b'Koro (Vanuatu)'],
                    [b'krh', b'Kurama'],
                    [b'kri', b'Krio'],
                    [b'krj', b'Kinaray-A'],
                    [b'krk', b'Kerek'],
                    [b'krl', b'Karelian'],
                    [b'krm', b'Krim'],
                    [b'krn', b'Sapo'],
                    [b'krp', b'Korop'],
                    [b'krr', b"Kru'ng 2"],
                    [b'krs', b'Gbaya (Sudan)'],
                    [b'krt', b'Kanuri, Tumari'],
                    [b'kru', b'Kurukh'],
                    [b'krv', b'Kavet'],
                    [b'krw', b'Krahn, Western'],
                    [b'krx', b'Karon'],
                    [b'kry', b'Kryts'],
                    [b'krz', b'Kanum, Sota'],
                    [b'ksa', b'Shuwa-Zamani'],
                    [b'ksb', b'Shambala'],
                    [b'ksc', b'Kalinga, Southern'],
                    [b'ksd', b'Kuanua'],
                    [b'kse', b'Kuni'],
                    [b'ksf', b'Bafia'],
                    [b'ksg', b'Kusaghe'],
                    [b'ksh', 'K\xf6lsch'],
                    [b'ksi', b'Krisa'],
                    [b'ksj', b'Uare'],
                    [b'ksk', b'Kansa'],
                    [b'ksl', b'Kumalu'],
                    [b'ksm', b'Kumba'],
                    [b'ksn', b'Kasiguranin'],
                    [b'kso', b'Kofa'],
                    [b'ksp', b'Kaba'],
                    [b'ksq', b'Kwaami'],
                    [b'ksr', b'Borong'],
                    [b'kss', b'Kisi, Southern'],
                    [b'kst', 'Winy\xe9'],
                    [b'ksu', b'Khamyang'],
                    [b'ksv', b'Kusu'],
                    [b'ksw', b"Karen, S'gaw"],
                    [b'ksx', b'Kedang'],
                    [b'ksy', b'Kharia Thar'],
                    [b'ksz', b'Kodaku'],
                    [b'kta', b'Katua'],
                    [b'ktb', b'Kambaata'],
                    [b'ktc', b'Kholok'],
                    [b'ktd', b'Kokata'],
                    [b'kte', b'Nubri'],
                    [b'ktf', b'Kwami'],
                    [b'ktg', b'Kalkutung'],
                    [b'kth', b'Karanga'],
                    [b'kti', b'Muyu, North'],
                    [b'ktj', b'Krumen, Plapo'],
                    [b'ktk', b'Kaniet'],
                    [b'ktl', b'Koroshi'],
                    [b'ktm', b'Kurti'],
                    [b'ktn', 'Kariti\xe2na'],
                    [b'kto', b'Kuot'],
                    [b'ktp', b'Kaduo'],
                    [b'ktq', b'Katabaga'],
                    [b'ktr', b'Kota Marudu Tinagas'],
                    [b'kts', b'Muyu, South'],
                    [b'ktt', b'Ketum'],
                    [b'ktu', b'Kituba (Democratic Republic of Congo)'],
                    [b'ktv', b'Katu, Eastern'],
                    [b'ktw', b'Kato'],
                    [b'ktx', 'Kaxarar\xed'],
                    [b'kty', 'Kango (Bas-U\xe9l\xe9 District)'],
                    [b'ktz', b"Ju/'hoan"],
                    [b'kua', b'Kuanyama'],
                    [b'kub', b'Kutep'],
                    [b'kuc', b'Kwinsu'],
                    [b'kud', b"'Auhelawa"],
                    [b'kue', b'Kuman'],
                    [b'kuf', b'Katu, Western'],
                    [b'kug', b'Kupa'],
                    [b'kuh', b'Kushi'],
                    [b'kui', 'Kuik\xfaro-Kalap\xe1lo'],
                    [b'kuj', b'Kuria'],
                    [b'kuk', b"Kepo'"],
                    [b'kul', b'Kulere'],
                    [b'kum', b'Kumyk'],
                    [b'kun', b'Kunama'],
                    [b'kuo', b'Kumukio'],
                    [b'kup', b'Kunimaipa'],
                    [b'kuq', b'Karipuna'],
                    [b'kur', b'Kurdish'],
                    [b'kus', b'Kusaal'],
                    [b'kut', b'Kutenai'],
                    [b'kuu', b'Kuskokwim, Upper'],
                    [b'kuv', b'Kur'],
                    [b'kuw', b'Kpagua'],
                    [b'kux', b'Kukatja'],
                    [b'kuy', b"Kuuku-Ya'u"],
                    [b'kuz', b'Kunza'],
                    [b'kva', b'Bagvalal'],
                    [b'kvb', b'Kubu'],
                    [b'kvc', b'Kove'],
                    [b'kvd', b'Kui (Indonesia)'],
                    [b'kve', b'Kalabakan'],
                    [b'kvf', b'Kabalai'],
                    [b'kvg', b'Kuni-Boazi'],
                    [b'kvh', b'Komodo'],
                    [b'kvi', b'Kwang'],
                    [b'kvj', b'Psikye'],
                    [b'kvk', b'Korean Sign Language'],
                    [b'kvl', b'Kayaw'],
                    [b'kvm', b'Kendem'],
                    [b'kvn', b'Kuna, Border'],
                    [b'kvo', b'Dobel'],
                    [b'kvp', b'Kompane'],
                    [b'kvq', b'Karen, Geba'],
                    [b'kvr', b'Kerinci'],
                    [b'kvs', b'Kunggara'],
                    [b'kvt', b'Karen, Lahta'],
                    [b'kvu', b'Karen, Yinbaw'],
                    [b'kvv', b'Kola'],
                    [b'kvw', b'Wersing'],
                    [b'kvx', b'Koli, Parkari'],
                    [b'kvy', b'Karen, Yintale'],
                    [b'kvz', b'Tsakwambo'],
                    [b'kwa', 'D\xe2w'],
                    [b'kwb', b'Kwa'],
                    [b'kwc', b'Likwala'],
                    [b'kwd', b'Kwaio'],
                    [b'kwe', b'Kwerba'],
                    [b'kwf', b"Kwara'ae"],
                    [b'kwg', b'Sara Kaba Deme'],
                    [b'kwh', b'Kowiai'],
                    [b'kwi', b'Awa-Cuaiquer'],
                    [b'kwj', b'Kwanga'],
                    [b'kwk', b'Kwakiutl'],
                    [b'kwl', b'Kofyar'],
                    [b'kwm', b'Kwambi'],
                    [b'kwn', b'Kwangali'],
                    [b'kwo', b'Kwomtari'],
                    [b'kwp', b'Kodia'],
                    [b'kwq', b'Kwak'],
                    [b'kwr', b'Kwer'],
                    [b'kws', b'Kwese'],
                    [b'kwt', b'Kwesten'],
                    [b'kwu', b'Kwakum'],
                    [b'kwv', 'Sara Kaba N\xe1\xe0'],
                    [b'kww', b'Kwinti'],
                    [b'kwx', b'Khirwar'],
                    [b'kwy', b'Kongo, San Salvador'],
                    [b'kwz', b'Kwadi'],
                    [b'kxa', b'Kairiru'],
                    [b'kxb', b'Krobu'],
                    [b'kxc', b'Konso'],
                    [b'kxd', b'Brunei'],
                    [b'kxe', b'Kakihum'],
                    [b'kxf', b'Karen, Manumanaw'],
                    [b'kxh', b'Karo (Ethiopia)'],
                    [b'kxi', b'Murut, Keningau'],
                    [b'kxj', b'Kulfa'],
                    [b'kxk', b'Karen, Zayein'],
                    [b'kxl', b'Kurux, Nepali'],
                    [b'kxm', b'Khmer, Northern'],
                    [b'kxn', b'Melanau, Kanowit-Tanjong'],
                    [b'kxo', 'Kano\xe9'],
                    [b'kxp', b'Koli, Wadiyara'],
                    [b'kxq', 'Kanum, Sm\xe4rky'],
                    [b'kxr', b'Koro (Papua New Guinea)'],
                    [b'kxs', b'Kangjia'],
                    [b'kxt', b'Koiwat'],
                    [b'kxu', b'Kui (India)'],
                    [b'kxv', b'Kuvi'],
                    [b'kxw', b'Konai'],
                    [b'kxx', b'Likuba'],
                    [b'kxy', b'Kayong'],
                    [b'kxz', b'Kerewo'],
                    [b'kya', b'Kwaya'],
                    [b'kyb', b'Kalinga, Butbut'],
                    [b'kyc', b'Kyaka'],
                    [b'kyd', b'Karey'],
                    [b'kye', b'Krache'],
                    [b'kyf', b'Kouya'],
                    [b'kyg', b'Keyagana'],
                    [b'kyh', b'Karok'],
                    [b'kyi', b'Kiput'],
                    [b'kyj', b'Karao'],
                    [b'kyk', b'Kamayo'],
                    [b'kyl', b'Kalapuya'],
                    [b'kym', b'Kpatili'],
                    [b'kyn', b'Binukidnon, Northern'],
                    [b'kyo', b'Kelon'],
                    [b'kyp', b'Kang'],
                    [b'kyq', b'Kenga'],
                    [b'kyr', 'Kuru\xe1ya'],
                    [b'kys', b'Kayan, Baram'],
                    [b'kyt', b'Kayagar'],
                    [b'kyu', b'Kayah, Western'],
                    [b'kyv', b'Kayort'],
                    [b'kyw', b'Kudmali'],
                    [b'kyx', b'Rapoisi'],
                    [b'kyy', b'Kambaira'],
                    [b'kyz', 'Kayab\xed'],
                    [b'kza', b'Karaboro, Western'],
                    [b'kzb', b'Kaibobo'],
                    [b'kzc', b'Kulango, Bondoukou'],
                    [b'kzd', b'Kadai'],
                    [b'kze', b'Kosena'],
                    [b'kzf', b"Kaili, Da'a"],
                    [b'kzg', b'Kikai'],
                    [b'kzi', b'Kelabit'],
                    [b'kzj', b'Kadazan, Coastal'],
                    [b'kzk', b'Kazukuru'],
                    [b'kzl', b'Kayeli'],
                    [b'kzm', b'Kais'],
                    [b'kzn', b'Kokola'],
                    [b'kzo', b'Kaningi'],
                    [b'kzp', b'Kaidipang'],
                    [b'kzq', b'Kaike'],
                    [b'kzr', b'Karang'],
                    [b'kzs', b'Dusun, Sugut'],
                    [b'kzt', b'Dusun, Tambunan'],
                    [b'kzu', b'Kayupulau'],
                    [b'kzv', b'Komyandaret'],
                    [b'kzw', 'Karir\xed-Xoc\xf3'],
                    [b'kzx', b'Kamarian'],
                    [b'kzy', b'Kango (Tshopo District)'],
                    [b'kzz', b'Kalabra'],
                    [b'laa', b'Subanen, Southern'],
                    [b'lab', b'Linear A'],
                    [b'lac', b'Lacandon'],
                    [b'lad', b'Ladino'],
                    [b'lae', b'Pattani'],
                    [b'laf', b'Lafofa'],
                    [b'lag', b'Langi'],
                    [b'lah', b'Lahnda'],
                    [b'lai', b'Lambya'],
                    [b'laj', b'Lango (Uganda)'],
                    [b'lak', b'Laka (Nigeria)'],
                    [b'lal', b'Lalia'],
                    [b'lam', b'Lamba'],
                    [b'lan', b'Laru'],
                    [b'lao', b'Lao'],
                    [b'lap', b'Laka (Chad)'],
                    [b'laq', b'Qabiao'],
                    [b'lar', b'Larteh'],
                    [b'las', b'Lama (Togo)'],
                    [b'lat', b'Latin'],
                    [b'lau', b'Laba'],
                    [b'lav', b'Latvian'],
                    [b'law', b'Lauje'],
                    [b'lax', b'Tiwa'],
                    [b'lay', b'Lama (Myanmar)'],
                    [b'laz', b'Aribwatsa'],
                    [b'lba', b'Lui'],
                    [b'lbb', b'Label'],
                    [b'lbc', b'Lakkia'],
                    [b'lbe', b'Lak'],
                    [b'lbf', b'Tinani'],
                    [b'lbg', b'Laopang'],
                    [b'lbi', b"La'bi"],
                    [b'lbj', b'Ladakhi'],
                    [b'lbk', b'Bontok, Central'],
                    [b'lbl', b'Bikol, Libon'],
                    [b'lbm', b'Lodhi'],
                    [b'lbn', b'Lamet'],
                    [b'lbo', b'Laven'],
                    [b'lbq', b'Wampar'],
                    [b'lbr', b'Lohorung'],
                    [b'lbs', b'Libyan Sign Language'],
                    [b'lbt', b'Lachi'],
                    [b'lbu', b'Labu'],
                    [b'lbv', b'Lavatbura-Lamusong'],
                    [b'lbw', b'Tolaki'],
                    [b'lbx', b'Lawangan'],
                    [b'lby', b'Lamu-Lamu'],
                    [b'lbz', b'Lardil'],
                    [b'lcc', b'Legenyem'],
                    [b'lcd', b'Lola'],
                    [b'lce', b'Loncong'],
                    [b'lcf', b'Lubu'],
                    [b'lch', b'Luchazi'],
                    [b'lcl', b'Lisela'],
                    [b'lcm', b'Tungag'],
                    [b'lcp', b'Lawa, Western'],
                    [b'lcq', b'Luhu'],
                    [b'lcs', b'Lisabata-Nuniali'],
                    [b'lda', b'Kla-Dan'],
                    [b'ldb', 'Du\u0303ya'],
                    [b'ldd', b'Luri'],
                    [b'ldg', b'Lenyima'],
                    [b'ldh', b'Lamja-Dengsa-Tola'],
                    [b'ldi', b'Laari'],
                    [b'ldj', b'Lemoro'],
                    [b'ldk', b'Leelau'],
                    [b'ldl', b'Kaan'],
                    [b'ldm', b'Landoma'],
                    [b'ldn', 'L\xe1adan'],
                    [b'ldo', b'Loo'],
                    [b'ldp', b'Tso'],
                    [b'ldq', b'Lufu'],
                    [b'lea', b'Lega-Shabunda'],
                    [b'leb', b'Lala-Bisa'],
                    [b'lec', b'Leco'],
                    [b'led', b'Lendu'],
                    [b'lee', 'Ly\xe9l\xe9'],
                    [b'lef', b'Lelemi'],
                    [b'leg', b'Lengua'],
                    [b'leh', b'Lenje'],
                    [b'lei', b'Lemio'],
                    [b'lej', b'Lengola'],
                    [b'lek', b'Leipon'],
                    [b'lel', b'Lele (Democratic Republic of Congo)'],
                    [b'lem', b'Nomaande'],
                    [b'len', b'Lenca'],
                    [b'leo', b'Leti (Cameroon)'],
                    [b'lep', b'Lepcha'],
                    [b'leq', b'Lembena'],
                    [b'ler', b'Lenkau'],
                    [b'les', b'Lese'],
                    [b'let', b'Lesing-Gelimi'],
                    [b'leu', b'Kara (Papua New Guinea)'],
                    [b'lev', b'Lamma'],
                    [b'lew', b'Kaili, Ledo'],
                    [b'lex', b'Luang'],
                    [b'ley', b'Lemolang'],
                    [b'lez', b'Lezghian'],
                    [b'lfa', b'Lefa'],
                    [b'lfn', b'Lingua Franca Nova'],
                    [b'lga', b'Lungga'],
                    [b'lgb', b'Laghu'],
                    [b'lgg', b'Lugbara'],
                    [b'lgh', b'Laghuu'],
                    [b'lgi', b'Lengilu'],
                    [b'lgk', b'Lingarak'],
                    [b'lgl', b'Wala'],
                    [b'lgm', b'Lega-Mwenga'],
                    [b'lgn', b'Opuuo'],
                    [b'lgq', b'Logba'],
                    [b'lgr', b'Lengo'],
                    [b'lgt', b'Pahi'],
                    [b'lgu', b'Longgu'],
                    [b'lgz', b'Ligenza'],
                    [b'lha', b'Laha (Viet Nam)'],
                    [b'lhh', b'Laha (Indonesia)'],
                    [b'lhi', b'Lahu Shi'],
                    [b'lhl', b'Lohar, Lahul'],
                    [b'lhm', b'Lhomi'],
                    [b'lhn', b'Lahanan'],
                    [b'lhp', b'Lhokpu'],
                    [b'lhs', 'Mlahs\xf6'],
                    [b'lht', b'Lo-Toga'],
                    [b'lhu', b'Lahu'],
                    [b'lia', b'Limba, West-Central'],
                    [b'lib', b'Likum'],
                    [b'lic', b'Hlai'],
                    [b'lid', b'Nyindrou'],
                    [b'lie', b'Likila'],
                    [b'lif', b'Limbu'],
                    [b'lig', b'Ligbi'],
                    [b'lih', b'Lihir'],
                    [b'lii', b'Lingkhim'],
                    [b'lij', b'Ligurian'],
                    [b'lik', b'Lika'],
                    [b'lil', b'Lillooet'],
                    [b'lim', b'Limburgan'],
                    [b'lin', b'Lingala'],
                    [b'lio', b'Liki'],
                    [b'lip', b'Sekpele'],
                    [b'liq', b'Libido'],
                    [b'lir', b'English, Liberian'],
                    [b'lis', b'Lisu'],
                    [b'lit', b'Lithuanian'],
                    [b'liu', b'Logorik'],
                    [b'liv', b'Liv'],
                    [b'liw', b'Col'],
                    [b'lix', b'Liabuku'],
                    [b'liy', b'Banda-Bambari'],
                    [b'liz', b'Libinza'],
                    [b'lja', b'Golpa'],
                    [b'lje', b'Rampi'],
                    [b'lji', b'Laiyolo'],
                    [b'ljl', b"Li'o"],
                    [b'ljp', b'Lampung Api'],
                    [b'ljw', b'Yirandali'],
                    [b'ljx', b'Yuru'],
                    [b'lka', b'Lakalei'],
                    [b'lkb', b'Kabras'],
                    [b'lkc', b'Kucong'],
                    [b'lkd', 'Lakond\xea'],
                    [b'lke', b'Kenyi'],
                    [b'lkh', b'Lakha'],
                    [b'lki', b'Laki'],
                    [b'lkj', b'Remun'],
                    [b'lkl', b'Laeko-Libuat'],
                    [b'lkm', b'Kalaamaya'],
                    [b'lkn', b'Lakon'],
                    [b'lko', b'Khayo'],
                    [b'lkr', 'P\xe4ri'],
                    [b'lks', b'Kisa'],
                    [b'lkt', b'Lakota'],
                    [b'lku', b'Kungkari'],
                    [b'lky', b'Lokoya'],
                    [b'lla', b'Lala-Roba'],
                    [b'llb', b'Lolo'],
                    [b'llc', b'Lele (Guinea)'],
                    [b'lld', b'Ladin'],
                    [b'lle', b'Lele (Papua New Guinea)'],
                    [b'llf', b'Hermit'],
                    [b'llg', b'Lole'],
                    [b'llh', b'Lamu'],
                    [b'lli', b'Teke-Laali'],
                    [b'llj', b'Ladji Ladji'],
                    [b'llk', b'Lelak'],
                    [b'lll', b'Lilau'],
                    [b'llm', b'Lasalimu'],
                    [b'lln', b'Lele (Chad)'],
                    [b'llo', b'Khlor'],
                    [b'llp', b'Efate, North'],
                    [b'llq', b'Lolak'],
                    [b'lls', b'Lithuanian Sign Language'],
                    [b'llu', b'Lau'],
                    [b'llx', b'Lauan'],
                    [b'lma', b'Limba, East'],
                    [b'lmb', b'Merei'],
                    [b'lmc', b'Limilngan'],
                    [b'lmd', b'Lumun'],
                    [b'lme', 'P\xe9v\xe9'],
                    [b'lmf', b'Lembata, South'],
                    [b'lmg', b'Lamogai'],
                    [b'lmh', b'Lambichhong'],
                    [b'lmi', b'Lombi'],
                    [b'lmj', b'Lembata, West'],
                    [b'lmk', b'Lamkang'],
                    [b'lml', b'Hano'],
                    [b'lmm', b'Lamam'],
                    [b'lmn', b'Lambadi'],
                    [b'lmo', b'Lombard'],
                    [b'lmp', b'Limbum'],
                    [b'lmq', b'Lamatuka'],
                    [b'lmr', b'Lamalera'],
                    [b'lmu', b'Lamenu'],
                    [b'lmv', b'Lomaiviti'],
                    [b'lmw', b'Miwok, Lake'],
                    [b'lmx', b'Laimbue'],
                    [b'lmy', b'Lamboya'],
                    [b'lmz', b'Lumbee'],
                    [b'lna', b'Langbashe'],
                    [b'lnb', b'Mbalanhu'],
                    [b'lnd', b'Lundayeh'],
                    [b'lng', b'Langobardic'],
                    [b'lnh', b'Lanoh'],
                    [b'lni', b"Daantanai'"],
                    [b'lnj', b'Leningitij'],
                    [b'lnl', b'Banda, South Central'],
                    [b'lnm', b'Langam'],
                    [b'lnn', b'Lorediakarkar'],
                    [b'lno', b'Lango (Sudan)'],
                    [b'lns', b"Lamnso'"],
                    [b'lnu', b'Longuda'],
                    [b'lnw', b'Lanima'],
                    [b'lnz', b'Lonzo'],
                    [b'loa', b'Loloda'],
                    [b'lob', b'Lobi'],
                    [b'loc', b'Inonhan'],
                    [b'loe', b'Saluan'],
                    [b'lof', b'Logol'],
                    [b'log', b'Logo'],
                    [b'loh', b'Narim'],
                    [b'loi', "Loma (C\xf4te d'Ivoire)"],
                    [b'loj', b'Lou'],
                    [b'lok', b'Loko'],
                    [b'lol', b'Mongo'],
                    [b'lom', b'Loma (Liberia)'],
                    [b'lon', b'Lomwe, Malawi'],
                    [b'loo', b'Lombo'],
                    [b'lop', b'Lopa'],
                    [b'loq', b'Lobala'],
                    [b'lor', 'T\xe9\xe9n'],
                    [b'los', b'Loniu'],
                    [b'lot', b'Otuho'],
                    [b'lou', b'Creole French, Louisiana'],
                    [b'lov', b'Lopi'],
                    [b'low', b'Lobu, Tampias'],
                    [b'lox', b'Loun'],
                    [b'loy', b'Loke'],
                    [b'loz', b'Lozi'],
                    [b'lpa', b'Lelepa'],
                    [b'lpe', b'Lepki'],
                    [b'lpn', b'Naga, Long Phuri'],
                    [b'lpo', b'Lipo'],
                    [b'lpx', b'Lopit'],
                    [b'lra', b"Rara Bakati'"],
                    [b'lrc', b'Luri, Northern'],
                    [b'lre', b'Laurentian'],
                    [b'lrg', b'Laragia'],
                    [b'lri', b'Marachi'],
                    [b'lrk', b'Loarki'],
                    [b'lrl', b'Lari'],
                    [b'lrm', b'Marama'],
                    [b'lrn', b'Lorang'],
                    [b'lro', b'Laro'],
                    [b'lrr', b'Yamphu, Southern'],
                    [b'lrt', b'Malay, Larantuka'],
                    [b'lrv', b'Larevat'],
                    [b'lrz', b'Lemerig'],
                    [b'lsa', b'Lasgerdi'],
                    [b'lsd', b'Lishana Deni'],
                    [b'lse', b'Lusengo'],
                    [b'lsg', b'Lyons Sign Language'],
                    [b'lsh', b'Lish'],
                    [b'lsi', b'Lashi'],
                    [b'lsl', b'Latvian Sign Language'],
                    [b'lsm', b'Saamia'],
                    [b'lso', b'Laos Sign Language'],
                    [b'lsp', b'Panamanian Sign Language'],
                    [b'lsr', b'Aruop'],
                    [b'lss', b'Lasi'],
                    [b'lst', b'Trinidad and Tobago Sign Language'],
                    [b'lsy', b'Mauritian Sign Language'],
                    [b'ltc', b'Chinese, Late Middle'],
                    [b'ltg', b'Latgalian'],
                    [b'lti', b'Leti (Indonesia)'],
                    [b'ltn', 'Latund\xea'],
                    [b'lto', b'Tsotso'],
                    [b'lts', b'Tachoni'],
                    [b'ltu', b'Latu'],
                    [b'ltz', b'Luxembourgish'],
                    [b'lua', b'Luba-Lulua'],
                    [b'lub', b'Luba-Katanga'],
                    [b'luc', b'Aringa'],
                    [b'lud', b'Ludian'],
                    [b'lue', b'Luvale'],
                    [b'luf', b'Laua'],
                    [b'lug', b'Ganda'],
                    [b'lui', b'Luiseno'],
                    [b'luj', b'Luna'],
                    [b'luk', b'Lunanakha'],
                    [b'lul', b"Olu'bo"],
                    [b'lum', b'Luimbi'],
                    [b'lun', b'Lunda'],
                    [b'luo', b'Luo (Kenya and Tanzania)'],
                    [b'lup', b'Lumbu'],
                    [b'luq', b'Lucumi'],
                    [b'lur', b'Laura'],
                    [b'lus', b'Lushai'],
                    [b'lut', b'Lushootseed'],
                    [b'luu', b'Lumba-Yakkha'],
                    [b'luv', b'Luwati'],
                    [b'luw', b'Luo (Cameroon)'],
                    [b'luy', b'Luyia'],
                    [b'luz', b'Luri, Southern'],
                    [b'lva', b"Maku'a"],
                    [b'lvk', b'Lavukaleve'],
                    [b'lvs', b'Latvian, Standard'],
                    [b'lvu', b'Levuka'],
                    [b'lwa', b'Lwalu'],
                    [b'lwe', b'Lewo Eleng'],
                    [b'lwg', b'Wanga'],
                    [b'lwh', b'Lachi, White'],
                    [b'lwl', b'Lawa, Eastern'],
                    [b'lwm', b'Laomian'],
                    [b'lwo', b'Luwo'],
                    [b'lwt', b'Lewotobi'],
                    [b'lwu', b'Lawu'],
                    [b'lww', b'Lewo'],
                    [b'lya', b'Layakha'],
                    [b'lyg', b'Lyngngam'],
                    [b'lyn', b'Luyana'],
                    [b'lzh', b'Chinese, Literary'],
                    [b'lzl', b'Litzlitz'],
                    [b'lzn', b'Naga, Leinong'],
                    [b'lzz', b'Laz'],
                    [b'maa', 'Mazatec, San Jer\xf3nimo Tec\xf3atl'],
                    [b'mab', b'Mixtec, Yutanduchi'],
                    [b'mad', b'Madurese'],
                    [b'mae', b'Bo-Rukul'],
                    [b'maf', b'Mafa'],
                    [b'mag', b'Magahi'],
                    [b'mah', b'Marshallese'],
                    [b'mai', b'Maithili'],
                    [b'maj', 'Mazatec, Jalapa De D\xedaz'],
                    [b'mak', b'Makasar'],
                    [b'mal', b'Malayalam'],
                    [b'mam', b'Mam'],
                    [b'man', b'Mandingo'],
                    [b'maq', 'Mazatec, Chiquihuitl\xe1n'],
                    [b'mar', b'Marathi'],
                    [b'mas', b'Masai'],
                    [b'mat', b'Matlatzinca, San Francisco'],
                    [b'mau', b'Mazatec, Huautla'],
                    [b'mav', 'Sater\xe9-Maw\xe9'],
                    [b'maw', b'Mampruli'],
                    [b'max', b'Malay, North Moluccan'],
                    [b'maz', b'Mazahua, Central'],
                    [b'mba', b'Higaonon'],
                    [b'mbb', b'Manobo, Western Bukidnon'],
                    [b'mbc', b'Macushi'],
                    [b'mbd', b'Manobo, Dibabawon'],
                    [b'mbe', b'Molale'],
                    [b'mbf', b'Malay, Baba'],
                    [b'mbh', b'Mangseng'],
                    [b'mbi', b'Manobo, Ilianen'],
                    [b'mbj', 'Nad\xebb'],
                    [b'mbk', b'Malol'],
                    [b'mbl', 'Maxakal\xed'],
                    [b'mbm', b'Ombamba'],
                    [b'mbn', 'Macagu\xe1n'],
                    [b'mbo', b'Mbo (Cameroon)'],
                    [b'mbp', b'Malayo'],
                    [b'mbq', b'Maisin'],
                    [b'mbr', 'Nukak Mak\xfa'],
                    [b'mbs', b'Manobo, Sarangani'],
                    [b'mbt', b'Manobo, Matigsalug'],
                    [b'mbu', b'Mbula-Bwazza'],
                    [b'mbv', b'Mbulungish'],
                    [b'mbw', b'Maring'],
                    [b'mbx', b'Mari (East Sepik Province)'],
                    [b'mby', b'Memoni'],
                    [b'mbz', b'Mixtec, Amoltepec'],
                    [b'mca', b'Maca'],
                    [b'mcb', b'Machiguenga'],
                    [b'mcc', b'Bitur'],
                    [b'mcd', b'Sharanahua'],
                    [b'mce', b'Mixtec, Itundujia'],
                    [b'mcf', 'Mats\xe9s'],
                    [b'mcg', b'Mapoyo'],
                    [b'mch', b'Maquiritari'],
                    [b'mci', b'Mese'],
                    [b'mcj', b'Mvanip'],
                    [b'mck', b'Mbunda'],
                    [b'mcl', b'Macaguaje'],
                    [b'mcm', b'Creole Portuguese, Malaccan'],
                    [b'mcn', b'Masana'],
                    [b'mco', 'Mixe, Coatl\xe1n'],
                    [b'mcp', b'Makaa'],
                    [b'mcq', b'Ese'],
                    [b'mcr', b'Menya'],
                    [b'mcs', b'Mambai'],
                    [b'mct', b'Mengisa'],
                    [b'mcu', b'Mambila, Cameroon'],
                    [b'mcv', b'Minanibai'],
                    [b'mcw', b'Mawa (Chad)'],
                    [b'mcx', b'Mpiemo'],
                    [b'mcy', b'Watut, South'],
                    [b'mcz', b'Mawan'],
                    [b'mda', b'Mada (Nigeria)'],
                    [b'mdb', b'Morigi'],
                    [b'mdc', b'Male (Papua New Guinea)'],
                    [b'mdd', b'Mbum'],
                    [b'mde', b'Maba (Chad)'],
                    [b'mdf', b'Moksha'],
                    [b'mdg', b'Massalat'],
                    [b'mdh', b'Maguindanaon'],
                    [b'mdi', b'Mamvu'],
                    [b'mdj', b'Mangbetu'],
                    [b'mdk', b'Mangbutu'],
                    [b'mdl', b'Maltese Sign Language'],
                    [b'mdm', b'Mayogo'],
                    [b'mdn', b'Mbati'],
                    [b'mdp', b'Mbala'],
                    [b'mdq', b'Mbole'],
                    [b'mdr', b'Mandar'],
                    [b'mds', b'Maria (Papua New Guinea)'],
                    [b'mdt', b'Mbere'],
                    [b'mdu', b'Mboko'],
                    [b'mdv', 'Mixtec, Santa Luc\xeda Monteverde'],
                    [b'mdw', b'Mbosi'],
                    [b'mdx', b'Dizin'],
                    [b'mdy', b'Male (Ethiopia)'],
                    [b'mdz', 'Suru\xed Do Par\xe1'],
                    [b'mea', b'Menka'],
                    [b'meb', b'Ikobi'],
                    [b'mec', b'Mara'],
                    [b'med', b'Melpa'],
                    [b'mee', b'Mengen'],
                    [b'mef', b'Megam'],
                    [b'meh', b'Mixtec, Southwestern Tlaxiaco'],
                    [b'mei', b'Midob'],
                    [b'mej', b'Meyah'],
                    [b'mek', b'Mekeo'],
                    [b'mel', b'Melanau, Central'],
                    [b'mem', b'Mangala'],
                    [b'men', b'Mende (Sierra Leone)'],
                    [b'meo', b'Malay, Kedah'],
                    [b'mep', b'Miriwung'],
                    [b'meq', b'Merey'],
                    [b'mer', b'Meru'],
                    [b'mes', b'Masmaje'],
                    [b'met', b'Mato'],
                    [b'meu', b'Motu'],
                    [b'mev', b'Mano'],
                    [b'mew', b'Maaka'],
                    [b'mey', b'Hassaniyya'],
                    [b'mez', b'Menominee'],
                    [b'mfa', b'Malay, Pattani'],
                    [b'mfb', b'Bangka'],
                    [b'mfc', b'Mba'],
                    [b'mfd', b'Mendankwe-Nkwen'],
                    [b'mfe', b'Morisyen'],
                    [b'mff', b'Naki'],
                    [b'mfg', b'Mogofin'],
                    [b'mfh', b'Matal'],
                    [b'mfi', b'Wandala'],
                    [b'mfj', b'Mefele'],
                    [b'mfk', b'Mofu, North'],
                    [b'mfl', b'Putai'],
                    [b'mfm', b'Marghi South'],
                    [b'mfn', b'Mbembe, Cross River'],
                    [b'mfo', b'Mbe'],
                    [b'mfp', b'Malay, Makassar'],
                    [b'mfq', b'Moba'],
                    [b'mfr', b'Marithiel'],
                    [b'mfs', b'Mexican Sign Language'],
                    [b'mft', b'Mokerang'],
                    [b'mfu', b'Mbwela'],
                    [b'mfv', b'Mandjak'],
                    [b'mfw', b'Mulaha'],
                    [b'mfx', b'Melo'],
                    [b'mfy', b'Mayo'],
                    [b'mfz', b'Mabaan'],
                    [b'mga', b'Irish, Middle (900-1200)'],
                    [b'mgb', b'Mararit'],
                    [b'mgc', b'Morokodo'],
                    [b'mgd', b'Moru'],
                    [b'mge', b'Mango'],
                    [b'mgf', b'Maklew'],
                    [b'mgg', b'Mpumpong'],
                    [b'mgh', b'Makhuwa-Meetto'],
                    [b'mgi', b'Lijili'],
                    [b'mgj', b'Abureni'],
                    [b'mgk', b'Mawes'],
                    [b'mgl', b'Maleu-Kilenge'],
                    [b'mgm', b'Mambae'],
                    [b'mgn', b'Mbangi'],
                    [b'mgo', b"Meta'"],
                    [b'mgp', b'Magar, Eastern'],
                    [b'mgq', b'Malila'],
                    [b'mgr', b'Mambwe-Lungu'],
                    [b'mgs', b'Manda (Tanzania)'],
                    [b'mgt', b'Mongol'],
                    [b'mgu', b'Mailu'],
                    [b'mgv', b'Matengo'],
                    [b'mgw', b'Matumbi'],
                    [b'mgy', b'Mbunga'],
                    [b'mgz', b'Mbugwe'],
                    [b'mha', b'Manda (India)'],
                    [b'mhb', b'Mahongwe'],
                    [b'mhc', b'Mocho'],
                    [b'mhd', b'Mbugu'],
                    [b'mhe', b'Besisi'],
                    [b'mhf', b'Mamaa'],
                    [b'mhg', b'Margu'],
                    [b'mhh', b'Maskoy Pidgin'],
                    [b'mhi', b"Ma'di"],
                    [b'mhj', b'Mogholi'],
                    [b'mhk', b'Mungaka'],
                    [b'mhl', b'Mauwake'],
                    [b'mhm', b'Makhuwa-Moniga'],
                    [b'mhn', 'M\xf3cheno'],
                    [b'mho', b'Mashi (Zambia)'],
                    [b'mhp', b'Malay, Balinese'],
                    [b'mhq', b'Mandan'],
                    [b'mhr', b'Mari, Eastern'],
                    [b'mhs', b'Buru (Indonesia)'],
                    [b'mht', b'Mandahuaca'],
                    [b'mhu', b'Digaro-Mishmi'],
                    [b'mhw', b'Mbukushu'],
                    [b'mhx', b'Maru'],
                    [b'mhy', b"Ma'anyan"],
                    [b'mhz', b'Mor (Mor Islands)'],
                    [b'mia', b'Miami'],
                    [b'mib', 'Mixtec, Atatl\xe1huca'],
                    [b'mic', b"Mi'kmaq"],
                    [b'mid', b'Mandaic'],
                    [b'mie', b'Mixtec, Ocotepec'],
                    [b'mif', b'Mofu-Gudur'],
                    [b'mig', b'Mixtec, San Miguel El Grande'],
                    [b'mih', b'Mixtec, Chayuco'],
                    [b'mii', 'Mixtec, Chigmecatitl\xe1n'],
                    [b'mij', b'Abar'],
                    [b'mik', b'Mikasuki'],
                    [b'mil', 'Mixtec, Pe\xf1oles'],
                    [b'mim', b'Mixtec, Alacatlatzala'],
                    [b'min', b'Minangkabau'],
                    [b'mio', b'Mixtec, Pinotepa Nacional'],
                    [b'mip', b'Mixtec, Apasco-Apoala'],
                    [b'miq', 'M\xedskito'],
                    [b'mir', b'Mixe, Isthmus'],
                    [b'mis', b'Uncoded languages'],
                    [b'mit', b'Mixtec, Southern Puebla'],
                    [b'miu', b'Mixtec, Cacaloxtepec'],
                    [b'miw', b'Akoye'],
                    [b'mix', b'Mixtec, Mixtepec'],
                    [b'miy', b'Mixtec, Ayutla'],
                    [b'miz', b'Mixtec, Coatzospan'],
                    [b'mjc', b'Mixtec, San Juan Colorado'],
                    [b'mjd', b'Maidu, Northwest'],
                    [b'mje', b'Muskum'],
                    [b'mjg', b'Tu'],
                    [b'mjh', b'Mwera (Nyasa)'],
                    [b'mji', b'Kim Mun'],
                    [b'mjj', b'Mawak'],
                    [b'mjk', b'Matukar'],
                    [b'mjl', b'Mandeali'],
                    [b'mjm', b'Medebur'],
                    [b'mjn', b'Ma (Papua New Guinea)'],
                    [b'mjo', b'Malankuravan'],
                    [b'mjp', b'Malapandaram'],
                    [b'mjq', b'Malaryan'],
                    [b'mjr', b'Malavedan'],
                    [b'mjs', b'Miship'],
                    [b'mjt', b'Sauria Paharia'],
                    [b'mju', b'Manna-Dora'],
                    [b'mjv', b'Mannan'],
                    [b'mjw', b'Karbi'],
                    [b'mjx', b'Mahali'],
                    [b'mjy', b'Mahican'],
                    [b'mjz', b'Majhi'],
                    [b'mka', b'Mbre'],
                    [b'mkb', b'Mal Paharia'],
                    [b'mkc', b'Siliput'],
                    [b'mkd', b'Macedonian'],
                    [b'mke', b'Mawchi'],
                    [b'mkf', b'Miya'],
                    [b'mkg', b'Mak (China)'],
                    [b'mki', b'Dhatki'],
                    [b'mkj', b'Mokilese'],
                    [b'mkk', b'Byep'],
                    [b'mkl', b'Mokole'],
                    [b'mkm', b'Moklen'],
                    [b'mkn', b'Malay, Kupang'],
                    [b'mko', b'Mingang Doso'],
                    [b'mkp', b'Moikodi'],
                    [b'mkq', b'Miwok, Bay'],
                    [b'mkr', b'Malas'],
                    [b'mks', b'Mixtec, Silacayoapan'],
                    [b'mkt', b'Vamale'],
                    [b'mku', b'Maninka, Konyanka'],
                    [b'mkv', b'Mafea'],
                    [b'mkw', b'Kituba (Congo)'],
                    [b'mkx', b'Manobo, Kinamiging'],
                    [b'mky', b'Makian, East'],
                    [b'mkz', b'Makasae'],
                    [b'mla', b'Malo'],
                    [b'mlb', b'Mbule'],
                    [b'mlc', b'Cao Lan'],
                    [b'mle', b'Manambu'],
                    [b'mlf', b'Mal'],
                    [b'mlg', b'Malagasy'],
                    [b'mlh', b'Mape'],
                    [b'mli', b'Malimpung'],
                    [b'mlj', b'Miltu'],
                    [b'mlk', b'Ilwana'],
                    [b'mll', b'Malua Bay'],
                    [b'mlm', b'Mulam'],
                    [b'mln', b'Malango'],
                    [b'mlo', b'Mlomp'],
                    [b'mlp', b'Bargam'],
                    [b'mlq', b'Maninkakan, Western'],
                    [b'mlr', b'Vame'],
                    [b'mls', b'Masalit'],
                    [b'mlt', b'Maltese'],
                    [b'mlu', b"To'abaita"],
                    [b'mlv', b'Motlav'],
                    [b'mlw', b'Moloko'],
                    [b'mlx', b'Malfaxal'],
                    [b'mlz', b'Malaynon'],
                    [b'mma', b'Mama'],
                    [b'mmb', b'Momina'],
                    [b'mmc', 'Mazahua, Michoac\xe1n'],
                    [b'mmd', b'Maonan'],
                    [b'mme', b'Mae'],
                    [b'mmf', b'Mundat'],
                    [b'mmg', b'Ambrym, North'],
                    [b'mmh', 'Mehin\xe1ku'],
                    [b'mmi', b'Musar'],
                    [b'mmj', b'Majhwar'],
                    [b'mmk', b'Mukha-Dora'],
                    [b'mml', b'Man Met'],
                    [b'mmm', b'Maii'],
                    [b'mmn', b'Mamanwa'],
                    [b'mmo', b'Buang, Mangga'],
                    [b'mmp', b'Siawi'],
                    [b'mmq', b'Musak'],
                    [b'mmr', b'Miao, Western Xiangxi'],
                    [b'mmt', b'Malalamai'],
                    [b'mmu', b'Mmaala'],
                    [b'mmv', b'Miriti'],
                    [b'mmw', b'Emae'],
                    [b'mmx', b'Madak'],
                    [b'mmy', b'Migaama'],
                    [b'mmz', b'Mabaale'],
                    [b'mna', b'Mbula'],
                    [b'mnb', b'Muna'],
                    [b'mnc', b'Manchu'],
                    [b'mnd', 'Mond\xe9'],
                    [b'mne', b'Naba'],
                    [b'mnf', b'Mundani'],
                    [b'mng', b'Mnong, Eastern'],
                    [b'mnh', b'Mono (Democratic Republic of Congo)'],
                    [b'mni', b'Manipuri'],
                    [b'mnj', b'Munji'],
                    [b'mnk', b'Mandinka'],
                    [b'mnl', b'Tiale'],
                    [b'mnm', b'Mapena'],
                    [b'mnn', b'Mnong, Southern'],
                    [b'mnp', b'Chinese, Min Bei'],
                    [b'mnq', b'Minriq'],
                    [b'mnr', b'Mono (USA)'],
                    [b'mns', b'Mansi'],
                    [b'mnu', b'Mer'],
                    [b'mnv', b'Rennell-Bellona'],
                    [b'mnw', b'Mon'],
                    [b'mnx', b'Manikion'],
                    [b'mny', b'Manyawa'],
                    [b'mnz', b'Moni'],
                    [b'moa', b'Mwan'],
                    [b'moc', 'Mocov\xed'],
                    [b'mod', b'Mobilian'],
                    [b'moe', b'Montagnais'],
                    [b'mog', b'Mongondow'],
                    [b'moh', b'Mohawk'],
                    [b'moi', b'Mboi'],
                    [b'moj', b'Monzombo'],
                    [b'mok', b'Morori'],
                    [b'mom', b'Mangue'],
                    [b'mon', b'Mongolian'],
                    [b'moo', b'Monom'],
                    [b'mop', 'Mop\xe1n Maya'],
                    [b'moq', b'Mor (Bomberai Peninsula)'],
                    [b'mor', b'Moro'],
                    [b'mos', b'Mossi'],
                    [b'mot', 'Bar\xed'],
                    [b'mou', b'Mogum'],
                    [b'mov', b'Mohave'],
                    [b'mow', b'Moi (Congo)'],
                    [b'mox', b'Molima'],
                    [b'moy', b'Shekkacho'],
                    [b'moz', b'Mukulu'],
                    [b'mpa', b'Mpoto'],
                    [b'mpb', b'Mullukmulluk'],
                    [b'mpc', b'Mangarayi'],
                    [b'mpd', b'Machinere'],
                    [b'mpe', b'Majang'],
                    [b'mpg', b'Marba'],
                    [b'mph', b'Maung'],
                    [b'mpi', b'Mpade'],
                    [b'mpj', b'Martu Wangka'],
                    [b'mpk', b'Mbara (Chad)'],
                    [b'mpl', b'Watut, Middle'],
                    [b'mpm', 'Mixtec, Yosond\xfaa'],
                    [b'mpn', b'Mindiri'],
                    [b'mpo', b'Miu'],
                    [b'mpp', b'Migabac'],
                    [b'mpq', 'Mat\xeds'],
                    [b'mpr', b'Vangunu'],
                    [b'mps', b'Dadibi'],
                    [b'mpt', b'Mian'],
                    [b'mpu', 'Makur\xe1p'],
                    [b'mpv', b'Mungkip'],
                    [b'mpw', b'Mapidian'],
                    [b'mpx', b'Misima-Panaeati'],
                    [b'mpy', b'Mapia'],
                    [b'mpz', b'Mpi'],
                    [b'mqa', b'Maba (Indonesia)'],
                    [b'mqb', b'Mbuko'],
                    [b'mqc', b'Mangole'],
                    [b'mqe', b'Matepi'],
                    [b'mqf', b'Momuna'],
                    [b'mqg', b'Malay, Kota Bangun Kutai'],
                    [b'mqh', b'Mixtec, Tlazoyaltepec'],
                    [b'mqi', b'Mariri'],
                    [b'mqj', b'Mamasa'],
                    [b'mqk', b'Manobo, Rajah Kabunsuwan'],
                    [b'mql', b'Mbelime'],
                    [b'mqm', b'Marquesan, South'],
                    [b'mqn', b'Moronene'],
                    [b'mqo', b'Modole'],
                    [b'mqp', b'Manipa'],
                    [b'mqq', b'Minokok'],
                    [b'mqr', b'Mander'],
                    [b'mqs', b'Makian, West'],
                    [b'mqt', b'Mok'],
                    [b'mqu', b'Mandari'],
                    [b'mqv', b'Mosimo'],
                    [b'mqw', b'Murupi'],
                    [b'mqx', b'Mamuju'],
                    [b'mqy', b'Manggarai'],
                    [b'mqz', b'Pano'],
                    [b'mra', b'Mlabri'],
                    [b'mrb', b'Marino'],
                    [b'mrc', b'Maricopa'],
                    [b'mrd', b'Magar, Western'],
                    [b'mre', b"Martha's Vineyard Sign Language"],
                    [b'mrf', b'Elseng'],
                    [b'mrg', b'Mising'],
                    [b'mrh', b'Chin, Mara'],
                    [b'mri', b'Maori'],
                    [b'mrj', b'Mari, Western'],
                    [b'mrk', b'Hmwaveke'],
                    [b'mrl', b'Mortlockese'],
                    [b'mrm', b'Merlav'],
                    [b'mrn', b'Cheke Holo'],
                    [b'mro', b'Mru'],
                    [b'mrp', b'Morouas'],
                    [b'mrq', b'Marquesan, North'],
                    [b'mrr', b'Maria (India)'],
                    [b'mrs', b'Maragus'],
                    [b'mrt', b'Marghi Central'],
                    [b'mru', b'Mono (Cameroon)'],
                    [b'mrv', b'Mangareva'],
                    [b'mrw', b'Maranao'],
                    [b'mrx', b'Maremgi'],
                    [b'mry', b'Mandaya'],
                    [b'mrz', b'Marind'],
                    [b'msa', b'Malay (macrolanguage)'],
                    [b'msb', b'Masbatenyo'],
                    [b'msc', b'Maninka, Sankaran'],
                    [b'msd', b'Yucatec Maya Sign Language'],
                    [b'mse', b'Musey'],
                    [b'msf', b'Mekwei'],
                    [b'msg', b'Moraid'],
                    [b'msh', b'Malagasy, Masikoro'],
                    [b'msi', b'Malay, Sabah'],
                    [b'msj', b'Ma (Democratic Republic of Congo)'],
                    [b'msk', b'Mansaka'],
                    [b'msl', b'Molof'],
                    [b'msm', b'Manobo, Agusan'],
                    [b'msn', 'Vur\xebs'],
                    [b'mso', b'Mombum'],
                    [b'msp', 'Maritsau\xe1'],
                    [b'msq', b'Caac'],
                    [b'msr', b'Mongolian Sign Language'],
                    [b'mss', b'Masela, West'],
                    [b'msu', b'Musom'],
                    [b'msv', b'Maslam'],
                    [b'msw', b'Mansoanka'],
                    [b'msx', b'Moresada'],
                    [b'msy', b'Aruamu'],
                    [b'msz', b'Momare'],
                    [b'mta', b'Manobo, Cotabato'],
                    [b'mtb', b'Anyin Morofo'],
                    [b'mtc', b'Munit'],
                    [b'mtd', b'Mualang'],
                    [b'mte', b'Mono (Solomon Islands)'],
                    [b'mtf', b'Murik (Papua New Guinea)'],
                    [b'mtg', b'Una'],
                    [b'mth', b'Munggui'],
                    [b'mti', b'Maiwa (Papua New Guinea)'],
                    [b'mtj', b'Moskona'],
                    [b'mtk', b"Mbe'"],
                    [b'mtl', b'Montol'],
                    [b'mtm', b'Mator'],
                    [b'mtn', b'Matagalpa'],
                    [b'mto', b'Mixe, Totontepec'],
                    [b'mtp', 'Wich\xed Lhamt\xe9s Nocten'],
                    [b'mtq', b'Muong'],
                    [b'mtr', b'Mewari'],
                    [b'mts', b'Yora'],
                    [b'mtt', b'Mota'],
                    [b'mtu', b'Mixtec, Tututepec'],
                    [b'mtv', b"Asaro'o"],
                    [b'mtw', b'Binukidnon, Southern'],
                    [b'mtx', 'Mixtec, Tida\xe1'],
                    [b'mty', b'Nabi'],
                    [b'mua', b'Mundang'],
                    [b'mub', b'Mubi'],
                    [b'muc', b'Ajumbu'],
                    [b'mud', b'Aleut, Mednyj'],
                    [b'mue', b'Media Lengua'],
                    [b'mug', b'Musgu'],
                    [b'muh', 'M\xfcnd\xfc'],
                    [b'mui', b'Musi'],
                    [b'muj', b'Mabire'],
                    [b'muk', b'Mugom'],
                    [b'mul', b'Multiple languages'],
                    [b'mum', b'Maiwala'],
                    [b'muo', b'Nyong'],
                    [b'mup', b'Malvi'],
                    [b'muq', b'Miao, Eastern Xiangxi'],
                    [b'mur', b'Murle'],
                    [b'mus', b'Creek'],
                    [b'mut', b'Muria, Western'],
                    [b'muu', b'Yaaku'],
                    [b'muv', b'Muthuvan'],
                    [b'mux', b'Bo-Ung'],
                    [b'muy', b'Muyang'],
                    [b'muz', b'Mursi'],
                    [b'mva', b'Manam'],
                    [b'mvb', b'Mattole'],
                    [b'mvd', b'Mamboru'],
                    [b'mve', b'Marwari (Pakistan)'],
                    [b'mvf', b'Mongolian, Peripheral'],
                    [b'mvg', 'Mixtec, Yucua\xf1e'],
                    [b'mvh', b'Mulgi'],
                    [b'mvi', b'Miyako'],
                    [b'mvk', b'Mekmek'],
                    [b'mvl', b'Mbara (Australia)'],
                    [b'mvm', b'Muya'],
                    [b'mvn', b'Minaveha'],
                    [b'mvo', b'Marovo'],
                    [b'mvp', b'Duri'],
                    [b'mvq', b'Moere'],
                    [b'mvr', b'Marau'],
                    [b'mvs', b'Massep'],
                    [b'mvt', b'Mpotovoro'],
                    [b'mvu', b'Marfa'],
                    [b'mvv', b'Murut, Tagal'],
                    [b'mvw', b'Machinga'],
                    [b'mvx', b'Meoswar'],
                    [b'mvy', b'Kohistani, Indus'],
                    [b'mvz', b'Mesqan'],
                    [b'mwa', b'Mwatebu'],
                    [b'mwb', b'Juwal'],
                    [b'mwc', b'Are'],
                    [b'mwe', b'Mwera (Chimwera)'],
                    [b'mwf', b'Murrinh-Patha'],
                    [b'mwg', b'Aiklep'],
                    [b'mwh', b'Mouk-Aria'],
                    [b'mwi', b'Labo'],
                    [b'mwj', b'Maligo'],
                    [b'mwk', b'Maninkakan, Kita'],
                    [b'mwl', b'Mirandese'],
                    [b'mwm', b'Sar'],
                    [b'mwn', b'Nyamwanga'],
                    [b'mwo', b'Maewo, Central'],
                    [b'mwp', b'Kala Lagaw Ya'],
                    [b'mwq', 'Chin, M\xfcn'],
                    [b'mwr', b'Marwari'],
                    [b'mws', b'Mwimbi-Muthambi'],
                    [b'mwt', b'Moken'],
                    [b'mwu', b'Mittu'],
                    [b'mwv', b'Mentawai'],
                    [b'mww', b'Hmong Daw'],
                    [b'mwx', b'Mediak'],
                    [b'mwy', b'Mosiro'],
                    [b'mwz', b'Moingi'],
                    [b'mxa', b'Mixtec, Northwest Oaxaca'],
                    [b'mxb', 'Mixtec, Tezoatl\xe1n'],
                    [b'mxc', b'Manyika'],
                    [b'mxd', b'Modang'],
                    [b'mxe', b'Mele-Fila'],
                    [b'mxf', b'Malgbe'],
                    [b'mxg', b'Mbangala'],
                    [b'mxh', b'Mvuba'],
                    [b'mxi', b'Mozarabic'],
                    [b'mxj', b'Miju-Mishmi'],
                    [b'mxk', b'Monumbo'],
                    [b'mxl', b'Gbe, Maxi'],
                    [b'mxm', b'Meramera'],
                    [b'mxn', b'Moi (Indonesia)'],
                    [b'mxo', b'Mbowe'],
                    [b'mxp', b'Mixe, Tlahuitoltepec'],
                    [b'mxq', b'Mixe, Juquila'],
                    [b'mxr', b'Murik (Malaysia)'],
                    [b'mxs', b'Mixtec, Huitepec'],
                    [b'mxt', b'Mixtec, Jamiltepec'],
                    [b'mxu', b'Mada (Cameroon)'],
                    [b'mxv', 'Mixtec, Metlat\xf3noc'],
                    [b'mxw', b'Namo'],
                    [b'mxx', b'Mahou'],
                    [b'mxy', 'Mixtec, Southeastern Nochixtl\xe1n'],
                    [b'mxz', b'Masela, Central'],
                    [b'mya', b'Burmese'],
                    [b'myb', b'Mbay'],
                    [b'myc', b'Mayeka'],
                    [b'myd', b'Maramba'],
                    [b'mye', b'Myene'],
                    [b'myf', b'Bambassi'],
                    [b'myg', b'Manta'],
                    [b'myh', b'Makah'],
                    [b'myi', b'Mina (India)'],
                    [b'myj', b'Mangayat'],
                    [b'myk', b'Senoufo, Mamara'],
                    [b'myl', b'Moma'],
                    [b'mym', b"Me'en"],
                    [b'myo', b'Anfillo'],
                    [b'myp', 'Pirah\xe3'],
                    [b'myr', b'Muniche'],
                    [b'mys', b'Mesmes'],
                    [b'myu', 'Munduruk\xfa'],
                    [b'myv', b'Erzya'],
                    [b'myw', b'Muyuw'],
                    [b'myx', b'Masaaba'],
                    [b'myy', b'Macuna'],
                    [b'myz', b'Mandaic, Classical'],
                    [b'mza', 'Mixtec, Santa Mar\xeda Zacatepec'],
                    [b'mzb', b'Tumzabt'],
                    [b'mzc', b'Madagascar Sign Language'],
                    [b'mzd', b'Malimba'],
                    [b'mze', b'Morawa'],
                    [b'mzg', b'Monastic Sign Language'],
                    [b'mzh', 'Wich\xed Lhamt\xe9s G\xfcisnay'],
                    [b'mzi', 'Mazatec, Ixcatl\xe1n'],
                    [b'mzj', b'Manya'],
                    [b'mzk', b'Mambila, Nigeria'],
                    [b'mzl', 'Mixe, Mazatl\xe1n'],
                    [b'mzm', b'Mumuye'],
                    [b'mzn', b'Mazanderani'],
                    [b'mzo', b'Matipuhy'],
                    [b'mzp', b'Movima'],
                    [b'mzq', b'Mori Atas'],
                    [b'mzr', 'Mar\xfabo'],
                    [b'mzs', b'Macanese'],
                    [b'mzt', b'Mintil'],
                    [b'mzu', b'Inapang'],
                    [b'mzv', b'Manza'],
                    [b'mzw', b'Deg'],
                    [b'mzx', b'Mawayana'],
                    [b'mzy', b'Mozambican Sign Language'],
                    [b'mzz', b'Maiadomu'],
                    [b'naa', b'Namla'],
                    [b'nab', 'Nambiku\xe1ra, Southern'],
                    [b'nac', b'Narak'],
                    [b'nad', b'Nijadali'],
                    [b'nae', b"Naka'ela"],
                    [b'naf', b'Nabak'],
                    [b'nag', b'Naga Pidgin'],
                    [b'naj', b'Nalu'],
                    [b'nak', b'Nakanai'],
                    [b'nal', b'Nalik'],
                    [b'nam', b"Ngan'gityemerri"],
                    [b'nan', b'Chinese, Min Nan'],
                    [b'nao', b'Naaba'],
                    [b'nap', b'Neapolitan'],
                    [b'naq', b'Nama (Namibia)'],
                    [b'nar', b'Iguta'],
                    [b'nas', b'Naasioi'],
                    [b'nat', b'Hungworo'],
                    [b'nau', b'Nauru'],
                    [b'nav', b'Navajo'],
                    [b'naw', b'Nawuri'],
                    [b'nax', b'Nakwi'],
                    [b'nay', b'Narrinyeri'],
                    [b'naz', b'Nahuatl, Coatepec'],
                    [b'nba', b'Nyemba'],
                    [b'nbb', b'Ndoe'],
                    [b'nbc', b'Naga, Chang'],
                    [b'nbd', b'Ngbinda'],
                    [b'nbe', b'Naga, Konyak'],
                    [b'nbg', b'Nagarchal'],
                    [b'nbh', b'Ngamo'],
                    [b'nbi', b'Naga, Mao'],
                    [b'nbj', b'Ngarinman'],
                    [b'nbk', b'Nake'],
                    [b'nbl', b'Ndebele, South'],
                    [b'nbm', b"Ngbaka Ma'bo"],
                    [b'nbn', b'Kuri'],
                    [b'nbo', b'Nkukoli'],
                    [b'nbp', b'Nnam'],
                    [b'nbq', b'Nggem'],
                    [b'nbr', b'Numana-Nunku-Gbantu-Numbu'],
                    [b'nbs', b'Namibian Sign Language'],
                    [b'nbt', b'Na'],
                    [b'nbu', b'Naga, Rongmei'],
                    [b'nbv', b'Ngamambo'],
                    [b'nbw', b'Ngbandi, Southern'],
                    [b'nby', b'Ningera'],
                    [b'nca', b'Iyo'],
                    [b'ncb', b'Nicobarese, Central'],
                    [b'ncc', b'Ponam'],
                    [b'ncd', b'Nachering'],
                    [b'nce', b'Yale'],
                    [b'ncf', b'Notsi'],
                    [b'ncg', b"Nisga'a"],
                    [b'nch', b'Nahuatl, Central Huasteca'],
                    [b'nci', b'Nahuatl, Classical'],
                    [b'ncj', b'Nahuatl, Northern Puebla'],
                    [b'nck', b'Nakara'],
                    [b'ncl', 'Nahuatl, Michoac\xe1n'],
                    [b'ncm', b'Nambo'],
                    [b'ncn', b'Nauna'],
                    [b'nco', b'Sibe'],
                    [b'ncp', b'Ndaktup'],
                    [b'ncr', b'Ncane'],
                    [b'ncs', b'Nicaraguan Sign Language'],
                    [b'nct', b'Naga, Chothe'],
                    [b'ncu', b'Chumburung'],
                    [b'ncx', b'Nahuatl, Central Puebla'],
                    [b'ncz', b'Natchez'],
                    [b'nda', b'Ndasa'],
                    [b'ndb', b'Kenswei Nsei'],
                    [b'ndc', b'Ndau'],
                    [b'ndd', b'Nde-Nsele-Nta'],
                    [b'nde', b'Ndebele, North'],
                    [b'ndf', b'Nadruvian'],
                    [b'ndg', b'Ndengereko'],
                    [b'ndh', b'Ndali'],
                    [b'ndi', b'Samba Leko'],
                    [b'ndj', b'Ndamba'],
                    [b'ndk', b'Ndaka'],
                    [b'ndl', b'Ndolo'],
                    [b'ndm', b'Ndam'],
                    [b'ndn', b'Ngundi'],
                    [b'ndo', b'Ndonga'],
                    [b'ndp', b'Ndo'],
                    [b'ndq', b'Ndombe'],
                    [b'ndr', b'Ndoola'],
                    [b'nds', b'German, Low'],
                    [b'ndt', b'Ndunga'],
                    [b'ndu', b'Dugun'],
                    [b'ndv', b'Ndut'],
                    [b'ndw', b'Ndobo'],
                    [b'ndx', b'Nduga'],
                    [b'ndy', b'Lutos'],
                    [b'ndz', b'Ndogo'],
                    [b'nea', b"Ngad'a, Eastern"],
                    [b'neb', "Toura (C\xf4te d'Ivoire)"],
                    [b'nec', b'Nedebang'],
                    [b'ned', b'Nde-Gbite'],
                    [b'nee', 'N\xeal\xeamwa-Nixumwak'],
                    [b'nef', b'Nefamese'],
                    [b'neg', b'Negidal'],
                    [b'neh', b'Nyenkha'],
                    [b'nei', b'Hittite, Neo-'],
                    [b'nej', b'Neko'],
                    [b'nek', b'Neku'],
                    [b'nem', b'Nemi'],
                    [b'nen', b'Nengone'],
                    [b'neo', 'N\xe1-Meo'],
                    [b'nep', b'Nepali (macrolanguage)'],
                    [b'neq', b'Mixe, North Central'],
                    [b'ner', b'Yahadian'],
                    [b'nes', b'Kinnauri, Bhoti'],
                    [b'net', b'Nete'],
                    [b'neu', b'Neo'],
                    [b'nev', b'Nyaheun'],
                    [b'new', b'Newari'],
                    [b'nex', b'Neme'],
                    [b'ney', b'Neyo'],
                    [b'nez', b'Nez Perce'],
                    [b'nfa', b'Dhao'],
                    [b'nfd', b'Ahwai'],
                    [b'nfl', b'Ayiwo'],
                    [b'nfr', b'Nafaanra'],
                    [b'nfu', b'Mfumte'],
                    [b'nga', b'Ngbaka'],
                    [b'ngb', b'Ngbandi, Northern'],
                    [b'ngc', b'Ngombe (Democratic Republic of Congo)'],
                    [b'ngd', b'Ngando (Central African Republic)'],
                    [b'nge', b'Ngemba'],
                    [b'ngg', b'Ngbaka Manza'],
                    [b'ngh', b'N/u'],
                    [b'ngi', b'Ngizim'],
                    [b'ngj', b'Ngie'],
                    [b'ngk', b'Dalabon'],
                    [b'ngl', b'Lomwe'],
                    [b'ngm', b"Ngatik Men's Creole"],
                    [b'ngn', b'Ngwo'],
                    [b'ngo', b'Ngoni'],
                    [b'ngp', b'Ngulu'],
                    [b'ngq', b'Ngurimi'],
                    [b'ngr', b'Engdewu'],
                    [b'ngs', b'Gvoko'],
                    [b'ngt', b'Ngeq'],
                    [b'ngu', b'Nahuatl, Guerrero'],
                    [b'ngv', b'Nagumi'],
                    [b'ngw', b'Ngwaba'],
                    [b'ngx', b'Nggwahyi'],
                    [b'ngy', b'Tibea'],
                    [b'ngz', b'Ngungwel'],
                    [b'nha', b'Nhanda'],
                    [b'nhb', b'Beng'],
                    [b'nhc', b'Nahuatl, Tabasco'],
                    [b'nhd', 'Chirip\xe1'],
                    [b'nhe', b'Nahuatl, Eastern Huasteca'],
                    [b'nhf', b'Nhuwala'],
                    [b'nhg', b'Nahuatl, Tetelcingo'],
                    [b'nhh', b'Nahari'],
                    [
                        b'nhi',
                        'Nahuatl, Zacatl\xe1n-Ahuacatl\xe1n-Tepetzintla'
                    ],
                    [b'nhk', b'Nahuatl, Isthmus-Cosoleacaque'],
                    [b'nhm', b'Nahuatl, Morelos'],
                    [b'nhn', b'Nahuatl, Central'],
                    [b'nho', b'Takuu'],
                    [b'nhp', b'Nahuatl, Isthmus-Pajapan'],
                    [b'nhq', b'Nahuatl, Huaxcaleca'],
                    [b'nhr', b'Naro'],
                    [b'nht', b'Nahuatl, Ometepec'],
                    [b'nhu', b'Noone'],
                    [b'nhv', b'Nahuatl, Temascaltepec'],
                    [b'nhw', b'Nahuatl, Western Huasteca'],
                    [b'nhx', b'Nahuatl, Isthmus-Mecayapan'],
                    [b'nhy', b'Nahuatl, Northern Oaxaca'],
                    [b'nhz', 'Nahuatl, Santa Mar\xeda La Alta'],
                    [b'nia', b'Nias'],
                    [b'nib', b'Nakame'],
                    [b'nid', b'Ngandi'],
                    [b'nie', b'Niellim'],
                    [b'nif', b'Nek'],
                    [b'nig', b'Ngalakan'],
                    [b'nih', b'Nyiha (Tanzania)'],
                    [b'nii', b'Nii'],
                    [b'nij', b'Ngaju'],
                    [b'nik', b'Nicobarese, Southern'],
                    [b'nil', b'Nila'],
                    [b'nim', b'Nilamba'],
                    [b'nin', b'Ninzo'],
                    [b'nio', b'Nganasan'],
                    [b'niq', b'Nandi'],
                    [b'nir', b'Nimboran'],
                    [b'nis', b'Nimi'],
                    [b'nit', b'Kolami, Southeastern'],
                    [b'niu', b'Niuean'],
                    [b'niv', b'Gilyak'],
                    [b'niw', b'Nimo'],
                    [b'nix', b'Hema'],
                    [b'niy', b'Ngiti'],
                    [b'niz', b'Ningil'],
                    [b'nja', b'Nzanyi'],
                    [b'njb', b'Naga, Nocte'],
                    [b'njd', b'Ndonde Hamba'],
                    [b'njh', b'Naga, Lotha'],
                    [b'nji', b'Gudanji'],
                    [b'njj', b'Njen'],
                    [b'njl', b'Njalgulgule'],
                    [b'njm', b'Naga, Angami'],
                    [b'njn', b'Naga, Liangmai'],
                    [b'njo', b'Naga, Ao'],
                    [b'njr', b'Njerep'],
                    [b'njs', b'Nisa'],
                    [b'njt', b'Ndyuka-Trio Pidgin'],
                    [b'nju', b'Ngadjunmaya'],
                    [b'njx', b'Kunyi'],
                    [b'njy', b'Njyem'],
                    [b'njz', b'Nyishi'],
                    [b'nka', b'Nkoya'],
                    [b'nkb', b'Naga, Khoibu'],
                    [b'nkc', b'Nkongho'],
                    [b'nkd', b'Koireng'],
                    [b'nke', b'Duke'],
                    [b'nkf', b'Naga, Inpui'],
                    [b'nkg', b'Nekgini'],
                    [b'nkh', b'Naga, Khezha'],
                    [b'nki', b'Naga, Thangal'],
                    [b'nkj', b'Nakai'],
                    [b'nkk', b'Nokuku'],
                    [b'nkm', b'Namat'],
                    [b'nkn', b'Nkangala'],
                    [b'nko', b'Nkonya'],
                    [b'nkp', b'Niuatoputapu'],
                    [b'nkq', b'Nkami'],
                    [b'nkr', b'Nukuoro'],
                    [b'nks', b'Asmat, North'],
                    [b'nkt', b'Nyika (Tanzania)'],
                    [b'nku', b'Kulango, Bouna'],
                    [b'nkv', b'Nyika (Malawi and Zambia)'],
                    [b'nkw', b'Nkutu'],
                    [b'nkx', b'Nkoroo'],
                    [b'nkz', b'Nkari'],
                    [b'nla', b'Ngombale'],
                    [b'nlc', b'Nalca'],
                    [b'nld', b'Dutch'],
                    [b'nle', b'Nyala, East'],
                    [b'nlg', b'Gela'],
                    [b'nli', b'Grangali'],
                    [b'nlj', b'Nyali'],
                    [b'nlk', b'Yali, Ninia'],
                    [b'nll', b'Nihali'],
                    [b'nlo', b'Ngul'],
                    [b'nlq', b'Naga, Lao'],
                    [b'nlu', b'Nchumbulu'],
                    [b'nlv', b'Nahuatl, Orizaba'],
                    [b'nlw', b'Walangama'],
                    [b'nlx', b'Nahali'],
                    [b'nly', b'Nyamal'],
                    [b'nlz', 'Nal\xf6go'],
                    [b'nma', b'Naga, Maram'],
                    [b'nmb', b'Nambas, Big'],
                    [b'nmc', b'Ngam'],
                    [b'nmd', b'Ndumu'],
                    [b'nme', b'Naga, Mzieme'],
                    [b'nmf', b'Naga, Tangkhul (India)'],
                    [b'nmg', b'Kwasio'],
                    [b'nmh', b'Naga, Monsang'],
                    [b'nmi', b'Nyam'],
                    [b'nmj', b'Ngombe (Central African Republic)'],
                    [b'nmk', b'Namakura'],
                    [b'nml', b'Ndemli'],
                    [b'nmm', b'Manangba'],
                    [b'nmn', '!X\xf3\xf5'],
                    [b'nmo', b'Naga, Moyon'],
                    [b'nmp', b'Nimanbur'],
                    [b'nmq', b'Nambya'],
                    [b'nmr', b'Nimbari'],
                    [b'nms', b'Letemboi'],
                    [b'nmt', b'Namonuito'],
                    [b'nmu', b'Maidu, Northeast'],
                    [b'nmv', b'Ngamini'],
                    [b'nmw', b'Nimoa'],
                    [b'nmx', b'Nama (Papua New Guinea)'],
                    [b'nmy', b'Namuyi'],
                    [b'nmz', b'Nawdm'],
                    [b'nna', b'Nyangumarta'],
                    [b'nnb', b'Nande'],
                    [b'nnc', b'Nancere'],
                    [b'nnd', b'Ambae, West'],
                    [b'nne', b'Ngandyera'],
                    [b'nnf', b'Ngaing'],
                    [b'nng', b'Naga, Maring'],
                    [b'nnh', b'Ngiemboon'],
                    [b'nni', b'Nuaulu, North'],
                    [b'nnj', b'Nyangatom'],
                    [b'nnk', b'Nankina'],
                    [b'nnl', b'Naga, Northern Rengma'],
                    [b'nnm', b'Namia'],
                    [b'nnn', b'Ngete'],
                    [b'nno', b'Norwegian Nynorsk'],
                    [b'nnp', b'Naga, Wancho'],
                    [b'nnq', b'Ngindo'],
                    [b'nnr', b'Narungga'],
                    [b'nns', b'Ningye'],
                    [b'nnt', b'Nanticoke'],
                    [b'nnu', b'Dwang'],
                    [b'nnv', b'Nugunu (Australia)'],
                    [b'nnw', b'Nuni, Southern'],
                    [b'nnx', b'Ngong'],
                    [b'nny', b'Nyangga'],
                    [b'nnz', b"Nda'nda'"],
                    [b'noa', b'Woun Meu'],
                    [b'nob', 'Norwegian Bokm\xe5l'],
                    [b'noc', b'Nuk'],
                    [b'nod', b'Thai, Northern'],
                    [b'noe', b'Nimadi'],
                    [b'nof', b'Nomane'],
                    [b'nog', b'Nogai'],
                    [b'noh', b'Nomu'],
                    [b'noi', b'Noiri'],
                    [b'noj', b'Nonuya'],
                    [b'nok', b'Nooksack'],
                    [b'nol', b'Nomlaki'],
                    [b'nom', 'Nocam\xe1n'],
                    [b'non', b'Norse, Old'],
                    [b'nop', b'Numanggang'],
                    [b'noq', b'Ngongo'],
                    [b'nor', b'Norwegian'],
                    [b'nos', b'Nisu, Eastern'],
                    [b'not', b'Nomatsiguenga'],
                    [b'nou', b'Ewage-Notu'],
                    [b'nov', b'Novial'],
                    [b'now', b'Nyambo'],
                    [b'noy', b'Noy'],
                    [b'noz', b'Nayi'],
                    [b'npa', b'Nar Phu'],
                    [b'npb', b'Nupbikha'],
                    [b'npg', b'Naga, Ponyo-Gongwang'],
                    [b'nph', b'Naga, Phom'],
                    [b'npi', b'Nepali (individual language)'],
                    [b'npl', b'Nahuatl, Southeastern Puebla'],
                    [b'npn', b'Mondropolon'],
                    [b'npo', b'Naga, Pochuri'],
                    [b'nps', b'Nipsan'],
                    [b'npu', b'Naga, Puimei'],
                    [b'npy', b'Napu'],
                    [b'nqg', b'Nago, Southern'],
                    [b'nqk', b'Ede Nago, Kura'],
                    [b'nqm', b'Ndom'],
                    [b'nqn', b'Nen'],
                    [b'nqo', b"N'Ko"],
                    [b'nqq', b'Naga, Kyan-Karyaw'],
                    [b'nqy', b'Naga, Akyaung Ari'],
                    [b'nra', b'Ngom'],
                    [b'nrb', b'Nara'],
                    [b'nrc', b'Noric'],
                    [b'nre', b'Naga, Southern Rengma'],
                    [b'nrg', b'Narango'],
                    [b'nri', b'Naga, Chokri'],
                    [b'nrk', b'Ngarla'],
                    [b'nrl', b'Ngarluma'],
                    [b'nrm', b'Narom'],
                    [b'nrn', b'Norn'],
                    [b'nrp', b'Picene, North'],
                    [b'nrr', b'Norra'],
                    [b'nrt', b'Kalapuya, Northern'],
                    [b'nru', b'Narua'],
                    [b'nrx', b'Ngurmbur'],
                    [b'nrz', b'Lala'],
                    [b'nsa', b'Naga, Sangtam'],
                    [b'nsc', b'Nshi'],
                    [b'nsd', b'Nisu, Southern'],
                    [b'nse', b'Nsenga'],
                    [b'nsf', b'Nisu, Northwestern'],
                    [b'nsg', b'Ngasa'],
                    [b'nsh', b'Ngoshie'],
                    [b'nsi', b'Nigerian Sign Language'],
                    [b'nsk', b'Naskapi'],
                    [b'nsl', b'Norwegian Sign Language'],
                    [b'nsm', b'Naga, Sumi'],
                    [b'nsn', b'Nehan'],
                    [b'nso', b'Pedi'],
                    [b'nsp', b'Nepalese Sign Language'],
                    [b'nsq', b'Miwok, Northern Sierra'],
                    [b'nsr', b'Maritime Sign Language'],
                    [b'nss', b'Nali'],
                    [b'nst', b'Naga, Tase'],
                    [b'nsu', b'Nahuatl, Sierra Negra'],
                    [b'nsv', b'Nisu, Southwestern'],
                    [b'nsw', b'Navut'],
                    [b'nsx', b'Nsongo'],
                    [b'nsy', b'Nasal'],
                    [b'nsz', b'Nisenan'],
                    [b'nte', b'Nathembo'],
                    [b'ntg', b'Ngantangarra'],
                    [b'nti', b'Natioro'],
                    [b'ntj', b'Ngaanyatjarra'],
                    [b'ntk', b'Ikoma-Nata-Isenye'],
                    [b'ntm', b'Nateni'],
                    [b'nto', b'Ntomba'],
                    [b'ntp', b'Tepehuan, Northern'],
                    [b'ntr', b'Delo'],
                    [b'nts', b'Natagaimas'],
                    [b'ntu', 'Nat\xfcgu'],
                    [b'ntw', b'Nottoway'],
                    [b'ntx', b'Naga, Tangkhul (Myanmar)'],
                    [b'nty', b'Mantsi'],
                    [b'ntz', b'Natanzi'],
                    [b'nua', b'Yuanga'],
                    [b'nuc', b'Nukuini'],
                    [b'nud', b'Ngala'],
                    [b'nue', b'Ngundu'],
                    [b'nuf', b'Nusu'],
                    [b'nug', b'Nungali'],
                    [b'nuh', b'Ndunda'],
                    [b'nui', b'Ngumbi'],
                    [b'nuj', b'Nyole'],
                    [b'nuk', b'Nuu-chah-nulth'],
                    [b'nul', b'Nusa Laut'],
                    [b'num', b"Niuafo'ou"],
                    [b'nun', b'Anong'],
                    [b'nuo', 'Ngu\xf4n'],
                    [b'nup', b'Nupe-Nupe-Tako'],
                    [b'nuq', b'Nukumanu'],
                    [b'nur', b'Nukuria'],
                    [b'nus', b'Nuer'],
                    [b'nut', b'Nung (Viet Nam)'],
                    [b'nuu', b'Ngbundu'],
                    [b'nuv', b'Nuni, Northern'],
                    [b'nuw', b'Nguluwan'],
                    [b'nux', b'Mehek'],
                    [b'nuy', b'Nunggubuyu'],
                    [b'nuz', b'Nahuatl, Tlamacazapa'],
                    [b'nvh', b'Nasarian'],
                    [b'nvm', b'Namiae'],
                    [b'nvo', b'Nyokon'],
                    [b'nwa', b'Nawathinehena'],
                    [b'nwb', b'Nyabwa'],
                    [b'nwc', b'Newari, Classical'],
                    [b'nwe', b'Ngwe'],
                    [b'nwg', b'Ngayawung'],
                    [b'nwi', b'Tanna, Southwest'],
                    [b'nwm', b'Nyamusa-Molo'],
                    [b'nwo', b'Nauo'],
                    [b'nwr', b'Nawaru'],
                    [b'nwx', b'Newar, Middle'],
                    [b'nwy', b'Nottoway-Meherrin'],
                    [b'nxa', b'Nauete'],
                    [b'nxd', b'Ngando (Democratic Republic of Congo)'],
                    [b'nxe', b'Nage'],
                    [b'nxg', b"Ngad'a"],
                    [b'nxi', b'Nindi'],
                    [b'nxk', b'Naga, Koki'],
                    [b'nxl', b'Nuaulu, South'],
                    [b'nxm', b'Numidian'],
                    [b'nxn', b'Ngawun'],
                    [b'nxq', b'Naxi'],
                    [b'nxr', b'Ninggerum'],
                    [b'nxu', b'Narau'],
                    [b'nxx', b'Nafri'],
                    [b'nya', b'Nyanja'],
                    [b'nyb', b'Nyangbo'],
                    [b'nyc', b'Nyanga-li'],
                    [b'nyd', b'Nyore'],
                    [b'nye', b'Nyengo'],
                    [b'nyf', b'Giryama'],
                    [b'nyg', b'Nyindu'],
                    [b'nyh', b'Nyigina'],
                    [b'nyi', b'Ama (Sudan)'],
                    [b'nyj', b'Nyanga'],
                    [b'nyk', b'Nyaneka'],
                    [b'nyl', b'Nyeu'],
                    [b'nym', b'Nyamwezi'],
                    [b'nyn', b'Nyankole'],
                    [b'nyo', b'Nyoro'],
                    [b'nyp', b"Nyang'i"],
                    [b'nyq', b'Nayini'],
                    [b'nyr', b'Nyiha (Malawi)'],
                    [b'nys', b'Nyunga'],
                    [b'nyt', b'Nyawaygi'],
                    [b'nyu', b'Nyungwe'],
                    [b'nyv', b'Nyulnyul'],
                    [b'nyw', b'Nyaw'],
                    [b'nyx', b'Nganyaywana'],
                    [b'nyy', b'Nyakyusa-Ngonde'],
                    [b'nza', b'Mbembe, Tigon'],
                    [b'nzb', b'Njebi'],
                    [b'nzi', b'Nzima'],
                    [b'nzk', b'Nzakara'],
                    [b'nzm', b'Naga, Zeme'],
                    [b'nzs', b'New Zealand Sign Language'],
                    [b'nzu', b'Teke-Nzikou'],
                    [b'nzy', b'Nzakambay'],
                    [b'nzz', b'Dogon, Nanga Dama'],
                    [b'oaa', b'Orok'],
                    [b'oac', b'Oroch'],
                    [b'oar', b'Aramaic, Old (up to 700 BCE)'],
                    [b'oav', b'Avar, Old'],
                    [b'obi', 'Obispe\xf1o'],
                    [b'obk', b'Bontok, Southern'],
                    [b'obl', b'Oblo'],
                    [b'obm', b'Moabite'],
                    [b'obo', b'Manobo, Obo'],
                    [b'obr', b'Burmese, Old'],
                    [b'obt', b'Breton, Old'],
                    [b'obu', b'Obulom'],
                    [b'oca', b'Ocaina'],
                    [b'och', b'Chinese, Old'],
                    [b'oci', b'Occitan (post 1500)'],
                    [b'oco', b'Cornish, Old'],
                    [b'ocu', b'Matlatzinca, Atzingo'],
                    [b'oda', b'Odut'],
                    [b'odk', b'Od'],
                    [b'odt', b'Dutch, Old'],
                    [b'odu', b'Odual'],
                    [b'ofo', b'Ofo'],
                    [b'ofs', b'Frisian, Old'],
                    [b'ofu', b'Efutop'],
                    [b'ogb', b'Ogbia'],
                    [b'ogc', b'Ogbah'],
                    [b'oge', b'Georgian, Old'],
                    [b'ogg', b'Ogbogolo'],
                    [b'ogo', b'Khana'],
                    [b'ogu', b'Ogbronuagum'],
                    [b'oht', b'Hittite, Old'],
                    [b'ohu', b'Hungarian, Old'],
                    [b'oia', b'Oirata'],
                    [b'oin', b'One, Inebu'],
                    [b'ojb', b'Ojibwa, Northwestern'],
                    [b'ojc', b'Ojibwa, Central'],
                    [b'ojg', b'Ojibwa, Eastern'],
                    [b'oji', b'Ojibwa'],
                    [b'ojp', b'Japanese, Old'],
                    [b'ojs', b'Ojibwa, Severn'],
                    [b'ojv', b'Ontong Java'],
                    [b'ojw', b'Ojibwa, Western'],
                    [b'oka', b'Okanagan'],
                    [b'okb', b'Okobo'],
                    [b'okd', b'Okodia'],
                    [b'oke', b'Okpe (Southwestern Edo)'],
                    [b'okg', b'Koko Babangk'],
                    [b'okh', b'Koresh-e Rostam'],
                    [b'oki', b'Okiek'],
                    [b'okj', b'Oko-Juwoi'],
                    [b'okk', b'One, Kwamtim'],
                    [b'okl', b'Kentish Sign Language, Old'],
                    [b'okm', b'Korean, Middle (10th-16th cent.)'],
                    [b'okn', b'Oki-No-Erabu'],
                    [b'oko', b'Korean, Old (3rd-9th cent.)'],
                    [b'okr', b'Kirike'],
                    [b'oks', b'Oko-Eni-Osayen'],
                    [b'oku', b'Oku'],
                    [b'okv', b'Orokaiva'],
                    [b'okx', b'Okpe (Northwestern Edo)'],
                    [b'ola', b'Walungge'],
                    [b'old', b'Mochi'],
                    [b'ole', b'Olekha'],
                    [b'olk', b'Olkol'],
                    [b'olm', b'Oloma'],
                    [b'olo', b'Livvi'],
                    [b'olr', b'Olrat'],
                    [b'oma', b'Omaha-Ponca'],
                    [b'omb', b'Ambae, East'],
                    [b'omc', b'Mochica'],
                    [b'ome', b'Omejes'],
                    [b'omg', b'Omagua'],
                    [b'omi', b'Omi'],
                    [b'omk', b'Omok'],
                    [b'oml', b'Ombo'],
                    [b'omn', b'Minoan'],
                    [b'omo', b'Utarmbung'],
                    [b'omp', b'Manipuri, Old'],
                    [b'omr', b'Marathi, Old'],
                    [b'omt', b'Omotik'],
                    [b'omu', b'Omurano'],
                    [b'omw', b'Tairora, South'],
                    [b'omx', b'Mon, Old'],
                    [b'ona', b'Ona'],
                    [b'onb', b'Lingao'],
                    [b'one', b'Oneida'],
                    [b'ong', b'Olo'],
                    [b'oni', b'Onin'],
                    [b'onj', b'Onjob'],
                    [b'onk', b'One, Kabore'],
                    [b'onn', b'Onobasulu'],
                    [b'ono', b'Onondaga'],
                    [b'onp', b'Sartang'],
                    [b'onr', b'One, Northern'],
                    [b'ons', b'Ono'],
                    [b'ont', b'Ontenu'],
                    [b'onu', b'Unua'],
                    [b'onw', b'Nubian, Old'],
                    [b'onx', b'Onin Based Pidgin'],
                    [b'ood', b"Tohono O'odham"],
                    [b'oog', b'Ong'],
                    [b'oon', '\xd6nge'],
                    [b'oor', b'Oorlams'],
                    [b'oos', b'Ossetic, Old'],
                    [b'opa', b'Okpamheri'],
                    [b'opk', b'Kopkaka'],
                    [b'opm', b'Oksapmin'],
                    [b'opo', b'Opao'],
                    [b'opt', b'Opata'],
                    [b'opy', 'Ofay\xe9'],
                    [b'ora', b'Oroha'],
                    [b'orc', b'Orma'],
                    [b'ore', 'Orej\xf3n'],
                    [b'org', b'Oring'],
                    [b'orh', b'Oroqen'],
                    [b'ori', b'Oriya (macrolanguage)'],
                    [b'orm', b'Oromo'],
                    [b'orn', b'Orang Kanaq'],
                    [b'oro', b'Orokolo'],
                    [b'orr', b'Oruma'],
                    [b'ors', b'Orang Seletar'],
                    [b'ort', b'Oriya, Adivasi'],
                    [b'oru', b'Ormuri'],
                    [b'orv', b'Russian, Old'],
                    [b'orw', b'Oro Win'],
                    [b'orx', b'Oro'],
                    [b'ory', b'Oriya (individual language)'],
                    [b'orz', b'Ormu'],
                    [b'osa', b'Osage'],
                    [b'osc', b'Oscan'],
                    [b'osi', b'Osing'],
                    [b'oso', b'Ososo'],
                    [b'osp', b'Spanish, Old'],
                    [b'oss', b'Ossetian'],
                    [b'ost', b'Osatu'],
                    [b'osu', b'One, Southern'],
                    [b'osx', b'Saxon, Old'],
                    [b'ota', b'Turkish, Ottoman (1500-1928)'],
                    [b'otb', b'Tibetan, Old'],
                    [b'otd', b'Ot Danum'],
                    [b'ote', b'Otomi, Mezquital'],
                    [b'oti', b'Oti'],
                    [b'otk', b'Turkish, Old'],
                    [b'otl', b'Otomi, Tilapa'],
                    [b'otm', b'Otomi, Eastern Highland'],
                    [b'otn', b'Otomi, Tenango'],
                    [b'otq', 'Otomi, Quer\xe9taro'],
                    [b'otr', b'Otoro'],
                    [b'ots', 'Otomi, Estado de M\xe9xico'],
                    [b'ott', b'Otomi, Temoaya'],
                    [b'otu', b'Otuke'],
                    [b'otw', b'Ottawa'],
                    [b'otx', b'Otomi, Texcatepec'],
                    [b'oty', b'Tamil, Old'],
                    [b'otz', b'Otomi, Ixtenco'],
                    [b'oua', b'Tagargrent'],
                    [b'oub', b'Glio-Oubi'],
                    [b'oue', b'Oune'],
                    [b'oui', b'Uighur, Old'],
                    [b'oum', b'Ouma'],
                    [b'oun', b'!O!ung'],
                    [b'owi', b'Owiniga'],
                    [b'owl', b'Welsh, Old'],
                    [b'oyb', b'Oy'],
                    [b'oyd', b'Oyda'],
                    [b'oym', b'Wayampi'],
                    [b'oyy', b"Oya'oya"],
                    [b'ozm', b'Koonzime'],
                    [b'pab', 'Parec\xeds'],
                    [b'pac', b'Pacoh'],
                    [b'pad', 'Paumar\xed'],
                    [b'pae', b'Pagibete'],
                    [b'paf', 'Paranaw\xe1t'],
                    [b'pag', b'Pangasinan'],
                    [b'pah', b'Tenharim'],
                    [b'pai', b'Pe'],
                    [b'pak', 'Parakan\xe3'],
                    [b'pal', b'Pahlavi'],
                    [b'pam', b'Pampanga'],
                    [b'pan', b'Panjabi'],
                    [b'pao', b'Paiute, Northern'],
                    [b'pap', b'Papiamento'],
                    [b'paq', b'Parya'],
                    [b'par', b'Panamint'],
                    [b'pas', b'Papasena'],
                    [b'pat', b'Papitalai'],
                    [b'pau', b'Palauan'],
                    [b'pav', 'Paka\xe1snovos'],
                    [b'paw', b'Pawnee'],
                    [b'pax', 'Pankarar\xe9'],
                    [b'pay', b'Pech'],
                    [b'paz', 'Pankarar\xfa'],
                    [b'pbb', 'P\xe1ez'],
                    [b'pbc', b'Patamona'],
                    [b'pbe', b'Popoloca, Mezontla'],
                    [b'pbf', b'Popoloca, Coyotepec'],
                    [b'pbg', b'Paraujano'],
                    [b'pbh', "E'\xf1apa Woromaipu"],
                    [b'pbi', b'Parkwa'],
                    [b'pbl', b'Mak (Nigeria)'],
                    [b'pbn', b'Kpasam'],
                    [b'pbo', b'Papel'],
                    [b'pbp', b'Badyara'],
                    [b'pbr', b'Pangwa'],
                    [b'pbs', b'Pame, Central'],
                    [b'pbt', b'Pashto, Southern'],
                    [b'pbu', b'Pashto, Northern'],
                    [b'pbv', b'Pnar'],
                    [b'pby', b'Pyu'],
                    [b'pca', 'Popoloca, Santa In\xe9s Ahuatempan'],
                    [b'pcb', b'Pear'],
                    [b'pcc', b'Bouyei'],
                    [b'pcd', b'Picard'],
                    [b'pce', b'Palaung, Ruching'],
                    [b'pcf', b'Paliyan'],
                    [b'pcg', b'Paniya'],
                    [b'pch', b'Pardhan'],
                    [b'pci', b'Duruwa'],
                    [b'pcj', b'Parenga'],
                    [b'pck', b'Chin, Paite'],
                    [b'pcl', b'Pardhi'],
                    [b'pcm', b'Pidgin, Nigerian'],
                    [b'pcn', b'Piti'],
                    [b'pcp', b'Pacahuara'],
                    [b'pcw', b'Pyapun'],
                    [b'pda', b'Anam'],
                    [b'pdc', b'German, Pennsylvania'],
                    [b'pdi', b'Pa Di'],
                    [b'pdn', b'Podena'],
                    [b'pdo', b'Padoe'],
                    [b'pdt', b'Plautdietsch'],
                    [b'pdu', b'Kayan'],
                    [b'pea', b'Indonesian, Peranakan'],
                    [b'peb', b'Pomo, Eastern'],
                    [b'ped', b'Mala (Papua New Guinea)'],
                    [b'pee', b'Taje'],
                    [b'pef', b'Pomo, Northeastern'],
                    [b'peg', b'Pengo'],
                    [b'peh', b'Bonan'],
                    [b'pei', b'Chichimeca-Jonaz'],
                    [b'pej', b'Pomo, Northern'],
                    [b'pek', b'Penchal'],
                    [b'pel', b'Pekal'],
                    [b'pem', b'Phende'],
                    [b'peo', b'Persian, Old (ca. 600-400 B.C.)'],
                    [b'pep', b'Kunja'],
                    [b'peq', b'Pomo, Southern'],
                    [b'pes', b'Persian, Iranian'],
                    [b'pev', 'P\xe9mono'],
                    [b'pex', b'Petats'],
                    [b'pey', b'Petjo'],
                    [b'pez', b'Penan, Eastern'],
                    [b'pfa', 'P\xe1\xe1fang'],
                    [b'pfe', b'Peere'],
                    [b'pfl', b'Pfaelzisch'],
                    [b'pga', b'Creole Arabic, Sudanese'],
                    [b'pgg', b'Pangwali'],
                    [b'pgi', b'Pagi'],
                    [b'pgk', b'Rerep'],
                    [b'pgl', b'Irish, Primitive'],
                    [b'pgn', b'Paelignian'],
                    [b'pgs', b'Pangseng'],
                    [b'pgu', b'Pagu'],
                    [b'pha', b'Pa-Hng'],
                    [b'phd', b'Phudagi'],
                    [b'phg', b'Phuong'],
                    [b'phh', b'Phukha'],
                    [b'phk', b'Phake'],
                    [b'phl', b'Phalura'],
                    [b'phm', b'Phimbi'],
                    [b'phn', b'Phoenician'],
                    [b'pho', b'Phunoi'],
                    [b'phq', b"Phana'"],
                    [b'phr', b'Pahari-Potwari'],
                    [b'pht', b'Phu Thai'],
                    [b'phu', b'Phuan'],
                    [b'phv', b'Pahlavani'],
                    [b'phw', b'Phangduwali'],
                    [b'pia', b'Pima Bajo'],
                    [b'pib', b'Yine'],
                    [b'pic', b'Pinji'],
                    [b'pid', b'Piaroa'],
                    [b'pie', b'Piro'],
                    [b'pif', b'Pingelapese'],
                    [b'pig', b'Pisabo'],
                    [b'pih', b'Pitcairn-Norfolk'],
                    [b'pii', b'Pini'],
                    [b'pij', b'Pijao'],
                    [b'pil', b'Yom'],
                    [b'pim', b'Powhatan'],
                    [b'pin', b'Piame'],
                    [b'pio', b'Piapoco'],
                    [b'pip', b'Pero'],
                    [b'pir', b'Piratapuyo'],
                    [b'pis', b'Pijin'],
                    [b'pit', b'Pitta Pitta'],
                    [b'piu', b'Pintupi-Luritja'],
                    [b'piv', b'Pileni'],
                    [b'piw', b'Pimbwe'],
                    [b'pix', b'Piu'],
                    [b'piy', b'Piya-Kwonci'],
                    [b'piz', b'Pije'],
                    [b'pjt', b'Pitjantjatjara'],
                    [b'pka', 'Pr\u0101krit, Ardham\u0101gadh\u012b'],
                    [b'pkb', b'Pokomo'],
                    [b'pkc', b'Paekche'],
                    [b'pkg', b'Pak-Tong'],
                    [b'pkh', b'Pankhu'],
                    [b'pkn', b'Pakanha'],
                    [b'pko', 'P\xf6koot'],
                    [b'pkp', b'Pukapuka'],
                    [b'pkr', b'Kurumba, Attapady'],
                    [b'pks', b'Pakistan Sign Language'],
                    [b'pkt', b'Maleng'],
                    [b'pku', b'Paku'],
                    [b'pla', b'Miani'],
                    [b'plb', b'Polonombauk'],
                    [b'plc', b'Palawano, Central'],
                    [b'pld', b'Polari'],
                    [b'ple', b"Palu'e"],
                    [b'plg', 'Pilag\xe1'],
                    [b'plh', b'Paulohi'],
                    [b'pli', b'Pali'],
                    [b'plj', b'Polci'],
                    [b'plk', b'Shina, Kohistani'],
                    [b'pll', b'Palaung, Shwe'],
                    [b'pln', b'Palenquero'],
                    [b'plo', b'Popoluca, Oluta'],
                    [b'plp', b'Palpa'],
                    [b'plq', b'Palaic'],
                    [b'plr', b'Senoufo, Palaka'],
                    [b'pls', b'Popoloca, San Marcos Tlalcoyalco'],
                    [b'plt', b'Malagasy, Plateau'],
                    [b'plu', 'Palik\xfar'],
                    [b'plv', b'Palawano, Southwest'],
                    [b'plw', b"Palawano, Brooke's Point"],
                    [b'ply', b'Bolyu'],
                    [b'plz', b'Paluan'],
                    [b'pma', b'Paama'],
                    [b'pmb', b'Pambia'],
                    [b'pmc', b'Palumata'],
                    [b'pmd', b'Pallanganmiddang'],
                    [b'pme', b'Pwaamei'],
                    [b'pmf', b'Pamona'],
                    [
                        b'pmh',
                        'Pr\u0101krit, M\u0101h\u0101r\u0101\u1e63\u1e6dri'
                    ],
                    [b'pmi', b'Pumi, Northern'],
                    [b'pmj', b'Pumi, Southern'],
                    [b'pmk', b'Pamlico'],
                    [b'pml', b'Lingua Franca'],
                    [b'pmm', b'Pomo'],
                    [b'pmn', b'Pam'],
                    [b'pmo', b'Pom'],
                    [b'pmq', b'Pame, Northern'],
                    [b'pmr', b'Paynamar'],
                    [b'pms', b'Piemontese'],
                    [b'pmt', b'Tuamotuan'],
                    [b'pmu', b'Panjabi, Mirpur'],
                    [b'pmw', b'Miwok, Plains'],
                    [b'pmx', b'Naga, Poumei'],
                    [b'pmy', b'Malay, Papuan'],
                    [b'pmz', b'Pame, Southern'],
                    [b'pna', b'Punan Bah-Biau'],
                    [b'pnb', b'Panjabi, Western'],
                    [b'pnc', b'Pannei'],
                    [b'pne', b'Penan, Western'],
                    [b'png', b'Pongu'],
                    [b'pnh', b'Penrhyn'],
                    [b'pni', b'Aoheng'],
                    [b'pnj', b'Pinjarup'],
                    [b'pnk', b'Paunaka'],
                    [b'pnl', b'Paleni'],
                    [b'pnm', b'Punan Batu 1'],
                    [b'pnn', b'Pinai-Hagahai'],
                    [b'pno', b'Panobo'],
                    [b'pnp', b'Pancana'],
                    [b'pnq', b'Pana (Burkina Faso)'],
                    [b'pnr', b'Panim'],
                    [b'pns', b'Ponosakan'],
                    [b'pnt', b'Pontic'],
                    [b'pnu', b'Bunu, Jiongnai'],
                    [b'pnv', b'Pinigura'],
                    [b'pnw', b'Panytyima'],
                    [b'pnx', b'Phong-Kniang'],
                    [b'pny', b'Pinyin'],
                    [b'pnz', b'Pana (Central African Republic)'],
                    [b'poc', b'Poqomam'],
                    [b'pod', b'Ponares'],
                    [b'poe', b'Popoloca, San Juan Atzingo'],
                    [b'pof', b'Poke'],
                    [b'pog', 'Potigu\xe1ra'],
                    [b'poh', b"Poqomchi'"],
                    [b'poi', b'Popoluca, Highland'],
                    [b'pok', 'Pokang\xe1'],
                    [b'pol', b'Polish'],
                    [b'pom', b'Pomo, Southeastern'],
                    [b'pon', b'Pohnpeian'],
                    [b'poo', b'Pomo, Central'],
                    [b'pop', 'Pwapw\xe2'],
                    [b'poq', b'Popoluca, Texistepec'],
                    [b'por', b'Portuguese'],
                    [b'pos', b'Popoluca, Sayula'],
                    [b'pot', b'Potawatomi'],
                    [b'pov', b'Crioulo, Upper Guinea'],
                    [b'pow', b'Popoloca, San Felipe Otlaltepec'],
                    [b'pox', b'Polabian'],
                    [b'poy', b'Pogolo'],
                    [b'ppa', b'Pao'],
                    [b'ppe', b'Papi'],
                    [b'ppi', b'Paipai'],
                    [b'ppk', b'Uma'],
                    [b'ppl', b'Pipil'],
                    [b'ppm', b'Papuma'],
                    [b'ppn', b'Papapana'],
                    [b'ppo', b'Folopa'],
                    [b'ppp', b'Pelende'],
                    [b'ppq', b'Pei'],
                    [b'pps', 'Popoloca, San Lu\xeds Temalacayuca'],
                    [b'ppt', b'Pare'],
                    [b'ppu', b'Papora'],
                    [b'pqa', b"Pa'a"],
                    [b'pqm', b'Malecite-Passamaquoddy'],
                    [b'prb', b"Lua'"],
                    [b'prc', b'Parachi'],
                    [b'prd', b'Parsi-Dari'],
                    [b'pre', b'Principense'],
                    [b'prf', b'Paranan'],
                    [b'prg', b'Prussian'],
                    [b'prh', b'Porohanon'],
                    [b'pri', 'Paic\xee'],
                    [b'prk', b'Parauk'],
                    [b'prl', b'Peruvian Sign Language'],
                    [b'prm', b'Kibiri'],
                    [b'prn', b'Prasuni'],
                    [b'pro', 'Proven\xe7al, Old (to 1500)'],
                    [b'prp', b'Parsi'],
                    [b'prq', 'Ash\xe9ninka Peren\xe9'],
                    [b'prr', b'Puri'],
                    [b'prs', b'Dari'],
                    [b'prt', b'Phai'],
                    [b'pru', b'Puragi'],
                    [b'prw', b'Parawen'],
                    [b'prx', b'Purik'],
                    [b'pry', b'Pray 3'],
                    [b'prz', b'Providencia Sign Language'],
                    [b'psa', b'Awyu, Asue'],
                    [b'psc', b'Persian Sign Language'],
                    [b'psd', b'Plains Indian Sign Language'],
                    [b'pse', b'Malay, Central'],
                    [b'psg', b'Penang Sign Language'],
                    [b'psh', b'Pashayi, Southwest'],
                    [b'psi', b'Pashayi, Southeast'],
                    [b'psl', b'Puerto Rican Sign Language'],
                    [b'psm', b'Pauserna'],
                    [b'psn', b'Panasuan'],
                    [b'pso', b'Polish Sign Language'],
                    [b'psp', b'Philippine Sign Language'],
                    [b'psq', b'Pasi'],
                    [b'psr', b'Portuguese Sign Language'],
                    [b'pss', b'Kaulong'],
                    [b'pst', b'Pashto, Central'],
                    [b'psu', 'Pr\u0101krit, Sauraseni'],
                    [b'psw', b'Port Sandwich'],
                    [b'psy', b'Piscataway'],
                    [b'pta', b'Pai Tavytera'],
                    [b'pth', 'Patax\xf3 H\xe3-Ha-H\xe3e'],
                    [b'pti', b'Pintiini'],
                    [b'ptn', b'Patani'],
                    [b'pto', "Zo'\xe9"],
                    [b'ptp', b'Patep'],
                    [b'ptr', b'Piamatsina'],
                    [b'ptt', b'Enrekang'],
                    [b'ptu', b'Bambam'],
                    [b'ptv', b'Port Vato'],
                    [b'ptw', b'Pentlatch'],
                    [b'pty', b'Pathiya'],
                    [b'pua', b'Purepecha, Western Highland'],
                    [b'pub', b'Purum'],
                    [b'puc', b'Punan Merap'],
                    [b'pud', b'Punan Aput'],
                    [b'pue', b'Puelche'],
                    [b'puf', b'Punan Merah'],
                    [b'pug', b'Phuie'],
                    [b'pui', b'Puinave'],
                    [b'puj', b'Punan Tubu'],
                    [b'puk', b'Pu Ko'],
                    [b'pum', b'Puma'],
                    [b'puo', b'Puoc'],
                    [b'pup', b'Pulabu'],
                    [b'puq', b'Puquina'],
                    [b'pur', 'Purubor\xe1'],
                    [b'pus', b'Pushto'],
                    [b'put', b'Putoh'],
                    [b'puu', b'Punu'],
                    [b'puw', b'Puluwatese'],
                    [b'pux', b'Puare'],
                    [b'puy', 'Purisime\xf1o'],
                    [b'puz', b'Naga, Purum'],
                    [b'pwa', b'Pawaia'],
                    [b'pwb', b'Panawa'],
                    [b'pwg', b'Gapapaiwa'],
                    [b'pwi', b'Patwin'],
                    [b'pwm', b'Molbog'],
                    [b'pwn', b'Paiwan'],
                    [b'pwo', b'Karen, Pwo Western'],
                    [b'pwr', b'Powari'],
                    [b'pww', b'Karen, Pwo Northern'],
                    [b'pxm', b'Mixe, Quetzaltepec'],
                    [b'pye', b'Krumen, Pye'],
                    [b'pym', b'Fyam'],
                    [b'pyn', 'Poyan\xe1wa'],
                    [b'pys', b'Paraguayan Sign Language'],
                    [b'pyu', b'Puyuma'],
                    [b'pyx', b'Pyu (Myanmar)'],
                    [b'pyy', b'Pyen'],
                    [b'pzn', b'Naga, Para'],
                    [b'qua', b'Quapaw'],
                    [b'qub', 'Quechua, Huallaga Hu\xe1nuco'],
                    [b'quc', b"K'iche'"],
                    [b'qud', 'Quichua, Calder\xf3n Highland'],
                    [b'que', b'Quechua'],
                    [b'quf', b'Quechua, Lambayeque'],
                    [b'qug', b'Quichua, Chimborazo Highland'],
                    [b'quh', b'Quechua, South Bolivian'],
                    [b'qui', b'Quileute'],
                    [b'quk', b'Quechua, Chachapoyas'],
                    [b'qul', b'Quechua, North Bolivian'],
                    [b'qum', b'Sipacapense'],
                    [b'qun', b'Quinault'],
                    [b'qup', b'Quechua, Southern Pastaza'],
                    [b'quq', b'Quinqui'],
                    [b'qur', b'Quechua, Yanahuanca Pasco'],
                    [b'qus', b'Quichua, Santiago del Estero'],
                    [b'quv', b'Sacapulteco'],
                    [b'quw', b'Quichua, Tena Lowland'],
                    [b'qux', b'Quechua, Yauyos'],
                    [b'quy', b'Quechua, Ayacucho'],
                    [b'quz', b'Quechua, Cusco'],
                    [b'qva', b'Quechua, Ambo-Pasco'],
                    [b'qvc', b'Quechua, Cajamarca'],
                    [b'qve', 'Quechua, Eastern Apur\xedmac'],
                    [b'qvh', 'Quechua, Huamal\xedes-Dos de Mayo Hu\xe1nuco'],
                    [b'qvi', b'Quichua, Imbabura Highland'],
                    [b'qvj', b'Quichua, Loja Highland'],
                    [b'qvl', b'Quechua, Cajatambo North Lima'],
                    [b'qvm', b'Quechua, Margos-Yarowilca-Lauricocha'],
                    [b'qvn', 'Quechua, North Jun\xedn'],
                    [b'qvo', b'Quechua, Napo Lowland'],
                    [b'qvp', b'Quechua, Pacaraos'],
                    [b'qvs', 'Quechua, San Mart\xedn'],
                    [b'qvw', b'Quechua, Huaylla Wanca'],
                    [b'qvy', b'Queyu'],
                    [b'qvz', b'Quichua, Northern Pastaza'],
                    [b'qwa', b'Quechua, Corongo Ancash'],
                    [b'qwc', b'Quechua, Classical'],
                    [b'qwh', b'Quechua, Huaylas Ancash'],
                    [b'qwm', b'Kuman (Russia)'],
                    [b'qws', b'Quechua, Sihuas Ancash'],
                    [b'qwt', b'Kwalhioqua-Tlatskanai'],
                    [b'qxa', 'Quechua, Chiqui\xe1n Ancash'],
                    [b'qxc', b'Quechua, Chincha'],
                    [b'qxh', 'Quechua, Panao Hu\xe1nuco'],
                    [b'qxl', b'Quichua, Salasaca Highland'],
                    [b'qxn', b'Quechua, Northern Conchucos Ancash'],
                    [b'qxo', b'Quechua, Southern Conchucos Ancash'],
                    [b'qxp', b'Quechua, Puno'],
                    [b'qxq', b"Qashqa'i"],
                    [b'qxr', 'Quichua, Ca\xf1ar Highland'],
                    [b'qxs', b'Qiang, Southern'],
                    [b'qxt', b'Quechua, Santa Ana de Tusi Pasco'],
                    [b'qxu', 'Quechua, Arequipa-La Uni\xf3n'],
                    [b'qxw', b'Quechua, Jauja Wanca'],
                    [b'qya', b'Quenya'],
                    [b'qyp', b'Quiripi'],
                    [b'raa', b'Dungmali'],
                    [b'rab', b'Camling'],
                    [b'rac', b'Rasawa'],
                    [b'rad', b'Rade'],
                    [b'raf', b'Meohang, Western'],
                    [b'rag', b'Logooli'],
                    [b'rah', b'Rabha'],
                    [b'rai', b'Ramoaaina'],
                    [b'raj', b'Rajasthani'],
                    [b'rak', b'Tulu-Bohuai'],
                    [b'ral', b'Ralte'],
                    [b'ram', b'Canela'],
                    [b'ran', b'Riantana'],
                    [b'rao', b'Rao'],
                    [b'rap', b'Rapanui'],
                    [b'raq', b'Saam'],
                    [b'rar', b'Rarotongan'],
                    [b'ras', b'Tegali'],
                    [b'rat', b'Razajerdi'],
                    [b'rau', b'Raute'],
                    [b'rav', b'Sampang'],
                    [b'raw', b'Rawang'],
                    [b'rax', b'Rang'],
                    [b'ray', b'Rapa'],
                    [b'raz', b'Rahambuu'],
                    [b'rbb', b'Palaung, Rumai'],
                    [b'rbk', b'Bontok, Northern'],
                    [b'rbl', b'Bikol, Miraya'],
                    [b'rbp', b'Barababaraba'],
                    [b'rcf', 'Creole French, R\xe9union'],
                    [b'rdb', b'Rudbari'],
                    [b'rea', b'Rerau'],
                    [b'reb', b'Rembong'],
                    [b'ree', b'Kayan, Rejang'],
                    [b'reg', b'Kara (Tanzania)'],
                    [b'rei', b'Reli'],
                    [b'rej', b'Rejang'],
                    [b'rel', b'Rendille'],
                    [b'rem', b'Remo'],
                    [b'ren', b'Rengao'],
                    [b'rer', b'Rer Bare'],
                    [b'res', b'Reshe'],
                    [b'ret', b'Retta'],
                    [b'rey', b'Reyesano'],
                    [b'rga', b'Roria'],
                    [b'rge', b'Romano-Greek'],
                    [b'rgk', b'Rangkas'],
                    [b'rgn', b'Romagnol'],
                    [b'rgr', 'Res\xedgaro'],
                    [b'rgs', b'Roglai, Southern'],
                    [b'rgu', b'Ringgou'],
                    [b'rhg', b'Rohingya'],
                    [b'rhp', b'Yahang'],
                    [b'ria', b'Riang (India)'],
                    [b'rie', b'Rien'],
                    [b'rif', b'Tarifit'],
                    [b'ril', b'Riang (Myanmar)'],
                    [b'rim', b'Nyaturu'],
                    [b'rin', b'Nungu'],
                    [b'rir', b'Ribun'],
                    [b'rit', b'Ritarungo'],
                    [b'riu', b'Riung'],
                    [b'rjg', b'Rajong'],
                    [b'rji', b'Raji'],
                    [b'rjs', b'Rajbanshi'],
                    [b'rka', b'Kraol'],
                    [b'rkb', b'Rikbaktsa'],
                    [b'rkh', b'Rakahanga-Manihiki'],
                    [b'rki', b'Rakhine'],
                    [b'rkm', b'Marka'],
                    [b'rkt', b'Rangpuri'],
                    [b'rkw', b'Arakwal'],
                    [b'rma', b'Rama'],
                    [b'rmb', b'Rembarunga'],
                    [b'rmc', b'Romani, Carpathian'],
                    [b'rmd', b'Danish, Traveller'],
                    [b'rme', b'Angloromani'],
                    [b'rmf', b'Romani, Kalo Finnish'],
                    [b'rmg', b'Norwegian, Traveller'],
                    [b'rmh', b'Murkim'],
                    [b'rmi', b'Lomavren'],
                    [b'rmk', b'Romkun'],
                    [b'rml', b'Romani, Baltic'],
                    [b'rmm', b'Roma'],
                    [b'rmn', b'Romani, Balkan'],
                    [b'rmo', b'Romani, Sinte'],
                    [b'rmp', b'Rempi'],
                    [b'rmq', 'Cal\xf3'],
                    [b'rms', b'Romanian Sign Language'],
                    [b'rmt', b'Domari'],
                    [b'rmu', b'Romani, Tavringer'],
                    [b'rmv', b'Romanova'],
                    [b'rmw', b'Romani, Welsh'],
                    [b'rmx', b'Romam'],
                    [b'rmy', b'Romani, Vlax'],
                    [b'rmz', b'Marma'],
                    [b'rna', b'Runa'],
                    [b'rnd', b'Ruund'],
                    [b'rng', b'Ronga'],
                    [b'rnl', b'Ranglong'],
                    [b'rnn', b'Roon'],
                    [b'rnp', b'Rongpo'],
                    [b'rnr', b'Nari Nari'],
                    [b'rnw', b'Rungwa'],
                    [b'rob', b"Tae'"],
                    [b'roc', b'Roglai, Cacgia'],
                    [b'rod', b'Rogo'],
                    [b'roe', b'Ronji'],
                    [b'rof', b'Rombo'],
                    [b'rog', b'Roglai, Northern'],
                    [b'roh', b'Romansh'],
                    [b'rol', b'Romblomanon'],
                    [b'rom', b'Romany'],
                    [b'ron', b'Romanian'],
                    [b'roo', b'Rotokas'],
                    [b'rop', b'Kriol'],
                    [b'ror', b'Rongga'],
                    [b'rou', b'Runga'],
                    [b'row', b'Dela-Oenale'],
                    [b'rpn', b'Repanbitip'],
                    [b'rpt', b'Rapting'],
                    [b'rri', b'Ririo'],
                    [b'rro', b'Waima'],
                    [b'rrt', b'Arritinngithigh'],
                    [b'rsb', b'Romano-Serbian'],
                    [b'rsi', b'Rennellese Sign Language'],
                    [b'rsl', b'Russian Sign Language'],
                    [b'rtc', b'Chin, Rungtu'],
                    [b'rth', b'Ratahan'],
                    [b'rtm', b'Rotuman'],
                    [b'rtw', b'Rathawi'],
                    [b'rub', b'Gungu'],
                    [b'ruc', b'Ruuli'],
                    [b'rue', b'Rusyn'],
                    [b'ruf', b'Luguru'],
                    [b'rug', b'Roviana'],
                    [b'ruh', b'Ruga'],
                    [b'rui', b'Rufiji'],
                    [b'ruk', b'Che'],
                    [b'run', b'Rundi'],
                    [b'ruo', b'Romanian, Istro'],
                    [b'rup', b'Romanian, Macedo-'],
                    [b'ruq', b'Romanian, Megleno'],
                    [b'rus', b'Russian'],
                    [b'rut', b'Rutul'],
                    [b'ruu', b'Lobu, Lanas'],
                    [b'ruy', b'Mala (Nigeria)'],
                    [b'ruz', b'Ruma'],
                    [b'rwa', b'Rawo'],
                    [b'rwk', b'Rwa'],
                    [b'rwm', b'Amba (Uganda)'],
                    [b'rwo', b'Rawa'],
                    [b'rwr', b'Marwari (India)'],
                    [b'rxd', b'Ngardi'],
                    [b'rxw', b'Karuwali'],
                    [b'ryn', b'Amami-Oshima, Northern'],
                    [b'rys', b'Yaeyama'],
                    [b'ryu', b'Okinawan, Central'],
                    [b'saa', b'Saba'],
                    [b'sab', b'Buglere'],
                    [b'sac', b'Meskwaki'],
                    [b'sad', b'Sandawe'],
                    [b'sae', 'Saban\xea'],
                    [b'saf', b'Safaliba'],
                    [b'sag', b'Sango'],
                    [b'sah', b'Yakut'],
                    [b'saj', b'Sahu'],
                    [b'sak', b'Sake'],
                    [b'sam', b'Aramaic, Samaritan'],
                    [b'san', b'Sanskrit'],
                    [b'sao', b'Sause'],
                    [b'sap', 'Sanapan\xe1'],
                    [b'saq', b'Samburu'],
                    [b'sar', b'Saraveca'],
                    [b'sas', b'Sasak'],
                    [b'sat', b'Santali'],
                    [b'sau', b'Saleman'],
                    [b'sav', b'Saafi-Saafi'],
                    [b'saw', b'Sawi'],
                    [b'sax', b'Sa'],
                    [b'say', b'Saya'],
                    [b'saz', b'Saurashtra'],
                    [b'sba', b'Ngambay'],
                    [b'sbb', b'Simbo'],
                    [b'sbc', b'Kele (Papua New Guinea)'],
                    [b'sbd', b'Samo, Southern'],
                    [b'sbe', b'Saliba'],
                    [b'sbf', b'Shabo'],
                    [b'sbg', b'Seget'],
                    [b'sbh', b'Sori-Harengan'],
                    [b'sbi', b'Seti'],
                    [b'sbj', b'Surbakhal'],
                    [b'sbk', b'Safwa'],
                    [b'sbl', b'Sambal, Botolan'],
                    [b'sbm', b'Sagala'],
                    [b'sbn', b'Bhil, Sindhi'],
                    [b'sbo', 'Sab\xfcm'],
                    [b'sbp', b'Sangu (Tanzania)'],
                    [b'sbq', b'Sileibi'],
                    [b'sbr', b'Sembakung Murut'],
                    [b'sbs', b'Subiya'],
                    [b'sbt', b'Kimki'],
                    [b'sbu', b'Bhoti, Stod'],
                    [b'sbv', b'Sabine'],
                    [b'sbw', b'Simba'],
                    [b'sbx', b'Seberuang'],
                    [b'sby', b'Soli'],
                    [b'sbz', b'Sara Kaba'],
                    [b'scb', b'Chut'],
                    [b'sce', b'Dongxiang'],
                    [b'scf', b'Creole French, San Miguel'],
                    [b'scg', b'Sanggau'],
                    [b'sch', b'Sakachep'],
                    [b'sci', b'Creole Malay, Sri Lankan'],
                    [b'sck', b'Sadri'],
                    [b'scl', b'Shina'],
                    [b'scn', b'Sicilian'],
                    [b'sco', b'Scots'],
                    [b'scp', b'Helambu Sherpa'],
                    [b'scq', b"Sa'och"],
                    [b'scs', b'Slavey, North'],
                    [b'scu', b'Shumcho'],
                    [b'scv', b'Sheni'],
                    [b'scw', b'Sha'],
                    [b'scx', b'Sicel'],
                    [b'sda', b"Toraja-Sa'dan"],
                    [b'sdb', b'Shabak'],
                    [b'sdc', b'Sardinian, Sassarese'],
                    [b'sde', b'Surubu'],
                    [b'sdf', b'Sarli'],
                    [b'sdg', b'Savi'],
                    [b'sdh', b'Kurdish, Southern'],
                    [b'sdj', b'Suundi'],
                    [b'sdk', b'Sos Kundi'],
                    [b'sdl', b'Saudi Arabian Sign Language'],
                    [b'sdm', b'Semandang'],
                    [b'sdn', b'Sardinian, Gallurese'],
                    [b'sdo', b'Bidayuh, Bukar-Sadung'],
                    [b'sdp', b'Sherdukpen'],
                    [b'sdr', b'Sadri, Oraon'],
                    [b'sds', b'Sened'],
                    [b'sdt', b'Shuadit'],
                    [b'sdu', b'Sarudu'],
                    [b'sdx', b'Melanau, Sibu'],
                    [b'sdz', b'Sallands'],
                    [b'sea', b'Semai'],
                    [b'seb', b'Senoufo, Shempire'],
                    [b'sec', b'Sechelt'],
                    [b'sed', b'Sedang'],
                    [b'see', b'Seneca'],
                    [b'sef', b'Senoufo, Cebaara'],
                    [b'seg', b'Segeju'],
                    [b'seh', b'Sena'],
                    [b'sei', b'Seri'],
                    [b'sej', b'Sene'],
                    [b'sek', b'Sekani'],
                    [b'sel', b'Selkup'],
                    [b'sen', 'S\xe9noufo, Nanerig\xe9'],
                    [b'seo', b'Suarmin'],
                    [b'sep', 'S\xe9noufo, S\xecc\xect\xe9'],
                    [b'seq', 'S\xe9noufo, Senara'],
                    [b'ser', b'Serrano'],
                    [b'ses', b'Songhai, Koyraboro Senni'],
                    [b'set', b'Sentani'],
                    [b'seu', b'Serui-Laut'],
                    [b'sev', b'Senoufo, Nyarafolo'],
                    [b'sew', b'Sewa Bay'],
                    [b'sey', b'Secoya'],
                    [b'sez', b'Chin, Senthang'],
                    [b'sfb', b'Langue des signes de Belgique Francophone'],
                    [b'sfe', b'Subanen, Eastern'],
                    [b'sfm', b'Miao, Small Flowery'],
                    [b'sfs', b'South African Sign Language'],
                    [b'sfw', b'Sehwi'],
                    [b'sga', b'Irish, Old (to 900)'],
                    [b'sgb', b'Ayta, Mag-antsi'],
                    [b'sgc', b'Kipsigis'],
                    [b'sgd', b'Surigaonon'],
                    [b'sge', b'Segai'],
                    [b'sgg', b'Swiss-German Sign Language'],
                    [b'sgh', b'Shughni'],
                    [b'sgi', b'Suga'],
                    [b'sgj', b'Surgujia'],
                    [b'sgk', b'Sangkong'],
                    [b'sgm', b'Singa'],
                    [b'sgo', b'Songa'],
                    [b'sgp', b'Singpho'],
                    [b'sgr', b'Sangisari'],
                    [b'sgs', b'Samogitian'],
                    [b'sgt', b'Brokpake'],
                    [b'sgu', b'Salas'],
                    [b'sgw', b'Sebat Bet Gurage'],
                    [b'sgx', b'Sierra Leone Sign Language'],
                    [b'sgy', b'Sanglechi'],
                    [b'sgz', b'Sursurunga'],
                    [b'sha', b'Shall-Zwall'],
                    [b'shb', b'Ninam'],
                    [b'shc', b'Sonde'],
                    [b'shd', b'Kundal Shahi'],
                    [b'she', b'Sheko'],
                    [b'shg', b'Shua'],
                    [b'shh', b'Shoshoni'],
                    [b'shi', b'Tachelhit'],
                    [b'shj', b'Shatt'],
                    [b'shk', b'Shilluk'],
                    [b'shl', b'Shendu'],
                    [b'shm', b'Shahrudi'],
                    [b'shn', b'Shan'],
                    [b'sho', b'Shanga'],
                    [b'shp', b'Shipibo-Conibo'],
                    [b'shq', b'Sala'],
                    [b'shr', b'Shi'],
                    [b'shs', b'Shuswap'],
                    [b'sht', b'Shasta'],
                    [b'shu', b'Arabic, Chadian'],
                    [b'shv', b'Shehri'],
                    [b'shw', b'Shwai'],
                    [b'shx', b'She'],
                    [b'shy', b'Tachawit'],
                    [b'shz', b'Senoufo, Syenara'],
                    [b'sia', b'Sami, Akkala'],
                    [b'sib', b'Sebop'],
                    [b'sid', b'Sidamo'],
                    [b'sie', b'Simaa'],
                    [b'sif', b'Siamou'],
                    [b'sig', b'Paasaal'],
                    [b'sih', b'Zire'],
                    [b'sii', b'Shom Peng'],
                    [b'sij', b'Numbami'],
                    [b'sik', b'Sikiana'],
                    [b'sil', b'Sisaala, Tumulung'],
                    [b'sim', b'Mende (Papua New Guinea)'],
                    [b'sin', b'Sinhala'],
                    [b'sip', b'Sikkimese'],
                    [b'siq', b'Sonia'],
                    [b'sir', b'Siri'],
                    [b'sis', b'Siuslaw'],
                    [b'siu', b'Sinagen'],
                    [b'siv', b'Sumariup'],
                    [b'siw', b'Siwai'],
                    [b'six', b'Sumau'],
                    [b'siy', b'Sivandi'],
                    [b'siz', b'Siwi'],
                    [b'sja', b'Epena'],
                    [b'sjb', b'Sajau Basap'],
                    [b'sjd', b'Sami, Kildin'],
                    [b'sje', b'Sami, Pite'],
                    [b'sjg', b'Assangori'],
                    [b'sjk', b'Sami, Kemi'],
                    [b'sjl', b'Sajalong'],
                    [b'sjm', b'Mapun'],
                    [b'sjn', b'Sindarin'],
                    [b'sjo', b'Xibe'],
                    [b'sjp', b'Surjapuri'],
                    [b'sjr', b'Siar-Lak'],
                    [b'sjs', b'Senhaja De Srair'],
                    [b'sjt', b'Sami, Ter'],
                    [b'sju', b'Sami, Ume'],
                    [b'sjw', b'Shawnee'],
                    [b'ska', b'Skagit'],
                    [b'skb', b'Saek'],
                    [b'skc', b'Ma Manda'],
                    [b'skd', b'Miwok, Southern Sierra'],
                    [b'ske', b'Seke (Vanuatu)'],
                    [b'skf', 'Sakirabi\xe1'],
                    [b'skg', b'Malagasy, Sakalava'],
                    [b'skh', b'Sikule'],
                    [b'ski', b'Sika'],
                    [b'skj', b'Seke (Nepal)'],
                    [b'skk', b'Sok'],
                    [b'skm', b'Kutong'],
                    [b'skn', b'Subanon, Kolibugan'],
                    [b'sko', b'Seko Tengah'],
                    [b'skp', b'Sekapan'],
                    [b'skq', b'Sininkere'],
                    [b'skr', b'Seraiki'],
                    [b'sks', b'Maia'],
                    [b'skt', b'Sakata'],
                    [b'sku', b'Sakao'],
                    [b'skv', b'Skou'],
                    [b'skw', b'Creole Dutch, Skepi'],
                    [b'skx', b'Seko Padang'],
                    [b'sky', b'Sikaiana'],
                    [b'skz', b'Sekar'],
                    [b'slc', 'S\xe1liba'],
                    [b'sld', b'Sissala'],
                    [b'sle', b'Sholaga'],
                    [b'slf', b'Swiss-Italian Sign Language'],
                    [b'slg', b'Selungai Murut'],
                    [b'slh', b'Salish, Southern Puget Sound'],
                    [b'sli', b'Silesian, Lower'],
                    [b'slj', 'Salum\xe1'],
                    [b'slk', b'Slovak'],
                    [b'sll', b'Salt-Yui'],
                    [b'slm', b'Sama, Pangutaran'],
                    [b'sln', b'Salinan'],
                    [b'slp', b'Lamaholot'],
                    [b'slq', b'Salchuq'],
                    [b'slr', b'Salar'],
                    [b'sls', b'Singapore Sign Language'],
                    [b'slt', b'Sila'],
                    [b'slu', b'Selaru'],
                    [b'slv', b'Slovenian'],
                    [b'slw', b'Sialum'],
                    [b'slx', b'Salampasu'],
                    [b'sly', b'Selayar'],
                    [b'slz', b"Ma'ya"],
                    [b'sma', b'Sami, Southern'],
                    [b'smb', b'Simbari'],
                    [b'smc', b'Som'],
                    [b'smd', b'Sama'],
                    [b'sme', b'Sami, Northern'],
                    [b'smf', b'Auwe'],
                    [b'smg', b'Simbali'],
                    [b'smh', b'Samei'],
                    [b'smj', b'Lule Sami'],
                    [b'smk', b'Bolinao'],
                    [b'sml', b'Sama, Central'],
                    [b'smm', b'Musasa'],
                    [b'smn', b'Sami, Inari'],
                    [b'smo', b'Samoan'],
                    [b'smp', b'Samaritan'],
                    [b'smq', b'Samo'],
                    [b'smr', b'Simeulue'],
                    [b'sms', b'Sami, Skolt'],
                    [b'smt', b'Simte'],
                    [b'smu', b'Somray'],
                    [b'smv', b'Samvedi'],
                    [b'smw', b'Sumbawa'],
                    [b'smx', b'Samba'],
                    [b'smy', b'Semnani'],
                    [b'smz', b'Simeku'],
                    [b'sna', b'Shona'],
                    [b'snb', b'Sebuyau'],
                    [b'snc', b'Sinaugoro'],
                    [b'snd', b'Sindhi'],
                    [b'sne', b'Bidayuh, Bau'],
                    [b'snf', b'Noon'],
                    [b'sng', b'Sanga (Democratic Republic of Congo)'],
                    [b'snh', b'Shinabo'],
                    [b'sni', b'Sensi'],
                    [b'snj', b'Sango, Riverain'],
                    [b'snk', b'Soninke'],
                    [b'snl', b'Sangil'],
                    [b'snm', b"Ma'di, Southern"],
                    [b'snn', b'Siona'],
                    [b'sno', b'Snohomish'],
                    [b'snp', b'Siane'],
                    [b'snq', b'Sangu (Gabon)'],
                    [b'snr', b'Sihan'],
                    [b'sns', b'South West Bay'],
                    [b'snu', b'Senggi'],
                    [b'snv', b"Sa'ban"],
                    [b'snw', b'Selee'],
                    [b'snx', b'Sam'],
                    [b'sny', b'Saniyo-Hiyewe'],
                    [b'snz', b'Sinsauru'],
                    [b'soa', b'Thai Song'],
                    [b'sob', b'Sobei'],
                    [b'soc', b'So (Democratic Republic of Congo)'],
                    [b'sod', b'Songoora'],
                    [b'soe', b'Songomeno'],
                    [b'sog', b'Sogdian'],
                    [b'soh', b'Aka'],
                    [b'soi', b'Sonha'],
                    [b'soj', b'Soi'],
                    [b'sok', b'Sokoro'],
                    [b'sol', b'Solos'],
                    [b'som', b'Somali'],
                    [b'soo', b'Songo'],
                    [b'sop', b'Songe'],
                    [b'soq', b'Kanasi'],
                    [b'sor', b'Somrai'],
                    [b'sos', b'Seeku'],
                    [b'sot', b'Sotho, Southern'],
                    [b'sou', b'Thai, Southern'],
                    [b'sov', b'Sonsorol'],
                    [b'sow', b'Sowanda'],
                    [b'sox', b'Swo'],
                    [b'soy', b'Miyobe'],
                    [b'soz', b'Temi'],
                    [b'spa', b'Spanish'],
                    [b'spb', b'Sepa (Indonesia)'],
                    [b'spc', 'Sap\xe9'],
                    [b'spd', b'Saep'],
                    [b'spe', b'Sepa (Papua New Guinea)'],
                    [b'spg', b'Sian'],
                    [b'spi', b'Saponi'],
                    [b'spk', b'Sengo'],
                    [b'spl', b'Selepet'],
                    [b'spm', b'Akukem'],
                    [b'spo', b'Spokane'],
                    [b'spp', b'Senoufo, Supyire'],
                    [b'spq', b'Spanish, Loreto-Ucayali'],
                    [b'spr', b'Saparua'],
                    [b'sps', b'Saposa'],
                    [b'spt', b'Bhoti, Spiti'],
                    [b'spu', b'Sapuan'],
                    [b'spv', b'Sambalpuri'],
                    [b'spx', b'Picene, South'],
                    [b'spy', b'Sabaot'],
                    [b'sqa', b'Shama-Sambuga'],
                    [b'sqh', b'Shau'],
                    [b'sqi', b'Albanian'],
                    [b'sqk', b'Albanian Sign Language'],
                    [b'sqm', b'Suma'],
                    [b'sqn', b'Susquehannock'],
                    [b'sqo', b'Sorkhei'],
                    [b'sqq', b'Sou'],
                    [b'sqr', b'Arabic, Siculo'],
                    [b'sqs', b'Sri Lankan Sign Language'],
                    [b'sqt', b'Soqotri'],
                    [b'squ', b'Squamish'],
                    [b'sra', b'Saruga'],
                    [b'srb', b'Sora'],
                    [b'src', b'Sardinian, Logudorese'],
                    [b'srd', b'Sardinian'],
                    [b'sre', b'Sara'],
                    [b'srf', b'Nafi'],
                    [b'srg', b'Sulod'],
                    [b'srh', b'Sarikoli'],
                    [b'sri', b'Siriano'],
                    [b'srk', b'Serudung Murut'],
                    [b'srl', b'Isirawa'],
                    [b'srm', b'Saramaccan'],
                    [b'srn', b'Sranan Tongo'],
                    [b'sro', b'Sardinian, Campidanese'],
                    [b'srp', b'Serbian'],
                    [b'srq', 'Sirion\xf3'],
                    [b'srr', b'Serer'],
                    [b'srs', b'Sarsi'],
                    [b'srt', b'Sauri'],
                    [b'sru', 'Suru\xed'],
                    [b'srv', b'Sorsoganon, Southern'],
                    [b'srw', b'Serua'],
                    [b'srx', b'Sirmauri'],
                    [b'sry', b'Sera'],
                    [b'srz', b'Shahmirzadi'],
                    [b'ssb', b'Sama, Southern'],
                    [b'ssc', b'Suba-Simbiti'],
                    [b'ssd', b'Siroi'],
                    [b'sse', b'Balangingi'],
                    [b'ssf', b'Thao'],
                    [b'ssg', b'Seimat'],
                    [b'ssh', b'Arabic, Shihhi'],
                    [b'ssi', b'Sansi'],
                    [b'ssj', b'Sausi'],
                    [b'ssk', b'Sunam'],
                    [b'ssl', b'Sisaala, Western'],
                    [b'ssm', b'Semnam'],
                    [b'ssn', b'Waata'],
                    [b'sso', b'Sissano'],
                    [b'ssp', b'Spanish Sign Language'],
                    [b'ssq', b"So'a"],
                    [b'ssr', b'Swiss-French Sign Language'],
                    [b'sss', 'S\xf4'],
                    [b'sst', b'Sinasina'],
                    [b'ssu', b'Susuami'],
                    [b'ssv', b'Shark Bay'],
                    [b'ssw', b'Swati'],
                    [b'ssx', b'Samberigi'],
                    [b'ssy', b'Saho'],
                    [b'ssz', b'Sengseng'],
                    [b'sta', b'Settla'],
                    [b'stb', b'Subanen, Northern'],
                    [b'std', b'Sentinel'],
                    [b'ste', b'Liana-Seti'],
                    [b'stf', b'Seta'],
                    [b'stg', b'Trieng'],
                    [b'sth', b'Shelta'],
                    [b'sti', b'Stieng, Bulo'],
                    [b'stj', b'Samo, Matya'],
                    [b'stk', b'Arammba'],
                    [b'stl', b'Stellingwerfs'],
                    [b'stm', b'Setaman'],
                    [b'stn', b'Owa'],
                    [b'sto', b'Stoney'],
                    [b'stp', b'Tepehuan, Southeastern'],
                    [b'stq', b'Saterfriesisch'],
                    [b'str', b'Salish, Straits'],
                    [b'sts', b'Shumashti'],
                    [b'stt', b'Stieng, Budeh'],
                    [b'stu', b'Samtao'],
                    [b'stv', b"Silt'e"],
                    [b'stw', b'Satawalese'],
                    [b'sty', b'Tatar, Siberian'],
                    [b'sua', b'Sulka'],
                    [b'sub', b'Suku'],
                    [b'suc', b'Subanon, Western'],
                    [b'sue', b'Suena'],
                    [b'sug', b'Suganga'],
                    [b'sui', b'Suki'],
                    [b'suj', b'Shubi'],
                    [b'suk', b'Sukuma'],
                    [b'sun', b'Sundanese'],
                    [b'suq', b'Suri'],
                    [b'sur', b'Mwaghavul'],
                    [b'sus', b'Susu'],
                    [b'sut', b'Subtiaba'],
                    [b'suv', b'Puroik'],
                    [b'suw', b'Sumbwa'],
                    [b'sux', b'Sumerian'],
                    [b'suy', 'Suy\xe1'],
                    [b'suz', b'Sunwar'],
                    [b'sva', b'Svan'],
                    [b'svb', b'Ulau-Suain'],
                    [b'svc', b'Creole English, Vincentian'],
                    [b'sve', b'Serili'],
                    [b'svk', b'Slovakian Sign Language'],
                    [b'svm', b'Slavomolisano'],
                    [b'svr', b'Savara'],
                    [b'svs', b'Savosavo'],
                    [b'svx', b'Skalvian'],
                    [b'swa', b'Swahili (macrolanguage)'],
                    [b'swb', b'Comorian, Maore'],
                    [b'swc', b'Swahili, Congo'],
                    [b'swe', b'Swedish'],
                    [b'swf', b'Sere'],
                    [b'swg', b'Swabian'],
                    [b'swh', b'Swahili (individual language)'],
                    [b'swi', b'Sui'],
                    [b'swj', b'Sira'],
                    [b'swk', b'Sena, Malawi'],
                    [b'swl', b'Swedish Sign Language'],
                    [b'swm', b'Samosa'],
                    [b'swn', b'Sawknah'],
                    [b'swo', b'Shanenawa'],
                    [b'swp', b'Suau'],
                    [b'swq', b'Sharwa'],
                    [b'swr', b'Saweru'],
                    [b'sws', b'Seluwasan'],
                    [b'swt', b'Sawila'],
                    [b'swu', b'Suwawa'],
                    [b'swv', b'Shekhawati'],
                    [b'sww', b'Sowa'],
                    [b'swx', 'Suruah\xe1'],
                    [b'swy', b'Sarua'],
                    [b'sxb', b'Suba'],
                    [b'sxc', b'Sicanian'],
                    [b'sxe', b'Sighu'],
                    [b'sxg', b'Shixing'],
                    [b'sxk', b'Kalapuya, Southern'],
                    [b'sxl', b'Selian'],
                    [b'sxm', b'Samre'],
                    [b'sxn', b'Sangir'],
                    [b'sxo', b'Sorothaptic'],
                    [b'sxr', b'Saaroa'],
                    [b'sxs', b'Sasaru'],
                    [b'sxu', b'Saxon, Upper'],
                    [b'sxw', b'Gbe, Saxwe'],
                    [b'sya', b'Siang'],
                    [b'syb', b'Subanen, Central'],
                    [b'syc', b'Syriac, Classical'],
                    [b'syi', b'Seki'],
                    [b'syk', b'Sukur'],
                    [b'syl', b'Sylheti'],
                    [b'sym', b'Samo, Maya'],
                    [b'syn', b'Senaya'],
                    [b'syo', b'Suoy'],
                    [b'syr', b'Syriac'],
                    [b'sys', b'Sinyar'],
                    [b'syw', b'Kagate'],
                    [b'syy', b'Al-Sayyid Bedouin Sign Language'],
                    [b'sza', b'Semelai'],
                    [b'szb', b'Ngalum'],
                    [b'szc', b'Semaq Beri'],
                    [b'szd', b'Seru'],
                    [b'sze', b'Seze'],
                    [b'szg', b'Sengele'],
                    [b'szl', b'Silesian'],
                    [b'szn', b'Sula'],
                    [b'szp', b'Suabo'],
                    [b'szv', b'Isu (Fako Division)'],
                    [b'szw', b'Sawai'],
                    [b'taa', b'Tanana, Lower'],
                    [b'tab', b'Tabassaran'],
                    [b'tac', b'Tarahumara, Lowland'],
                    [b'tad', b'Tause'],
                    [b'tae', b'Tariana'],
                    [b'taf', 'Tapirap\xe9'],
                    [b'tag', b'Tagoi'],
                    [b'tah', b'Tahitian'],
                    [b'taj', b'Tamang, Eastern'],
                    [b'tak', b'Tala'],
                    [b'tal', b'Tal'],
                    [b'tam', b'Tamil'],
                    [b'tan', b'Tangale'],
                    [b'tao', b'Yami'],
                    [b'tap', b'Taabwa'],
                    [b'taq', b'Tamasheq'],
                    [b'tar', b'Tarahumara, Central'],
                    [b'tas', b'Tay Boi'],
                    [b'tat', b'Tatar'],
                    [b'tau', b'Tanana, Upper'],
                    [b'tav', b'Tatuyo'],
                    [b'taw', b'Tai'],
                    [b'tax', b'Tamki'],
                    [b'tay', b'Atayal'],
                    [b'taz', b'Tocho'],
                    [b'tba', 'Aikan\xe3'],
                    [b'tbb', b'Tapeba'],
                    [b'tbc', b'Takia'],
                    [b'tbd', b'Kaki Ae'],
                    [b'tbe', b'Tanimbili'],
                    [b'tbf', b'Mandara'],
                    [b'tbg', b'Tairora, North'],
                    [b'tbh', b'Thurawal'],
                    [b'tbi', b'Gaam'],
                    [b'tbj', b'Tiang'],
                    [b'tbk', b'Tagbanwa, Calamian'],
                    [b'tbl', b'Tboli'],
                    [b'tbm', b'Tagbu'],
                    [b'tbn', b'Tunebo, Barro Negro'],
                    [b'tbo', b'Tawala'],
                    [b'tbp', b'Taworta'],
                    [b'tbr', b'Tumtum'],
                    [b'tbs', b'Tanguat'],
                    [b'tbt', b'Tembo (Kitembo)'],
                    [b'tbu', b'Tubar'],
                    [b'tbv', b'Tobo'],
                    [b'tbw', b'Tagbanwa'],
                    [b'tbx', b'Kapin'],
                    [b'tby', b'Tabaru'],
                    [b'tbz', b'Ditammari'],
                    [b'tca', b'Ticuna'],
                    [b'tcb', b'Tanacross'],
                    [b'tcc', b'Datooga'],
                    [b'tcd', b'Tafi'],
                    [b'tce', b'Tutchone, Southern'],
                    [b'tcf', b"Me'phaa, Malinaltepec"],
                    [b'tcg', b'Tamagario'],
                    [b'tch', b'Creole English, Turks And Caicos'],
                    [b'tci', 'W\xe1ra'],
                    [b'tck', b'Tchitchege'],
                    [b'tcl', b'Taman (Myanmar)'],
                    [b'tcm', b'Tanahmerah'],
                    [b'tcn', b'Tichurong'],
                    [b'tco', b'Taungyo'],
                    [b'tcp', b'Chin, Tawr'],
                    [b'tcq', b'Kaiy'],
                    [b'tcs', b'Creole, Torres Strait'],
                    [b'tct', b"T'en"],
                    [b'tcu', b'Tarahumara, Southeastern'],
                    [b'tcw', 'Totonac, Tecpatl\xe1n'],
                    [b'tcx', b'Toda'],
                    [b'tcy', b'Tulu'],
                    [b'tcz', b'Chin, Thado'],
                    [b'tda', b'Tagdal'],
                    [b'tdb', b'Panchpargania'],
                    [b'tdc', 'Ember\xe1-Tad\xf3'],
                    [b'tdd', 'Tai N\xfca'],
                    [b'tde', b'Dogon, Tiranige Diga'],
                    [b'tdf', b'Talieng'],
                    [b'tdg', b'Tamang, Western'],
                    [b'tdh', b'Thulung'],
                    [b'tdi', b'Tomadino'],
                    [b'tdj', b'Tajio'],
                    [b'tdk', b'Tambas'],
                    [b'tdl', b'Sur'],
                    [b'tdn', b'Tondano'],
                    [b'tdo', b'Teme'],
                    [b'tdq', b'Tita'],
                    [b'tdr', b'Todrah'],
                    [b'tds', b'Doutai'],
                    [b'tdt', b'Tetun Dili'],
                    [b'tdu', b'Dusun, Tempasuk'],
                    [b'tdv', b'Toro'],
                    [b'tdx', b'Malagasy, Tandroy-Mahafaly'],
                    [b'tdy', b'Tadyawan'],
                    [b'tea', b'Temiar'],
                    [b'teb', b'Tetete'],
                    [b'tec', b'Terik'],
                    [b'ted', b'Krumen, Tepo'],
                    [b'tee', b'Tepehua, Huehuetla'],
                    [b'tef', b'Teressa'],
                    [b'teg', b'Teke-Tege'],
                    [b'teh', b'Tehuelche'],
                    [b'tei', b'Torricelli'],
                    [b'tek', b'Teke, Ibali'],
                    [b'tel', b'Telugu'],
                    [b'tem', b'Timne'],
                    [b'ten', b'Tama (Colombia)'],
                    [b'teo', b'Teso'],
                    [b'tep', b'Tepecano'],
                    [b'teq', b'Temein'],
                    [b'ter', b'Tereno'],
                    [b'tes', b'Tengger'],
                    [b'tet', b'Tetum'],
                    [b'teu', b'Soo'],
                    [b'tev', b'Teor'],
                    [b'tew', b'Tewa (USA)'],
                    [b'tex', b'Tennet'],
                    [b'tey', b'Tulishi'],
                    [b'tfi', b'Gbe, Tofin'],
                    [b'tfn', b'Tanaina'],
                    [b'tfo', b'Tefaro'],
                    [b'tfr', b'Teribe'],
                    [b'tft', b'Ternate'],
                    [b'tga', b'Sagalla'],
                    [b'tgb', b'Tobilung'],
                    [b'tgc', b'Tigak'],
                    [b'tgd', b'Ciwogai'],
                    [b'tge', b'Tamang, Eastern Gorkha'],
                    [b'tgf', b'Chalikha'],
                    [b'tgh', b'Creole English, Tobagonian'],
                    [b'tgi', b'Lawunuia'],
                    [b'tgj', b'Tagin'],
                    [b'tgk', b'Tajik'],
                    [b'tgl', b'Tagalog'],
                    [b'tgn', b'Tandaganon'],
                    [b'tgo', b'Sudest'],
                    [b'tgp', b'Tangoa'],
                    [b'tgq', b'Tring'],
                    [b'tgr', b'Tareng'],
                    [b'tgs', b'Nume'],
                    [b'tgt', b'Tagbanwa, Central'],
                    [b'tgu', b'Tanggu'],
                    [b'tgv', b'Tingui-Boto'],
                    [b'tgw', b'Senoufo, Tagwana'],
                    [b'tgx', b'Tagish'],
                    [b'tgy', b'Togoyo'],
                    [b'tgz', b'Tagalaka'],
                    [b'tha', b'Thai'],
                    [b'thc', b'Tai Hang Tong'],
                    [b'thd', b'Thayore'],
                    [b'the', b'Tharu, Chitwania'],
                    [b'thf', b'Thangmi'],
                    [b'thh', b'Tarahumara, Northern'],
                    [b'thi', b'Tai Long'],
                    [b'thk', b'Tharaka'],
                    [b'thl', b'Tharu, Dangaura'],
                    [b'thm', b'Aheu'],
                    [b'thn', b'Thachanadan'],
                    [b'thp', b'Thompson'],
                    [b'thq', b'Tharu, Kochila'],
                    [b'thr', b'Tharu, Rana'],
                    [b'ths', b'Thakali'],
                    [b'tht', b'Tahltan'],
                    [b'thu', b'Thuri'],
                    [b'thv', b'Tamahaq, Tahaggart'],
                    [b'thw', b'Thudam'],
                    [b'thx', b'The'],
                    [b'thy', b'Tha'],
                    [b'thz', b'Tamajeq, Tayart'],
                    [b'tia', b'Tamazight, Tidikelt'],
                    [b'tic', b'Tira'],
                    [b'tid', b'Tidong'],
                    [b'tif', b'Tifal'],
                    [b'tig', b'Tigre'],
                    [b'tih', b'Murut, Timugon'],
                    [b'tii', b'Tiene'],
                    [b'tij', b'Tilung'],
                    [b'tik', b'Tikar'],
                    [b'til', b'Tillamook'],
                    [b'tim', b'Timbe'],
                    [b'tin', b'Tindi'],
                    [b'tio', b'Teop'],
                    [b'tip', b'Trimuris'],
                    [b'tiq', 'Ti\xe9fo'],
                    [b'tir', b'Tigrinya'],
                    [b'tis', b'Itneg, Masadiit'],
                    [b'tit', b'Tinigua'],
                    [b'tiu', b'Adasen'],
                    [b'tiv', b'Tiv'],
                    [b'tiw', b'Tiwi'],
                    [b'tix', b'Tiwa, Southern'],
                    [b'tiy', b'Tiruray'],
                    [b'tiz', b'Tai Hongjin'],
                    [b'tja', b'Tajuasohn'],
                    [b'tjg', b'Tunjung'],
                    [b'tji', b'Tujia, Northern'],
                    [b'tjl', b'Tai Laing'],
                    [b'tjm', b'Timucua'],
                    [b'tjn', b'Tonjon'],
                    [b'tjo', b'Tamazight, Temacine'],
                    [b'tjs', b'Tujia, Southern'],
                    [b'tju', b'Tjurruru'],
                    [b'tjw', b'Djabwurrung'],
                    [b'tka', 'Truk\xe1'],
                    [b'tkb', b'Buksa'],
                    [b'tkd', b'Tukudede'],
                    [b'tke', b'Takwane'],
                    [b'tkf', 'Tukumanf\xe9d'],
                    [b'tkg', b'Malagasy, Tesaka'],
                    [b'tkl', b'Tokelau'],
                    [b'tkm', b'Takelma'],
                    [b'tkn', b'Toku-No-Shima'],
                    [b'tkp', b'Tikopia'],
                    [b'tkq', b'Tee'],
                    [b'tkr', b'Tsakhur'],
                    [b'tks', b'Takestani'],
                    [b'tkt', b'Tharu, Kathoriya'],
                    [b'tku', b'Totonac, Upper Necaxa'],
                    [b'tkw', b'Teanu'],
                    [b'tkx', b'Tangko'],
                    [b'tkz', b'Takua'],
                    [b'tla', b'Tepehuan, Southwestern'],
                    [b'tlb', b'Tobelo'],
                    [b'tlc', b'Totonac, Yecuatla'],
                    [b'tld', b'Talaud'],
                    [b'tlf', b'Telefol'],
                    [b'tlg', b'Tofanma'],
                    [b'tlh', b'Klingon'],
                    [b'tli', b'Tlingit'],
                    [b'tlj', b'Talinga-Bwisi'],
                    [b'tlk', b'Taloki'],
                    [b'tll', b'Tetela'],
                    [b'tlm', b'Tolomako'],
                    [b'tln', b"Talondo'"],
                    [b'tlo', b'Talodi'],
                    [b'tlp', 'Totonac, Filomena Mata-Coahuitl\xe1n'],
                    [b'tlq', b'Tai Loi'],
                    [b'tlr', b'Talise'],
                    [b'tls', b'Tambotalo'],
                    [b'tlt', b'Teluti'],
                    [b'tlu', b'Tulehu'],
                    [b'tlv', b'Taliabu'],
                    [b'tlx', b'Khehek'],
                    [b'tly', b'Talysh'],
                    [b'tma', b'Tama (Chad)'],
                    [b'tmb', b'Katbol'],
                    [b'tmc', b'Tumak'],
                    [b'tmd', b'Haruai'],
                    [b'tme', 'Trememb\xe9'],
                    [b'tmf', b'Toba-Maskoy'],
                    [b'tmg', 'Ternate\xf1o'],
                    [b'tmh', b'Tamashek'],
                    [b'tmi', b'Tutuba'],
                    [b'tmj', b'Samarokena'],
                    [b'tmk', b'Tamang, Northwestern'],
                    [b'tml', b'Citak, Tamnim'],
                    [b'tmm', b'Tai Thanh'],
                    [b'tmn', b'Taman (Indonesia)'],
                    [b'tmo', b'Temoq'],
                    [b'tmp', 'Tai M\xe8ne'],
                    [b'tmq', b'Tumleo'],
                    [b'tmr', b'Aramaic, Jewish Babylonian (ca. 200-1200 CE)'],
                    [b'tms', b'Tima'],
                    [b'tmt', b'Tasmate'],
                    [b'tmu', b'Iau'],
                    [b'tmv', b'Tembo (Motembo)'],
                    [b'tmw', b'Temuan'],
                    [b'tmy', b'Tami'],
                    [b'tmz', b'Tamanaku'],
                    [b'tna', b'Tacana'],
                    [b'tnb', b'Tunebo, Western'],
                    [b'tnc', 'Tanimuca-Retuar\xe3'],
                    [b'tnd', b'Tunebo, Angosturas'],
                    [b'tne', b'Kallahan, Tinoc'],
                    [b'tng', b'Tobanga'],
                    [b'tnh', b'Maiani'],
                    [b'tni', b'Tandia'],
                    [b'tnk', b'Kwamera'],
                    [b'tnl', b'Lenakel'],
                    [b'tnm', b'Tabla'],
                    [b'tnn', b'Tanna, North'],
                    [b'tno', b'Toromono'],
                    [b'tnp', b'Whitesands'],
                    [b'tnq', b'Taino'],
                    [b'tnr', 'M\xe9nik'],
                    [b'tns', b'Tenis'],
                    [b'tnt', b'Tontemboan'],
                    [b'tnu', b'Tay Khang'],
                    [b'tnv', b'Tangchangya'],
                    [b'tnw', b'Tonsawang'],
                    [b'tnx', b'Tanema'],
                    [b'tny', b'Tongwe'],
                    [b'tnz', b'Tonga (Thailand)'],
                    [b'tob', b'Toba'],
                    [b'toc', b'Totonac, Coyutla'],
                    [b'tod', b'Toma'],
                    [b'toe', b'Tomedes'],
                    [b'tof', b'Gizrra'],
                    [b'tog', b'Tonga (Nyasa)'],
                    [b'toh', b'Gitonga'],
                    [b'toi', b'Tonga (Zambia)'],
                    [b'toj', b'Tojolabal'],
                    [b'tol', b'Tolowa'],
                    [b'tom', b'Tombulu'],
                    [b'ton', b'Tonga (Tonga Islands)'],
                    [b'too', 'Totonac, Xicotepec De Ju\xe1rez'],
                    [b'top', b'Totonac, Papantla'],
                    [b'toq', b'Toposa'],
                    [b'tor', b'Banda, Togbo-Vara'],
                    [b'tos', b'Totonac, Highland'],
                    [b'tou', b'Tho'],
                    [b'tov', b'Taromi, Upper'],
                    [b'tow', b'Jemez'],
                    [b'tox', b'Tobian'],
                    [b'toy', b'Topoiyo'],
                    [b'toz', b'To'],
                    [b'tpa', b'Taupota'],
                    [b'tpc', "Me'phaa, Azoy\xfa"],
                    [b'tpe', b'Tippera'],
                    [b'tpf', b'Tarpia'],
                    [b'tpg', b'Kula'],
                    [b'tpi', b'Tok Pisin'],
                    [b'tpj', 'Tapiet\xe9'],
                    [b'tpk', b'Tupinikin'],
                    [b'tpl', b"Me'phaa, Tlacoapa"],
                    [b'tpm', b'Tampulma'],
                    [b'tpn', 'Tupinamb\xe1'],
                    [b'tpo', b'Tai Pao'],
                    [b'tpp', b'Tepehua, Pisaflores'],
                    [b'tpq', b'Tukpa'],
                    [b'tpr', 'Tupar\xed'],
                    [b'tpt', b'Tepehua, Tlachichilco'],
                    [b'tpu', b'Tampuan'],
                    [b'tpv', b'Tanapag'],
                    [b'tpw', 'Tup\xed'],
                    [b'tpx', b"Me'phaa, Acatepec"],
                    [b'tpy', b'Trumai'],
                    [b'tpz', b'Tinputz'],
                    [b'tqb', 'Temb\xe9'],
                    [b'tql', b'Lehali'],
                    [b'tqm', b'Turumsa'],
                    [b'tqn', b'Tenino'],
                    [b'tqo', b'Toaripi'],
                    [b'tqp', b'Tomoip'],
                    [b'tqq', b'Tunni'],
                    [b'tqr', b'Torona'],
                    [b'tqt', b'Totonac, Western'],
                    [b'tqu', b'Touo'],
                    [b'tqw', b'Tonkawa'],
                    [b'tra', b'Tirahi'],
                    [b'trb', b'Terebu'],
                    [b'trc', b'Triqui, Copala'],
                    [b'trd', b'Turi'],
                    [b'tre', b'Tarangan, East'],
                    [b'trf', b'Creole English, Trinidadian'],
                    [b'trg', 'Lish\xe1n Did\xe1n'],
                    [b'trh', b'Turaka'],
                    [b'tri', 'Tri\xf3'],
                    [b'trj', b'Toram'],
                    [b'trl', b'Scottish, Traveller'],
                    [b'trm', b'Tregami'],
                    [b'trn', b'Trinitario'],
                    [b'tro', b'Naga, Tarao'],
                    [b'trp', b'Kok Borok'],
                    [b'trq', 'Triqui, San Mart\xedn Itunyoso'],
                    [b'trr', b'Taushiro'],
                    [b'trs', b'Triqui, Chicahuaxtla'],
                    [b'trt', b'Tunggare'],
                    [b'tru', b'Turoyo'],
                    [b'trv', b'Taroko'],
                    [b'trw', b'Torwali'],
                    [b'trx', b'Bidayuh, Tringgus-Sembaan'],
                    [b'try', b'Turung'],
                    [b'trz', 'Tor\xe1'],
                    [b'tsa', b'Tsaangi'],
                    [b'tsb', b'Tsamai'],
                    [b'tsc', b'Tswa'],
                    [b'tsd', b'Tsakonian'],
                    [b'tse', b'Tunisian Sign Language'],
                    [b'tsf', b'Tamang, Southwestern'],
                    [b'tsg', b'Tausug'],
                    [b'tsh', b'Tsuvan'],
                    [b'tsi', b'Tsimshian'],
                    [b'tsj', b'Tshangla'],
                    [b'tsk', b'Tseku'],
                    [b'tsl', "Ts'\xfcn-Lao"],
                    [b'tsm', b'Turkish Sign Language'],
                    [b'tsn', b'Tswana'],
                    [b'tso', b'Tsonga'],
                    [b'tsp', b'Toussian, Northern'],
                    [b'tsq', b'Thai Sign Language'],
                    [b'tsr', b'Akei'],
                    [b'tss', b'Taiwan Sign Language'],
                    [b'tst', b'Songway Kiini, Tondi'],
                    [b'tsu', b'Tsou'],
                    [b'tsv', b'Tsogo'],
                    [b'tsw', b'Tsishingini'],
                    [b'tsx', b'Mubami'],
                    [b'tsy', b'Tebul Sign Language'],
                    [b'tsz', b'Purepecha'],
                    [b'tta', b'Tutelo'],
                    [b'ttb', b'Gaa'],
                    [b'ttc', b'Tektiteko'],
                    [b'ttd', b'Tauade'],
                    [b'tte', b'Bwanabwana'],
                    [b'ttf', b'Tuotomb'],
                    [b'ttg', b'Tutong'],
                    [b'tth', b"Ta'oih, Upper"],
                    [b'tti', b'Tobati'],
                    [b'ttj', b'Tooro'],
                    [b'ttk', b'Totoro'],
                    [b'ttl', b'Totela'],
                    [b'ttm', b'Tutchone, Northern'],
                    [b'ttn', b'Towei'],
                    [b'tto', b"Ta'oih, Lower"],
                    [b'ttp', b'Tombelala'],
                    [b'ttq', b'Tamajaq, Tawallammat'],
                    [b'ttr', b'Tera'],
                    [b'tts', b'Thai, Northeastern'],
                    [b'ttt', b'Tat, Muslim'],
                    [b'ttu', b'Torau'],
                    [b'ttv', b'Titan'],
                    [b'ttw', b'Long Wat'],
                    [b'tty', b'Sikaritai'],
                    [b'ttz', b'Tsum'],
                    [b'tua', b'Wiarumus'],
                    [b'tub', 'T\xfcbatulabal'],
                    [b'tuc', b'Mutu'],
                    [b'tud', 'Tux\xe1'],
                    [b'tue', b'Tuyuca'],
                    [b'tuf', b'Tunebo, Central'],
                    [b'tug', b'Tunia'],
                    [b'tuh', b'Taulil'],
                    [b'tui', b'Tupuri'],
                    [b'tuj', b'Tugutil'],
                    [b'tuk', b'Turkmen'],
                    [b'tul', b'Tula'],
                    [b'tum', b'Tumbuka'],
                    [b'tun', b'Tunica'],
                    [b'tuo', b'Tucano'],
                    [b'tuq', b'Tedaga'],
                    [b'tur', b'Turkish'],
                    [b'tus', b'Tuscarora'],
                    [b'tuu', b'Tututni'],
                    [b'tuv', b'Turkana'],
                    [b'tux', 'Tuxin\xe1wa'],
                    [b'tuy', b'Tugen'],
                    [b'tuz', b'Turka'],
                    [b'tva', b'Vaghua'],
                    [b'tvd', b'Tsuvadi'],
                    [b'tve', b"Te'un"],
                    [b'tvk', b'Ambrym, Southeast'],
                    [b'tvl', b'Tuvalu'],
                    [b'tvm', b'Tela-Masbuar'],
                    [b'tvn', b'Tavoyan'],
                    [b'tvo', b'Tidore'],
                    [b'tvs', b'Taveta'],
                    [b'tvt', b'Naga, Tutsa'],
                    [b'tvu', b'Tunen'],
                    [b'tvw', b'Sedoa'],
                    [b'tvy', b'Pidgin, Timor'],
                    [b'twa', b'Twana'],
                    [b'twb', b'Tawbuid, Western'],
                    [b'twc', b'Teshenawa'],
                    [b'twd', b'Twents'],
                    [b'twe', b'Tewa (Indonesia)'],
                    [b'twf', b'Tiwa, Northern'],
                    [b'twg', b'Tereweng'],
                    [b'twh', 'Tai D\xf3n'],
                    [b'twi', b'Twi'],
                    [b'twl', b'Tawara'],
                    [b'twm', b'Monpa, Tawang'],
                    [b'twn', b'Twendi'],
                    [b'two', b'Tswapong'],
                    [b'twp', b'Ere'],
                    [b'twq', b'Tasawaq'],
                    [b'twr', b'Tarahumara, Southwestern'],
                    [b'twt', 'Turiw\xe1ra'],
                    [b'twu', b'Termanu'],
                    [b'tww', b'Tuwari'],
                    [b'twx', b'Tewe'],
                    [b'twy', b'Tawoyan'],
                    [b'txa', b'Tombonuo'],
                    [b'txb', b'Tokharian B'],
                    [b'txc', b'Tsetsaut'],
                    [b'txe', b'Totoli'],
                    [b'txg', b'Tangut'],
                    [b'txh', b'Thracian'],
                    [b'txi', b'Ikpeng'],
                    [b'txm', b'Tomini'],
                    [b'txn', b'Tarangan, West'],
                    [b'txo', b'Toto'],
                    [b'txq', b'Tii'],
                    [b'txr', b'Tartessian'],
                    [b'txs', b'Tonsea'],
                    [b'txt', b'Citak'],
                    [b'txu', 'Kayap\xf3'],
                    [b'txx', b'Tatana'],
                    [b'txy', b'Malagasy, Tanosy'],
                    [b'tya', b'Tauya'],
                    [b'tye', b'Kyanga'],
                    [b'tyh', b"O'du"],
                    [b'tyi', b'Teke-Tsaayi'],
                    [b'tyj', b'Tai Do'],
                    [b'tyl', b'Thu Lao'],
                    [b'tyn', b'Kombai'],
                    [b'typ', b'Thaypan'],
                    [b'tyr', b'Tai Daeng'],
                    [b'tys', 'T\xe0y Sa Pa'],
                    [b'tyt', 'T\xe0y Tac'],
                    [b'tyu', b'Kua'],
                    [b'tyv', b'Tuvinian'],
                    [b'tyx', b'Teke-Tyee'],
                    [b'tyz', 'T\xe0y'],
                    [b'tza', b'Tanzanian Sign Language'],
                    [b'tzh', b'Tzeltal'],
                    [b'tzj', b"Tz'utujil"],
                    [b'tzl', b'Talossan'],
                    [b'tzm', b'Tamazight, Central Atlas'],
                    [b'tzn', b'Tugun'],
                    [b'tzo', b'Tzotzil'],
                    [b'tzx', b'Tabriak'],
                    [b'uam', 'Uamu\xe9'],
                    [b'uan', b'Kuan'],
                    [b'uar', b'Tairuma'],
                    [b'uba', b'Ubang'],
                    [b'ubi', b'Ubi'],
                    [b'ubl', b"Bikol, Buhi'non"],
                    [b'ubr', b'Ubir'],
                    [b'ubu', b'Umbu-Ungu'],
                    [b'uby', b'Ubykh'],
                    [b'uda', b'Uda'],
                    [b'ude', b'Udihe'],
                    [b'udg', b'Muduga'],
                    [b'udi', b'Udi'],
                    [b'udj', b'Ujir'],
                    [b'udl', b'Wuzlam'],
                    [b'udm', b'Udmurt'],
                    [b'udu', b'Uduk'],
                    [b'ues', b'Kioko'],
                    [b'ufi', b'Ufim'],
                    [b'uga', b'Ugaritic'],
                    [b'ugb', b'Kuku-Ugbanh'],
                    [b'uge', b'Ughele'],
                    [b'ugn', b'Ugandan Sign Language'],
                    [b'ugo', b'Ugong'],
                    [b'ugy', b'Uruguayan Sign Language'],
                    [b'uha', b'Uhami'],
                    [b'uhn', b'Damal'],
                    [b'uig', b'Uighur'],
                    [b'uis', b'Uisai'],
                    [b'uiv', b'Iyive'],
                    [b'uji', b'Tanjijili'],
                    [b'uka', b'Kaburi'],
                    [b'ukg', b'Ukuriguma'],
                    [b'ukh', b'Ukhwejo'],
                    [b'ukl', b'Ukrainian Sign Language'],
                    [b'ukp', b'Ukpe-Bayobiri'],
                    [b'ukq', b'Ukwa'],
                    [b'ukr', b'Ukrainian'],
                    [b'uks', 'Urub\xfa-Kaapor Sign Language'],
                    [b'uku', b'Ukue'],
                    [b'ukw', b'Ukwuani-Aboh-Ndoni'],
                    [b'uky', b'Kuuk-Yak'],
                    [b'ula', b'Fungwa'],
                    [b'ulb', b'Ulukwumi'],
                    [b'ulc', b'Ulch'],
                    [b'ule', b'Lule'],
                    [b'ulf', b'Usku'],
                    [b'uli', b'Ulithian'],
                    [b'ulk', b'Meriam'],
                    [b'ull', b'Ullatan'],
                    [b'ulm', b"Ulumanda'"],
                    [b'uln', b'Unserdeutsch'],
                    [b'ulu', b"Uma' Lung"],
                    [b'ulw', b'Ulwa'],
                    [b'uma', b'Umatilla'],
                    [b'umb', b'Umbundu'],
                    [b'umc', b'Marrucinian'],
                    [b'umd', b'Umbindhamu'],
                    [b'umg', b'Umbuygamu'],
                    [b'umi', b'Ukit'],
                    [b'umm', b'Umon'],
                    [b'umn', b'Naga, Makyan'],
                    [b'umo', 'Umot\xedna'],
                    [b'ump', b'Umpila'],
                    [b'umr', b'Umbugarla'],
                    [b'ums', b'Pendau'],
                    [b'umu', b'Munsee'],
                    [b'una', b'Watut, North'],
                    [b'und', b'Undetermined'],
                    [b'une', b'Uneme'],
                    [b'ung', b'Ngarinyin'],
                    [b'unk', 'Enawen\xe9-Naw\xe9'],
                    [b'unm', b'Unami'],
                    [b'unn', b'Kurnai'],
                    [b'unr', b'Mundari'],
                    [b'unu', b'Unubahe'],
                    [b'unx', b'Munda'],
                    [b'unz', b'Kaili, Unde'],
                    [b'uok', b'Uokha'],
                    [b'upi', b'Umeda'],
                    [b'upv', b'Uripiv-Wala-Rano-Atchin'],
                    [b'ura', b'Urarina'],
                    [b'urb', 'Urub\xfa-Kaapor'],
                    [b'urc', b'Urningangg'],
                    [b'urd', b'Urdu'],
                    [b'ure', b'Uru'],
                    [b'urf', b'Uradhi'],
                    [b'urg', b'Urigina'],
                    [b'urh', b'Urhobo'],
                    [b'uri', b'Urim'],
                    [b'urk', b"Urak Lawoi'"],
                    [b'url', b'Urali'],
                    [b'urm', b'Urapmin'],
                    [b'urn', b'Uruangnirin'],
                    [b'uro', b'Ura (Papua New Guinea)'],
                    [b'urp', b'Uru-Pa-In'],
                    [b'urr', b'Lehalurup'],
                    [b'urt', b'Urat'],
                    [b'uru', b'Urumi'],
                    [b'urv', b'Uruava'],
                    [b'urw', b'Sop'],
                    [b'urx', b'Urimo'],
                    [b'ury', b'Orya'],
                    [b'urz', b'Uru-Eu-Wau-Wau'],
                    [b'usa', b'Usarufa'],
                    [b'ush', b'Ushojo'],
                    [b'usi', b'Usui'],
                    [b'usk', b'Usaghade'],
                    [b'usp', b'Uspanteco'],
                    [b'usu', b'Uya'],
                    [b'uta', b'Otank'],
                    [b'ute', b'Ute-Southern Paiute'],
                    [b'utp', b'Amba (Solomon Islands)'],
                    [b'utr', b'Etulo'],
                    [b'utu', b'Utu'],
                    [b'uum', b'Urum'],
                    [b'uun', b'Kulon-Pazeh'],
                    [b'uur', b'Ura (Vanuatu)'],
                    [b'uuu', b'U'],
                    [b'uve', b'Uvean, West'],
                    [b'uvh', b'Uri'],
                    [b'uvl', b'Lote'],
                    [b'uwa', b'Kuku-Uwanh'],
                    [b'uya', b'Doko-Uyanga'],
                    [b'uzb', b'Uzbek'],
                    [b'uzn', b'Uzbek, Northern'],
                    [b'uzs', b'Uzbek, Southern'],
                    [b'vaa', b'Vaagri Booli'],
                    [b'vae', b'Vale'],
                    [b'vaf', b'Vafsi'],
                    [b'vag', b'Vagla'],
                    [b'vah', b'Varhadi-Nagpuri'],
                    [b'vai', b'Vai'],
                    [b'vaj', b'Vasekela Bushman'],
                    [b'val', b'Vehes'],
                    [b'vam', b'Vanimo'],
                    [b'van', b'Valman'],
                    [b'vao', b'Vao'],
                    [b'vap', b'Vaiphei'],
                    [b'var', b'Huarijio'],
                    [b'vas', b'Vasavi'],
                    [b'vau', b'Vanuma'],
                    [b'vav', b'Varli'],
                    [b'vay', b'Wayu'],
                    [b'vbb', b'Babar, Southeast'],
                    [b'vbk', b'Bontok, Southwestern'],
                    [b'vec', b'Venetian'],
                    [b'ved', b'Veddah'],
                    [b'vel', b'Veluws'],
                    [b'vem', b'Vemgo-Mabas'],
                    [b'ven', b'Venda'],
                    [b'veo', 'Venture\xf1o'],
                    [b'vep', b'Veps'],
                    [b'ver', b'Mom Jango'],
                    [b'vgr', b'Vaghri'],
                    [b'vgt', b'Vlaamse Gebarentaal'],
                    [b'vic', b'Creole English, Virgin Islands'],
                    [b'vid', b'Vidunda'],
                    [b'vie', b'Vietnamese'],
                    [b'vif', b'Vili'],
                    [b'vig', b'Viemo'],
                    [b'vil', b'Vilela'],
                    [b'vin', b'Vinza'],
                    [b'vis', b'Vishavan'],
                    [b'vit', b'Viti'],
                    [b'viv', b'Iduna'],
                    [b'vka', b'Kariyarra'],
                    [b'vki', b'Ija-Zuba'],
                    [b'vkj', b'Kujarge'],
                    [b'vkk', b'Kaur'],
                    [b'vkl', b'Kulisusu'],
                    [b'vkm', b'Kamakan'],
                    [b'vko', b'Kodeoha'],
                    [b'vkp', b'Creole Portuguese, Korlai'],
                    [b'vkt', b'Malay, Tenggarong Kutai'],
                    [b'vku', b'Kurrama'],
                    [b'vlp', b'Valpei'],
                    [b'vls', b'Vlaams'],
                    [b'vma', b'Martuyhunira'],
                    [b'vmb', b'Barbaram'],
                    [b'vmc', b'Mixtec, Juxtlahuaca'],
                    [b'vmd', b'Koraga, Mudu'],
                    [b'vme', b'Masela, East'],
                    [b'vmf', 'Mainfr\xe4nkisch'],
                    [b'vmg', b'Lungalunga'],
                    [b'vmh', b'Maraghei'],
                    [b'vmi', b'Miwa'],
                    [b'vmj', b'Mixtec, Ixtayutla'],
                    [b'vmk', b'Makhuwa-Shirima'],
                    [b'vml', b'Malgana'],
                    [b'vmm', b'Mixtec, Mitlatongo'],
                    [b'vmp', b'Mazatec, Soyaltepec'],
                    [b'vmq', b'Mixtec, Soyaltepec'],
                    [b'vmr', b'Marenje'],
                    [b'vms', b'Moksela'],
                    [b'vmu', b'Muluridyi'],
                    [b'vmv', b'Maidu, Valley'],
                    [b'vmw', b'Makhuwa'],
                    [b'vmx', b'Mixtec, Tamazola'],
                    [b'vmy', b'Mazatec, Ayautla'],
                    [b'vmz', 'Mazatec, Mazatl\xe1n'],
                    [b'vnk', b'Vano'],
                    [b'vnm', b'Vinmavis'],
                    [b'vnp', b'Vunapu'],
                    [b'vol', 'Volap\xfck'],
                    [b'vor', b'Voro'],
                    [b'vot', b'Votic'],
                    [b'vra', b"Vera'a"],
                    [b'vro', 'V\xf5ro'],
                    [b'vrs', b'Varisi'],
                    [b'vrt', b'Burmbar'],
                    [b'vsi', b'Moldova Sign Language'],
                    [b'vsl', b'Venezuelan Sign Language'],
                    [b'vsv', b'Valencian Sign Language'],
                    [b'vto', b'Vitou'],
                    [b'vum', b'Vumbu'],
                    [b'vun', b'Vunjo'],
                    [b'vut', b'Vute'],
                    [b'vwa', b'Awa (China)'],
                    [b'waa', b'Walla Walla'],
                    [b'wab', b'Wab'],
                    [b'wac', b'Wasco-Wishram'],
                    [b'wad', b'Wandamen'],
                    [b'wae', b'Walser'],
                    [b'waf', 'Wakon\xe1'],
                    [b'wag', b"Wa'ema"],
                    [b'wah', b'Watubela'],
                    [b'wai', b'Wares'],
                    [b'waj', b'Waffa'],
                    [b'wal', b'Wolaytta'],
                    [b'wam', b'Wampanoag'],
                    [b'wan', b'Wan'],
                    [b'wao', b'Wappo'],
                    [b'wap', b'Wapishana'],
                    [b'waq', b'Wageman'],
                    [b'war', b'Waray (Philippines)'],
                    [b'was', b'Washo'],
                    [b'wat', b'Kaninuwa'],
                    [b'wau', 'Waur\xe1'],
                    [b'wav', b'Waka'],
                    [b'waw', b'Waiwai'],
                    [b'wax', b'Watam'],
                    [b'way', b'Wayana'],
                    [b'waz', b'Wampur'],
                    [b'wba', b'Warao'],
                    [b'wbb', b'Wabo'],
                    [b'wbe', b'Waritai'],
                    [b'wbf', b'Wara'],
                    [b'wbh', b'Wanda'],
                    [b'wbi', b'Vwanji'],
                    [b'wbj', b'Alagwa'],
                    [b'wbk', b'Waigali'],
                    [b'wbl', b'Wakhi'],
                    [b'wbm', b'Wa'],
                    [b'wbp', b'Warlpiri'],
                    [b'wbq', b'Waddar'],
                    [b'wbr', b'Wagdi'],
                    [b'wbt', b'Wanman'],
                    [b'wbv', b'Wajarri'],
                    [b'wbw', b'Woi'],
                    [b'wca', 'Yanom\xe1mi'],
                    [b'wci', b'Gbe, Waci'],
                    [b'wdd', b'Wandji'],
                    [b'wdg', b'Wadaginam'],
                    [b'wdj', b'Wadjiginy'],
                    [b'wdk', b'Wadikali'],
                    [b'wdu', b'Wadjigu'],
                    [b'wdy', b'Wadjabangayi'],
                    [b'wea', b'Wewaw'],
                    [b'wec', 'W\xe8 Western'],
                    [b'wed', b'Wedau'],
                    [b'weg', b'Wergaia'],
                    [b'weh', b'Weh'],
                    [b'wei', b'Kiunum'],
                    [b'wem', b'Gbe, Weme'],
                    [b'weo', b'Wemale'],
                    [b'wep', b'Westphalien'],
                    [b'wer', b'Weri'],
                    [b'wes', b'Pidgin, Cameroon'],
                    [b'wet', b'Perai'],
                    [b'weu', b'Chin, Rawngtu'],
                    [b'wew', b'Wejewa'],
                    [b'wfg', b'Yafi'],
                    [b'wga', b'Wagaya'],
                    [b'wgb', b'Wagawaga'],
                    [b'wgg', b'Wangganguru'],
                    [b'wgi', b'Wahgi'],
                    [b'wgo', b'Waigeo'],
                    [b'wgu', b'Wirangu'],
                    [b'wgy', b'Warrgamay'],
                    [b'wha', b'Manusela'],
                    [b'whg', b'Wahgi, North'],
                    [b'whk', b'Kenyah, Wahau'],
                    [b'whu', b'Kayan, Wahau'],
                    [b'wib', b'Toussian, Southern'],
                    [b'wic', b'Wichita'],
                    [b'wie', b'Wik-Epa'],
                    [b'wif', b'Wik-Keyangan'],
                    [b'wig', b'Wik-Ngathana'],
                    [b'wih', b"Wik-Me'anha"],
                    [b'wii', b'Minidien'],
                    [b'wij', b'Wik-Iiyanh'],
                    [b'wik', b'Wikalkan'],
                    [b'wil', b'Wilawila'],
                    [b'wim', b'Wik-Mungkan'],
                    [b'win', b'Ho-Chunk'],
                    [b'wir', 'Wiraf\xe9d'],
                    [b'wiu', b'Wiru'],
                    [b'wiv', b'Vitu'],
                    [b'wiy', b'Wiyot'],
                    [b'wja', b'Waja'],
                    [b'wji', b'Warji'],
                    [b'wka', b"Kw'adza"],
                    [b'wkb', b'Kumbaran'],
                    [b'wkd', b'Wakde'],
                    [b'wkl', b'Kalanadi'],
                    [b'wku', b'Kunduvadi'],
                    [b'wkw', b'Wakawaka'],
                    [b'wky', b'Wangkayutyuru'],
                    [b'wla', b'Walio'],
                    [b'wlc', b'Comorian, Mwali'],
                    [b'wle', b'Wolane'],
                    [b'wlg', b'Kunbarlang'],
                    [b'wli', b'Waioli'],
                    [b'wlk', b'Wailaki'],
                    [b'wll', b'Wali (Sudan)'],
                    [b'wlm', b'Welsh, Middle'],
                    [b'wln', b'Walloon'],
                    [b'wlo', b'Wolio'],
                    [b'wlr', b'Wailapa'],
                    [b'wls', b'Wallisian'],
                    [b'wlu', b'Wuliwuli'],
                    [b'wlv', 'Wich\xed Lhamt\xe9s Vejoz'],
                    [b'wlw', b'Walak'],
                    [b'wlx', b'Wali (Ghana)'],
                    [b'wly', b'Waling'],
                    [b'wma', b'Mawa (Nigeria)'],
                    [b'wmb', b'Wambaya'],
                    [b'wmc', b'Wamas'],
                    [b'wmd', 'Mamaind\xe9'],
                    [b'wme', b'Wambule'],
                    [b'wmh', b"Waima'a"],
                    [b'wmi', b'Wamin'],
                    [b'wmm', b'Maiwa (Indonesia)'],
                    [b'wmn', b'Waamwang'],
                    [b'wmo', b'Wom (Papua New Guinea)'],
                    [b'wms', b'Wambon'],
                    [b'wmt', b'Walmajarri'],
                    [b'wmw', b'Mwani'],
                    [b'wmx', b'Womo'],
                    [b'wnb', b'Wanambre'],
                    [b'wnc', b'Wantoat'],
                    [b'wnd', b'Wandarang'],
                    [b'wne', b'Waneci'],
                    [b'wng', b'Wanggom'],
                    [b'wni', b'Comorian, Ndzwani'],
                    [b'wnk', b'Wanukaka'],
                    [b'wnm', b'Wanggamala'],
                    [b'wnn', b'Wunumara'],
                    [b'wno', b'Wano'],
                    [b'wnp', b'Wanap'],
                    [b'wnu', b'Usan'],
                    [b'wnw', b'Wintu'],
                    [b'wny', b'Wanyi'],
                    [b'woa', b'Tyaraity'],
                    [b'wob', 'W\xe8 Northern'],
                    [b'woc', b'Wogeo'],
                    [b'wod', b'Wolani'],
                    [b'woe', b'Woleaian'],
                    [b'wof', b'Wolof, Gambian'],
                    [b'wog', b'Wogamusin'],
                    [b'woi', b'Kamang'],
                    [b'wok', b'Longto'],
                    [b'wol', b'Wolof'],
                    [b'wom', b'Wom (Nigeria)'],
                    [b'won', b'Wongo'],
                    [b'woo', b'Manombai'],
                    [b'wor', b'Woria'],
                    [b'wos', b'Hanga Hundi'],
                    [b'wow', b'Wawonii'],
                    [b'woy', b'Weyto'],
                    [b'wpc', b'Maco'],
                    [b'wra', b'Warapu'],
                    [b'wrb', b'Warluwara'],
                    [b'wrd', b'Warduji'],
                    [b'wrg', b'Warungu'],
                    [b'wrh', b'Wiradhuri'],
                    [b'wri', b'Wariyangga'],
                    [b'wrk', b'Garrwa'],
                    [b'wrl', b'Warlmanpa'],
                    [b'wrm', b'Warumungu'],
                    [b'wrn', b'Warnang'],
                    [b'wro', b'Worrorra'],
                    [b'wrp', b'Waropen'],
                    [b'wrr', b'Wardaman'],
                    [b'wrs', b'Waris'],
                    [b'wru', b'Waru'],
                    [b'wrv', b'Waruna'],
                    [b'wrw', b'Gugu Warra'],
                    [b'wrx', b'Wae Rana'],
                    [b'wry', b'Merwari'],
                    [b'wrz', b'Waray (Australia)'],
                    [b'wsa', b'Warembori'],
                    [b'wsi', b'Wusi'],
                    [b'wsk', b'Waskia'],
                    [b'wsr', b'Owenia'],
                    [b'wss', b'Wasa'],
                    [b'wsu', b'Wasu'],
                    [b'wsv', b'Wotapuri-Katarqalai'],
                    [b'wtf', b'Watiwa'],
                    [b'wth', b'Wathawurrung'],
                    [b'wti', b'Berta'],
                    [b'wtk', b'Watakataui'],
                    [b'wtm', b'Mewati'],
                    [b'wtw', b'Wotu'],
                    [b'wua', b'Wikngenchera'],
                    [b'wub', b'Wunambal'],
                    [b'wud', b'Wudu'],
                    [b'wuh', b'Wutunhua'],
                    [b'wul', b'Silimo'],
                    [b'wum', b'Wumbvu'],
                    [b'wun', b'Bungu'],
                    [b'wur', b'Wurrugu'],
                    [b'wut', b'Wutung'],
                    [b'wuu', b'Chinese, Wu'],
                    [b'wuv', b'Wuvulu-Aua'],
                    [b'wux', b'Wulna'],
                    [b'wuy', b'Wauyai'],
                    [b'wwa', b'Waama'],
                    [b'wwb', b'Wakabunga'],
                    [b'wwo', b'Wetamut'],
                    [b'wwr', b'Warrwa'],
                    [b'www', b'Wawa'],
                    [b'wxa', b'Waxianghua'],
                    [b'wxw', b'Wardandi'],
                    [b'wya', b'Wyandot'],
                    [b'wyb', b'Wangaaybuwan-Ngiyambaa'],
                    [b'wyi', b'Woiwurrung'],
                    [b'wym', b'Wymysorys'],
                    [b'wyr', 'Wayor\xf3'],
                    [b'wyy', b'Fijian, Western'],
                    [b'xaa', b'Arabic, Andalusian'],
                    [b'xab', b'Sambe'],
                    [b'xac', b'Kachari'],
                    [b'xad', b'Adai'],
                    [b'xae', b'Aequian'],
                    [b'xag', b'Aghwan'],
                    [b'xai', 'Kaimb\xe9'],
                    [b'xal', b'Kalmyk'],
                    [b'xam', b'/Xam'],
                    [b'xan', b'Xamtanga'],
                    [b'xao', b'Khao'],
                    [b'xap', b'Apalachee'],
                    [b'xaq', b'Aquitanian'],
                    [b'xar', b'Karami'],
                    [b'xas', b'Kamas'],
                    [b'xat', b'Katawixi'],
                    [b'xau', b'Kauwera'],
                    [b'xav', 'Xav\xe1nte'],
                    [b'xaw', b'Kawaiisu'],
                    [b'xay', b'Kayan Mahakam'],
                    [b'xba', b'Kamba (Brazil)'],
                    [b'xbb', b'Burdekin, Lower'],
                    [b'xbc', b'Bactrian'],
                    [b'xbd', b'Bindal'],
                    [b'xbe', b'Bigambal'],
                    [b'xbg', b'Bunganditj'],
                    [b'xbi', b'Kombio'],
                    [b'xbj', b'Birrpayi'],
                    [b'xbm', b'Breton, Middle'],
                    [b'xbn', b'Kenaboi'],
                    [b'xbo', b'Bolgarian'],
                    [b'xbp', b'Bibbulman'],
                    [b'xbr', b'Kambera'],
                    [b'xbw', 'Kambiw\xe1'],
                    [b'xbx', 'Kabix\xed'],
                    [b'xby', b'Batyala'],
                    [b'xcb', b'Cumbric'],
                    [b'xcc', b'Camunic'],
                    [b'xce', b'Celtiberian'],
                    [b'xcg', b'Gaulish, Cisalpine'],
                    [b'xch', b'Chemakum'],
                    [b'xcl', b'Armenian, Classical'],
                    [b'xcm', b'Comecrudo'],
                    [b'xcn', b'Cotoname'],
                    [b'xco', b'Chorasmian'],
                    [b'xcr', b'Carian'],
                    [b'xct', b'Tibetan, Classical'],
                    [b'xcu', b'Curonian'],
                    [b'xcv', b'Chuvantsy'],
                    [b'xcw', b'Coahuilteco'],
                    [b'xcy', b'Cayuse'],
                    [b'xda', b'Darkinyung'],
                    [b'xdc', b'Dacian'],
                    [b'xdk', b'Dharuk'],
                    [b'xdm', b'Edomite'],
                    [b'xdy', b'Dayak, Malayic'],
                    [b'xeb', b'Eblan'],
                    [b'xed', b'Hdi'],
                    [b'xeg', b'//Xegwi'],
                    [b'xel', b'Kelo'],
                    [b'xem', b'Kembayan'],
                    [b'xep', b'Epi-Olmec'],
                    [b'xer', 'Xer\xe9nte'],
                    [b'xes', b'Kesawai'],
                    [b'xet', 'Xet\xe1'],
                    [b'xeu', b'Keoru-Ahia'],
                    [b'xfa', b'Faliscan'],
                    [b'xga', b'Galatian'],
                    [b'xgb', b'Gbin'],
                    [b'xgd', b'Gudang'],
                    [b'xgf', 'Gabrielino-Fernande\xf1o'],
                    [b'xgg', b'Goreng'],
                    [b'xgi', b'Garingbal'],
                    [b'xgl', b'Galindan'],
                    [b'xgm', b'Guwinmal'],
                    [b'xgr', b'Garza'],
                    [b'xgu', b'Unggumi'],
                    [b'xgw', b'Guwa'],
                    [b'xha', b'Harami'],
                    [b'xhc', b'Hunnic'],
                    [b'xhd', b'Hadrami'],
                    [b'xhe', b'Khetrani'],
                    [b'xho', b'Xhosa'],
                    [b'xhr', b'Hernican'],
                    [b'xht', b'Hattic'],
                    [b'xhu', b'Hurrian'],
                    [b'xhv', b'Khua'],
                    [b'xib', b'Iberian'],
                    [b'xii', b'Xiri'],
                    [b'xil', b'Illyrian'],
                    [b'xin', b'Xinca'],
                    [b'xip', 'Xipin\xe1wa'],
                    [b'xir', 'Xiri\xe2na'],
                    [b'xiv', b'Indus Valley Language'],
                    [b'xiy', b'Xipaya'],
                    [b'xjb', b'Minjungbal'],
                    [b'xjt', b'Jaitmatang'],
                    [b'xka', b'Kalkoti'],
                    [b'xkb', b'Nago, Northern'],
                    [b'xkc', b"Kho'ini"],
                    [b'xkd', b'Kayan, Mendalam'],
                    [b'xke', b'Kereho'],
                    [b'xkf', b'Khengkha'],
                    [b'xkg', b'Kagoro'],
                    [b'xkh', b'Karahawyana'],
                    [b'xki', b'Kenyan Sign Language'],
                    [b'xkj', b'Kajali'],
                    [b'xkk', b"Kaco'"],
                    [b'xkl', b'Mainstream Kenyah'],
                    [b'xkn', b'Kayan, Kayan River'],
                    [b'xko', b'Kiorr'],
                    [b'xkp', b'Kabatei'],
                    [b'xkq', b'Koroni'],
                    [b'xkr', 'Xakriab\xe1'],
                    [b'xks', b'Kumbewaha'],
                    [b'xkt', b'Kantosi'],
                    [b'xku', b'Kaamba'],
                    [b'xkv', b'Kgalagadi'],
                    [b'xkw', b'Kembra'],
                    [b'xkx', b'Karore'],
                    [b'xky', b"Uma' Lasan"],
                    [b'xkz', b'Kurtokha'],
                    [b'xla', b'Kamula'],
                    [b'xlb', b'Loup B'],
                    [b'xlc', b'Lycian'],
                    [b'xld', b'Lydian'],
                    [b'xle', b'Lemnian'],
                    [b'xlg', b'Ligurian (Ancient)'],
                    [b'xli', b'Liburnian'],
                    [b'xln', b'Alanic'],
                    [b'xlo', b'Loup A'],
                    [b'xlp', b'Lepontic'],
                    [b'xls', b'Lusitanian'],
                    [b'xlu', b'Luwian, Cuneiform'],
                    [b'xly', b'Elymian'],
                    [b'xma', b'Mushungulu'],
                    [b'xmb', b'Mbonga'],
                    [b'xmc', b'Makhuwa-Marrevone'],
                    [b'xmd', b'Mbudum'],
                    [b'xme', b'Median'],
                    [b'xmf', b'Mingrelian'],
                    [b'xmg', b'Mengaka'],
                    [b'xmh', b'Kuku-Muminh'],
                    [b'xmj', b'Majera'],
                    [b'xmk', b'Macedonian, Ancient'],
                    [b'xml', b'Malaysian Sign Language'],
                    [b'xmm', b'Malay, Manado'],
                    [b'xmn', b'Persian, Manichaean Middle'],
                    [b'xmo', b'Morerebi'],
                    [b'xmp', b"Kuku-Mu'inh"],
                    [b'xmq', b'Kuku-Mangk'],
                    [b'xmr', b'Meroitic'],
                    [b'xms', b'Moroccan Sign Language'],
                    [b'xmt', b'Matbat'],
                    [b'xmu', b'Kamu'],
                    [b'xmv', b'Malagasy, Antankarana'],
                    [b'xmw', b'Malagasy, Tsimihety'],
                    [b'xmx', b'Maden'],
                    [b'xmy', b'Mayaguduna'],
                    [b'xmz', b'Mori Bawah'],
                    [b'xna', b'North Arabian, Ancient'],
                    [b'xnb', b'Kanakanabu'],
                    [b'xng', b'Mongolian, Middle'],
                    [b'xnh', b'Kuanhua'],
                    [b'xni', b'Ngarigu'],
                    [b'xnk', b'Nganakarti'],
                    [b'xnn', b'Kankanay, Northern'],
                    [b'xno', b'Anglo-Norman'],
                    [b'xnr', b'Kangri'],
                    [b'xns', b'Kanashi'],
                    [b'xnt', b'Narragansett'],
                    [b'xnu', b'Nukunul'],
                    [b'xny', b'Nyiyaparli'],
                    [b'xnz', b'Kenzi'],
                    [b'xoc', b"O'chi'chi'"],
                    [b'xod', b'Kokoda'],
                    [b'xog', b'Soga'],
                    [b'xoi', b'Kominimung'],
                    [b'xok', b'Xokleng'],
                    [b'xom', b'Komo (Sudan)'],
                    [b'xon', b'Konkomba'],
                    [b'xoo', 'Xukur\xfa'],
                    [b'xop', b'Kopar'],
                    [b'xor', b'Korubo'],
                    [b'xow', b'Kowaki'],
                    [b'xpa', b'Pirriya'],
                    [b'xpc', b'Pecheneg'],
                    [b'xpe', b'Kpelle, Liberia'],
                    [b'xpg', b'Phrygian'],
                    [b'xpi', b'Pictish'],
                    [b'xpj', b'Mpalitjanh'],
                    [b'xpk', b'Pano, Kulina'],
                    [b'xpm', b'Pumpokol'],
                    [b'xpn', 'Kapinaw\xe1'],
                    [b'xpo', b'Pochutec'],
                    [b'xpp', b'Puyo-Paekche'],
                    [b'xpq', b'Mohegan-Pequot'],
                    [b'xpr', b'Parthian'],
                    [b'xps', b'Pisidian'],
                    [b'xpt', b'Punthamara'],
                    [b'xpu', b'Punic'],
                    [b'xpy', b'Puyo'],
                    [b'xqa', b'Karakhanid'],
                    [b'xqt', b'Qatabanian'],
                    [b'xra', 'Krah\xf4'],
                    [b'xrb', b'Karaboro, Eastern'],
                    [b'xrd', b'Gundungurra'],
                    [b'xre', b'Kreye'],
                    [b'xrg', b'Minang'],
                    [b'xri', b'Krikati-Timbira'],
                    [b'xrm', b'Armazic'],
                    [b'xrn', b'Arin'],
                    [b'xrq', b'Karranga'],
                    [b'xrr', b'Raetic'],
                    [b'xrt', b'Aranama-Tamique'],
                    [b'xru', b'Marriammu'],
                    [b'xrw', b'Karawa'],
                    [b'xsa', b'Sabaean'],
                    [b'xsb', b'Sambal'],
                    [b'xsc', b'Scythian'],
                    [b'xsd', b'Sidetic'],
                    [b'xse', b'Sempan'],
                    [b'xsh', b'Shamang'],
                    [b'xsi', b'Sio'],
                    [b'xsj', b'Subi'],
                    [b'xsl', b'Slavey, South'],
                    [b'xsm', b'Kasem'],
                    [b'xsn', b'Sanga (Nigeria)'],
                    [b'xso', b'Solano'],
                    [b'xsp', b'Silopi'],
                    [b'xsq', b'Makhuwa-Saka'],
                    [b'xsr', b'Sherpa'],
                    [b'xss', b'Assan'],
                    [b'xsu', 'Sanum\xe1'],
                    [b'xsv', b'Sudovian'],
                    [b'xsy', b'Saisiyat'],
                    [b'xta', b'Mixtec, Alcozauca'],
                    [b'xtb', b'Mixtec, Chazumba'],
                    [b'xtc', b'Katcha-Kadugli-Miri'],
                    [b'xtd', b'Mixtec, Diuxi-Tilantongo'],
                    [b'xte', b'Ketengban'],
                    [b'xtg', b'Gaulish, Transalpine'],
                    [b'xth', b'Yitha Yitha'],
                    [b'xti', b'Mixtec, Sinicahua'],
                    [b'xtj', b'Mixtec, San Juan Teita'],
                    [b'xtl', b'Mixtec, Tijaltepec'],
                    [b'xtm', 'Mixtec, Magdalena Pe\xf1asco'],
                    [b'xtn', b'Mixtec, Northern Tlaxiaco'],
                    [b'xto', b'Tokharian A'],
                    [b'xtp', b'Mixtec, San Miguel Piedras'],
                    [b'xtq', b'Tumshuqese'],
                    [b'xtr', b'Tripuri, Early'],
                    [b'xts', b'Mixtec, Sindihui'],
                    [b'xtt', b'Mixtec, Tacahua'],
                    [b'xtu', b'Mixtec, Cuyamecalco'],
                    [b'xtv', b'Thawa'],
                    [b'xtw', 'Tawand\xea'],
                    [b'xty', b'Mixtec, Yoloxochitl'],
                    [b'xtz', b'Tasmanian'],
                    [b'xua', b'Kurumba, Alu'],
                    [b'xub', b'Kurumba, Betta'],
                    [b'xud', b'Umiida'],
                    [b'xug', b'Kunigami'],
                    [b'xuj', b'Kurumba, Jennu'],
                    [b'xul', b'Ngunawal'],
                    [b'xum', b'Umbrian'],
                    [b'xun', b'Unggaranggu'],
                    [b'xuo', b'Kuo'],
                    [b'xup', b'Umpqua, Upper'],
                    [b'xur', b'Urartian'],
                    [b'xut', b'Kuthant'],
                    [b'xuu', b'Kxoe'],
                    [b'xve', b'Venetic'],
                    [b'xvi', b'Kamviri'],
                    [b'xvn', b'Vandalic'],
                    [b'xvo', b'Volscian'],
                    [b'xvs', b'Vestinian'],
                    [b'xwa', b'Kwaza'],
                    [b'xwc', b'Woccon'],
                    [b'xwd', b'Wadi Wadi'],
                    [b'xwe', b'Gbe, Xwela'],
                    [b'xwg', b'Kwegu'],
                    [b'xwj', b'Wajuk'],
                    [b'xwk', b'Wangkumara'],
                    [b'xwl', b'Gbe, Western Xwla'],
                    [b'xwo', b'Oirat, Written'],
                    [b'xwr', b'Kwerba Mamberamo'],
                    [b'xwt', b'Wotjobaluk'],
                    [b'xww', b'Wemba Wemba'],
                    [b'xxb', b'Boro (Ghana)'],
                    [b'xxk', b"Ke'o"],
                    [b'xxm', b'Minkin'],
                    [b'xxr', 'Korop\xf3'],
                    [b'xxt', b'Tambora'],
                    [b'xya', b'Yaygir'],
                    [b'xyb', b'Yandjibara'],
                    [b'xyj', b'Mayi-Yapi'],
                    [b'xyk', b'Mayi-Kulan'],
                    [b'xyl', b'Yalakalore'],
                    [b'xyt', b'Mayi-Thakurti'],
                    [b'xyy', b'Yorta Yorta'],
                    [b'xzh', b'Zhang-Zhung'],
                    [b'xzm', b'Zemgalian'],
                    [b'xzp', b'Zapotec, Ancient'],
                    [b'yaa', b'Yaminahua'],
                    [b'yab', b'Yuhup'],
                    [b'yac', b'Yali, Pass Valley'],
                    [b'yad', b'Yagua'],
                    [b'yae', 'Pum\xe9'],
                    [b'yaf', b'Yaka (Democratic Republic of Congo)'],
                    [b'yag', 'Y\xe1mana'],
                    [b'yah', b'Yazgulyam'],
                    [b'yai', b'Yagnobi'],
                    [b'yaj', b'Banda-Yangere'],
                    [b'yak', b'Yakama'],
                    [b'yal', b'Yalunka'],
                    [b'yam', b'Yamba'],
                    [b'yan', b'Mayangna'],
                    [b'yao', b'Yao'],
                    [b'yap', b'Yapese'],
                    [b'yaq', b'Yaqui'],
                    [b'yar', b'Yabarana'],
                    [b'yas', b'Nugunu (Cameroon)'],
                    [b'yat', b'Yambeta'],
                    [b'yau', b'Yuwana'],
                    [b'yav', b'Yangben'],
                    [b'yaw', 'Yawalapit\xed'],
                    [b'yax', b'Yauma'],
                    [b'yay', b'Agwagwune'],
                    [b'yaz', b'Lokaa'],
                    [b'yba', b'Yala'],
                    [b'ybb', b'Yemba'],
                    [b'ybe', b'Yugur, West'],
                    [b'ybh', b'Yakha'],
                    [b'ybi', b'Yamphu'],
                    [b'ybj', b'Hasha'],
                    [b'ybk', b'Bokha'],
                    [b'ybl', b'Yukuben'],
                    [b'ybm', b'Yaben'],
                    [b'ybn', 'Yaba\xe2na'],
                    [b'ybo', b'Yabong'],
                    [b'ybx', b'Yawiyo'],
                    [b'yby', b'Yaweyuha'],
                    [b'ych', b'Chesu'],
                    [b'ycl', b'Lolopo'],
                    [b'ycn', b'Yucuna'],
                    [b'ycp', b'Chepya'],
                    [b'yda', b'Yanda'],
                    [b'ydd', b'Yiddish, Eastern'],
                    [b'yde', b'Yangum Dey'],
                    [b'ydg', b'Yidgha'],
                    [b'ydk', b'Yoidik'],
                    [b'yds', b'Yiddish Sign Language'],
                    [b'yea', b'Ravula'],
                    [b'yec', b'Yeniche'],
                    [b'yee', b'Yimas'],
                    [b'yei', b'Yeni'],
                    [b'yej', b'Yevanic'],
                    [b'yel', b'Yela'],
                    [b'yer', b'Tarok'],
                    [b'yes', b'Nyankpa'],
                    [b'yet', b'Yetfa'],
                    [b'yeu', b'Yerukula'],
                    [b'yev', b'Yapunda'],
                    [b'yey', b'Yeyi'],
                    [b'yga', b'Malyangapa'],
                    [b'ygi', b'Yiningayi'],
                    [b'ygl', b'Yangum Gel'],
                    [b'ygm', b'Yagomi'],
                    [b'ygp', b'Gepo'],
                    [b'ygr', b'Yagaria'],
                    [b'ygu', b'Yugul'],
                    [b'ygw', b'Yagwoia'],
                    [b'yha', b'Buyang, Baha'],
                    [b'yhd', b'Arabic, Judeo-Iraqi'],
                    [b'yhl', b'Phowa, Hlepho'],
                    [b'yia', b'Yinggarda'],
                    [b'yid', b'Yiddish'],
                    [b'yif', b'Ache'],
                    [b'yig', b'Nasu, Wusa'],
                    [b'yih', b'Yiddish, Western'],
                    [b'yii', b'Yidiny'],
                    [b'yij', b'Yindjibarndi'],
                    [b'yik', b'Lalo, Dongshanba'],
                    [b'yil', b'Yindjilandji'],
                    [b'yim', b'Naga, Yimchungru'],
                    [b'yin', b'Yinchia'],
                    [b'yip', b'Pholo'],
                    [b'yiq', b'Miqie'],
                    [b'yir', b'Awyu, North'],
                    [b'yis', b'Yis'],
                    [b'yit', b'Lalu, Eastern'],
                    [b'yiu', b'Awu'],
                    [b'yiv', b'Nisu, Northern'],
                    [b'yix', b'Yi, Axi'],
                    [b'yiz', b'Azhe'],
                    [b'yka', b'Yakan'],
                    [b'ykg', b'Yukaghir, Northern'],
                    [b'yki', b'Yoke'],
                    [b'ykk', b'Yakaikeke'],
                    [b'ykl', b'Khlula'],
                    [b'ykm', b'Kap'],
                    [b'ykn', b'Kua-nsi'],
                    [b'yko', b'Yasa'],
                    [b'ykr', b'Yekora'],
                    [b'ykt', b'Kathu'],
                    [b'yku', b'Kuamasi'],
                    [b'yky', b'Yakoma'],
                    [b'yla', b'Yaul'],
                    [b'ylb', b'Yaleba'],
                    [b'yle', b'Yele'],
                    [b'ylg', b'Yelogu'],
                    [b'yli', b'Yali, Angguruk'],
                    [b'yll', b'Yil'],
                    [b'ylm', b'Limi'],
                    [b'yln', b'Buyang, Langnian'],
                    [b'ylo', b'Yi, Naluo'],
                    [b'ylr', b'Yalarnnga'],
                    [b'ylu', b'Aribwaung'],
                    [b'yly', 'Ny\xe2layu'],
                    [b'ymb', b'Yambes'],
                    [b'ymc', b'Muji, Southern'],
                    [b'ymd', b'Muda'],
                    [b'yme', b'Yameo'],
                    [b'ymg', b'Yamongeri'],
                    [b'ymh', b'Mili'],
                    [b'ymi', b'Moji'],
                    [b'ymk', b'Makwe'],
                    [b'yml', b'Iamalele'],
                    [b'ymm', b'Maay'],
                    [b'ymn', b'Yamna'],
                    [b'ymo', b'Yangum Mon'],
                    [b'ymp', b'Yamap'],
                    [b'ymq', b'Muji, Qila'],
                    [b'ymr', b'Malasar'],
                    [b'yms', b'Mysian'],
                    [b'ymt', b'Mator-Taygi-Karagas'],
                    [b'ymx', b'Muji, Northern'],
                    [b'ymz', b'Muzi'],
                    [b'yna', b'Aluo'],
                    [b'ynd', b'Yandruwandha'],
                    [b'yne', b"Lang'e"],
                    [b'yng', b'Yango'],
                    [b'ynh', b'Yangho'],
                    [b'ynk', b'Yupik, Naukan'],
                    [b'ynl', b'Yangulam'],
                    [b'ynn', b'Yana'],
                    [b'yno', b'Yong'],
                    [b'ynq', b'Yendang'],
                    [b'yns', b'Yansi'],
                    [b'ynu', b'Yahuna'],
                    [b'yob', b'Yoba'],
                    [b'yog', b'Yogad'],
                    [b'yoi', b'Yonaguni'],
                    [b'yok', b'Yokuts'],
                    [b'yol', b'Yola'],
                    [b'yom', b'Yombe'],
                    [b'yon', b'Yongkom'],
                    [b'yor', b'Yoruba'],
                    [b'yot', b'Yotti'],
                    [b'yox', b'Yoron'],
                    [b'yoy', b'Yoy'],
                    [b'ypa', b'Phala'],
                    [b'ypb', b'Phowa, Labo'],
                    [b'ypg', b'Phola'],
                    [b'yph', b'Phupha'],
                    [b'ypm', b'Phuma'],
                    [b'ypn', b'Phowa, Ani'],
                    [b'ypo', b'Phola, Alo'],
                    [b'ypp', b'Phupa'],
                    [b'ypz', b'Phuza'],
                    [b'yra', b'Yerakai'],
                    [b'yrb', b'Yareba'],
                    [b'yre', 'Yaour\xe9'],
                    [b'yri', 'Yar\xed'],
                    [b'yrk', b'Nenets'],
                    [b'yrl', b'Nhengatu'],
                    [b'yrm', b'Yirrk-Mel'],
                    [b'yrn', b'Yerong'],
                    [b'yrs', b'Yarsun'],
                    [b'yrw', b'Yarawata'],
                    [b'yry', b'Yarluyandi'],
                    [b'ysc', b'Yassic'],
                    [b'ysd', b'Samatao'],
                    [b'ysg', b'Sonaga'],
                    [b'ysl', b'Yugoslavian Sign Language'],
                    [b'ysn', b'Sani'],
                    [b'yso', b'Nisi (China)'],
                    [b'ysp', b'Lolopo, Southern'],
                    [b'ysr', b'Yupik, Sirenik'],
                    [b'yss', b'Yessan-Mayo'],
                    [b'ysy', b'Sanie'],
                    [b'yta', b'Talu'],
                    [b'ytl', b'Tanglang'],
                    [b'ytp', b'Thopho'],
                    [b'ytw', b'Yout Wam'],
                    [b'yty', b'Yatay'],
                    [b'yua', b'Yucateco'],
                    [b'yub', b'Yugambal'],
                    [b'yuc', b'Yuchi'],
                    [b'yud', b'Arabic, Judeo-Tripolitanian'],
                    [b'yue', b'Chinese, Yue'],
                    [b'yuf', b'Havasupai-Walapai-Yavapai'],
                    [b'yug', b'Yug'],
                    [b'yui', 'Yurut\xed'],
                    [b'yuj', b'Karkar-Yuri'],
                    [b'yuk', b'Yuki'],
                    [b'yul', b'Yulu'],
                    [b'yum', b'Quechan'],
                    [b'yun', b'Bena (Nigeria)'],
                    [b'yup', b'Yukpa'],
                    [b'yuq', b'Yuqui'],
                    [b'yur', b'Yurok'],
                    [b'yut', b'Yopno'],
                    [b'yuu', b'Yugh'],
                    [b'yuw', b'Yau (Morobe Province)'],
                    [b'yux', b'Yukaghir, Southern'],
                    [b'yuy', b'Yugur, East'],
                    [b'yuz', b'Yuracare'],
                    [b'yva', b'Yawa'],
                    [b'yvt', b'Yavitero'],
                    [b'ywa', b'Kalou'],
                    [b'ywg', b'Yinhawangka'],
                    [b'ywl', b'Lalu, Western'],
                    [b'ywn', b'Yawanawa'],
                    [b'ywq', b'Yi, Wuding-Luquan'],
                    [b'ywr', b'Yawuru'],
                    [b'ywt', b'Lalo, Xishanba'],
                    [b'ywu', b'Nasu, Wumeng'],
                    [b'yww', b'Yawarawarga'],
                    [b'yxa', b'Mayawali'],
                    [b'yxg', b'Yagara'],
                    [b'yxl', b'Yardliyawarra'],
                    [b'yxm', b'Yinwum'],
                    [b'yxu', b'Yuyu'],
                    [b'yxy', b'Yabula Yabula'],
                    [b'yyr', b'Yir Yoront'],
                    [b'yyu', b'Yau (Sandaun Province)'],
                    [b'yyz', b'Ayizi'],
                    [b'yzg', b"Buyang, E'ma"],
                    [b'yzk', b'Zokhuo'],
                    [b'zaa', 'Zapotec, Sierra de Ju\xe1rez'],
                    [b'zab', 'Zapotec, San Juan Guelav\xeda'],
                    [b'zac', 'Zapotec, Ocotl\xe1n'],
                    [b'zad', b'Zapotec, Cajonos'],
                    [b'zae', b'Zapotec, Yareni'],
                    [b'zaf', b'Zapotec, Ayoquesco'],
                    [b'zag', b'Zaghawa'],
                    [b'zah', b'Zangwal'],
                    [b'zai', b'Zapotec, Isthmus'],
                    [b'zaj', b'Zaramo'],
                    [b'zak', b'Zanaki'],
                    [b'zal', b'Zauzou'],
                    [b'zam', 'Zapotec, Miahuatl\xe1n'],
                    [b'zao', b'Zapotec, Ozolotepec'],
                    [b'zap', b'Zapotec'],
                    [b'zaq', 'Zapotec, Alo\xe1pam'],
                    [b'zar', 'Zapotec, Rinc\xf3n'],
                    [b'zas', b'Zapotec, Santo Domingo Albarradas'],
                    [b'zat', b'Zapotec, Tabaa'],
                    [b'zau', b'Zangskari'],
                    [b'zav', b'Zapotec, Yatzachi'],
                    [b'zaw', b'Zapotec, Mitla'],
                    [b'zax', b'Zapotec, Xadani'],
                    [b'zay', b'Zayse-Zergulla'],
                    [b'zaz', b'Zari'],
                    [b'zbc', b'Berawan, Central'],
                    [b'zbe', b'Berawan, East'],
                    [b'zbl', b'Blissymbols'],
                    [b'zbt', b'Batui'],
                    [b'zbw', b'Berawan, West'],
                    [b'zca', b'Zapotec, Coatecas Altas'],
                    [b'zch', b'Zhuang, Central Hongshuihe'],
                    [b'zdj', b'Comorian, Ngazidja'],
                    [b'zea', b'Zeeuws'],
                    [b'zeg', b'Zenag'],
                    [b'zeh', b'Zhuang, Eastern Hongshuihe'],
                    [b'zen', b'Zenaga'],
                    [b'zga', b'Kinga'],
                    [b'zgb', b'Zhuang, Guibei'],
                    [b'zgh', b'Tamazight, Standard Moroccan'],
                    [b'zgm', b'Zhuang, Minz'],
                    [b'zgn', b'Zhuang, Guibian'],
                    [b'zgr', b'Magori'],
                    [b'zha', b'Zhuang'],
                    [b'zhb', b'Zhaba'],
                    [b'zhd', b'Zhuang, Dai'],
                    [b'zhi', b'Zhire'],
                    [b'zhn', b'Zhuang, Nong'],
                    [b'zho', b'Chinese'],
                    [b'zhw', b'Zhoa'],
                    [b'zia', b'Zia'],
                    [b'zib', b'Zimbabwe Sign Language'],
                    [b'zik', b'Zimakani'],
                    [b'zil', b'Zialo'],
                    [b'zim', b'Mesme'],
                    [b'zin', b'Zinza'],
                    [b'zir', b'Ziriya'],
                    [b'ziw', b'Zigula'],
                    [b'ziz', b'Zizilivakan'],
                    [b'zka', b'Kaimbulawa'],
                    [b'zkb', b'Koibal'],
                    [b'zkd', b'Kadu'],
                    [b'zkg', b'Koguryo'],
                    [b'zkh', b'Khorezmian'],
                    [b'zkk', b'Karankawa'],
                    [b'zkn', b'Kanan'],
                    [b'zko', b'Kott'],
                    [b'zkp', 'Kaing\xe1ng, S\xe3o Paulo'],
                    [b'zkr', b'Zakhring'],
                    [b'zkt', b'Kitan'],
                    [b'zku', b'Kaurna'],
                    [b'zkv', b'Krevinian'],
                    [b'zkz', b'Khazar'],
                    [b'zlj', b'Zhuang, Liujiang'],
                    [b'zlm', b'Malay (individual language)'],
                    [b'zln', b'Zhuang, Lianshan'],
                    [b'zlq', b'Zhuang, Liuqian'],
                    [b'zma', b'Manda (Australia)'],
                    [b'zmb', b'Zimba'],
                    [b'zmc', b'Margany'],
                    [b'zmd', b'Maridan'],
                    [b'zme', b'Mangerr'],
                    [b'zmf', b'Mfinu'],
                    [b'zmg', b'Marti Ke'],
                    [b'zmh', b'Makolkol'],
                    [b'zmi', b'Negeri Sembilan Malay'],
                    [b'zmj', b'Maridjabin'],
                    [b'zmk', b'Mandandanyi'],
                    [b'zml', b'Madngele'],
                    [b'zmm', b'Marimanindji'],
                    [b'zmn', b'Mbangwe'],
                    [b'zmo', b'Molo'],
                    [b'zmp', b'Mpuono'],
                    [b'zmq', b'Mituku'],
                    [b'zmr', b'Maranunggu'],
                    [b'zms', b'Mbesa'],
                    [b'zmt', b'Maringarr'],
                    [b'zmu', b'Muruwari'],
                    [b'zmv', b'Mbariman-Gudhinma'],
                    [b'zmw', b'Mbo (Democratic Republic of Congo)'],
                    [b'zmx', b'Bomitaba'],
                    [b'zmy', b'Mariyedi'],
                    [b'zmz', b'Mbandja'],
                    [b'zna', b'Zan Gula'],
                    [b'zne', b'Zande (individual language)'],
                    [b'zng', b'Mang'],
                    [b'znk', b'Manangkari'],
                    [b'zns', b'Mangas'],
                    [b'zoc', 'Zoque, Copainal\xe1'],
                    [b'zoh', b'Zoque, Chimalapa'],
                    [b'zom', b'Zou'],
                    [b'zoo', 'Zapotec, Asunci\xf3n Mixtepec'],
                    [b'zoq', b'Zoque, Tabasco'],
                    [b'zor', 'Zoque, Ray\xf3n'],
                    [b'zos', 'Zoque, Francisco Le\xf3n'],
                    [b'zpa', b'Zapotec, Lachiguiri'],
                    [b'zpb', b'Zapotec, Yautepec'],
                    [b'zpc', b'Zapotec, Choapan'],
                    [b'zpd', 'Zapotec, Southeastern Ixtl\xe1n'],
                    [b'zpe', b'Zapotec, Petapa'],
                    [b'zpf', b'Zapotec, San Pedro Quiatoni'],
                    [b'zpg', b'Zapotec, Guevea De Humboldt'],
                    [b'zph', b'Zapotec, Totomachapan'],
                    [b'zpi', 'Zapotec, Santa Mar\xeda Quiegolani'],
                    [b'zpj', b'Zapotec, Quiavicuzas'],
                    [b'zpk', b'Zapotec, Tlacolulita'],
                    [b'zpl', 'Zapotec, Lachix\xedo'],
                    [b'zpm', b'Zapotec, Mixtepec'],
                    [b'zpn', 'Zapotec, Santa In\xe9s Yatzechi'],
                    [b'zpo', 'Zapotec, Amatl\xe1n'],
                    [b'zpp', b'Zapotec, El Alto'],
                    [b'zpq', b'Zapotec, Zoogocho'],
                    [b'zpr', b'Zapotec, Santiago Xanica'],
                    [b'zps', 'Zapotec, Coatl\xe1n'],
                    [b'zpt', 'Zapotec, San Vicente Coatl\xe1n'],
                    [b'zpu', 'Zapotec, Yal\xe1lag'],
                    [b'zpv', b'Zapotec, Chichicapan'],
                    [b'zpw', b'Zapotec, Zaniza'],
                    [b'zpx', b'Zapotec, San Baltazar Loxicha'],
                    [b'zpy', b'Zapotec, Mazaltepec'],
                    [b'zpz', b'Zapotec, Texmelucan'],
                    [b'zqe', b'Zhuang, Qiubei'],
                    [b'zra', b'Kara (Korea)'],
                    [b'zrg', b'Mirgan'],
                    [b'zrn', b'Zerenkel'],
                    [b'zro', 'Z\xe1paro'],
                    [b'zrp', b'Zarphatic'],
                    [b'zrs', b'Mairasi'],
                    [b'zsa', b'Sarasira'],
                    [b'zsk', b'Kaskean'],
                    [b'zsl', b'Zambian Sign Language'],
                    [b'zsm', b'Malay, Standard'],
                    [b'zsr', b'Zapotec, Southern Rincon'],
                    [b'zsu', b'Sukurum'],
                    [b'zte', b'Zapotec, Elotepec'],
                    [b'ztg', 'Zapotec, Xanagu\xeda'],
                    [b'ztl', 'Zapotec, Lapagu\xeda-Guivini'],
                    [b'ztm', 'Zapotec, San Agust\xedn Mixtepec'],
                    [b'ztn', b'Zapotec, Santa Catarina Albarradas'],
                    [b'ztp', b'Zapotec, Loxicha'],
                    [b'ztq', 'Zapotec, Quioquitani-Quier\xed'],
                    [b'zts', b'Zapotec, Tilquiapan'],
                    [b'ztt', b'Zapotec, Tejalapan'],
                    [b'ztu', 'Zapotec, G\xfcil\xe1'],
                    [b'ztx', b'Zapotec, Zaachila'],
                    [b'zty', b'Zapotec, Yatee'],
                    [b'zua', b'Zeem'],
                    [b'zuh', b'Tokano'],
                    [b'zul', b'Zulu'],
                    [b'zum', b'Kumzari'],
                    [b'zun', b'Zuni'],
                    [b'zuy', b'Zumaya'],
                    [b'zwa', b'Zay'],
                    [b'zxx', b'No linguistic content'],
                    [b'zyb', b'Zhuang, Yongbei'],
                    [b'zyg', b'Zhuang, Yang'],
                    [b'zyj', b'Zhuang, Youjiang'],
                    [b'zyn', b'Zhuang, Yongnan'],
                    [b'zyp', b'Chin, Zyphe'],
                    [b'zza', b'Zaza'],
                    [b'zzj', b'Zhuang, Zuojiang']
                ]
            )
        )
    ]
